#####################################################################################
## biome classification scheme described in online Supplement                      ##

rm(list=ls(all=TRUE))
library(fields)
library(raster)
library(maps)
library(RColorBrewer)
library(spatial)
library(maptools)

### filesystem details (paths) removed ###
basepath <- "***"
climate.scen.path <- "***"
lpj.input.path <- "***"
shape.path <- "***"

reference.path <- "***"
scenario.path  <- "***"
metric.path <- "***" 

reference.runtime <- c(1901, 2009)
reference.nyear <- reference.runtime[2]-reference.runtime[1]+1
scenario.runtime  <- c(2010, 2115)
scenario.nyear <- scenario.runtime[2]-scenario.runtime[1]+1

period.nyear <- 30 # length of reference and scenario periods

reference.period  <- c(reference.runtime[2]-period.nyear+1, reference.runtime[2])
reference.period2 <- c(1901,1930)
scenario.period   <- c(scenario.runtime[2]-period.nyear+1, scenario.runtime[2])

climate.runtime <- c(1901, 2115)

models <- c(
"cccma_cgcm3_1",    # 1
"cnrm_cm3",         # 2
"csiro_mk3_0",      # 3
"gfdl_cm2_0",       # 4
"gfdl_cm2_1",       # 5
"giss_model_e_h",   # 6
"giss_model_e_r",   # 7
"iap_fgoals",       # 8
"inmcm3_0",         # 9
"ipsl_cm4",         #10
"miroc3_2_hires",   #11
"miroc3_2_medres",  #12
"miub_echo_g",      #13
"mpi_echam5",       #14
"mri_cgcm2_3_2a",   #15
"ncar_ccsm3_0",     #16
"ncar_pcm1",        #17
"ukmo_hadcm3",      #18
"ukmo_hadgem1"      #19
)

model.names.official <- c(
"CGCM3.1(T47)",			# 1
"CNRM-CM3",					# 2
"CSIRO-Mk3.0",			# 3
"GFDL-CM2.0",				# 4
"GFDL-CM2.1",				# 5
"GISS-EH",					# 6
"GISS-ER",					# 7
"FGOALS-g1.0",			# 8
"INM-CM3.0",				# 9
"IPSL-CM4",					#10
"MIROC3.2(hires)",	#11
"MIROC3.2(medres)",	#12
"ECHO-G",						#13
"ECHAM5/MPI-OM",		#14
"MRI-CGCM2.3.2",		#15
"CCSM3",						#16
"PCM",							#17
"UKMO-HadCM3",			#18
"UKMO-HadGEM1"			#19
) # end official model names

target.temps <- c(
"1.5K",
"2.0K",
"2.5K",
"3.0K",
"3.5K",
"4.0K",
"4.5K",
"5.0K"
)

# filenames
grid.name <- "grid.out"
fpc.name <- "fpc.bin"
vegc.name <- "vegc.bin"

biome.class.names.lpj <- c("Tropical Rainforest",										# 1
													 "Tropical Seasonal & Deciduous Forest",	# 2
													 "Temperate Broadleaved Evergreen Forest",# 3
													 "Temperate Broadleaved Deciduous Forest",# 4
													 "Mixed Forest",													# 6
													 "Temperate Coniferous Forest",						# 5
													 "Boreal Evergreen Forest",								# 7
													 "Boreal Deciduous Forest",								# 8
													 "Warm Woody Savanna, Woodland & Shrubland",		# 9
													 "Warm Savanna & Open Shrubland",							#10
													 "Warm Grassland",															#11
													 "Temperate Woody Savanna, Woodland & Shrubland",		# 12
													 "Temperate Savanna & Open Shrubland",							#13
													 "Temperate Grassland",															#14
													 "Arctic Tundra",													#15
													 "Desert",																#16
													 "Rocks and Ice",
													 "Water"
													 )

biome.class.cols <- c(#rev(brewer.pal(9, "YlGn"))[1:8],
#											rev(brewer.pal(9, "GnBu"))[2:3], # tropical trees
#											rev(brewer.pal(9, "YlGn"))[2:5], # temperate trees
#											rev(brewer.pal(9, "RdGy"))[2:3], # boreal trees
											rev(brewer.pal(9, "YlGn"))[-9], # all trees
											rev(brewer.pal(9, "Oranges"))[4:6], # warm below forest
											rev(brewer.pal(9, "GnBu"))[2:5], # cold below forest
											"cornsilk4", # desert
#											rev(brewer.pal(9, "Oranges")[1:8]), # savanna/grasslands
											"white",
											"lightblue"
											) #


fpc.names <- c("natvegfrac", 														#1
							 "Tropical Broadleaved Evergreen Tree",		#2
							 "Tropical Broadleaved Raingreen Tree",		#3
							 "Temperate Needleleaved Evergreen Tree",	#4
							 "Temperate Broadleaved Evergreen Tree",	#5
							 "Temperate Broadleaved Summergreen Tree",#6
							 "Boreal Needleleaved Evergreen Tree",		#7
							 "Boreal Broadleaved Summergreen Tree", # actually Boreal Summergreen Tree (broadleaved and needleleaved)
							 "C3 Perennial Grass",										#9
							 "C4 Perennial Grass"											#10
							 )
fpc.temperate <- c(4,5,6)
fpc.tropical  <- c(2,3)
fpc.boreal    <- c(7,8)
fpc.needle    <- c(4,7,8)
fpc.grass     <- c(9,10)
fpc.trees     <- sort(unique(c(fpc.temperate, fpc.tropical, fpc.boreal, fpc.needle)))

lpj.res <- 0.5


# reference climate
tmp.name <- paste(climate.scen.path, "cruts3.1_", models[1], "_", target.temps[1], "_", climate.runtime[1], "-", climate.runtime[2], "_tmp.clm", sep="")
print(paste("Loading reference climate from", tmp.name))
tmp.file <- file(tmp.name, "rb")
seek(tmp.file, where=10, origin="start") # skip "LPJCLIMATE"
endian <- "little"
header <- readBin(tmp.file, what=integer(), size=4, n=7, endian=endian)
if(header[1] != 1) {
	endian <- "big"
	seek(tmp.file, where=10, origin="start")
	header <- readBin(tmp.file, what=integer(), size=4, n=7, endian=endian)
	if(header[1] != 1) {
		close(tmp.file)
		stop("Error: cannot determine endian of temperature file")
	}
}
NCELL.input <- header[6]
reference.offset <- (reference.period[1]-header[3])*12*NCELL.input*2+38
seek(tmp.file, where=reference.offset, origin="start")
reference.tmp <- readBin(tmp.file, what=integer(), n=12*NCELL.input*period.nyear, size=2, endian=endian)/10
dim(reference.tmp) <- c(12, NCELL.input, period.nyear)
dayspermonth <- c(31,28,31,30,31,30,31,31,30,31,30,31)
reference.atemp <- apply(reference.tmp*dayspermonth, c(2), sum, na.rm=TRUE)/period.nyear/sum(dayspermonth)
reference.tmin <- apply(reference.tmp, c(1,2), mean, na.rm=TRUE)
reference.tmin <- apply(reference.tmin, 2, min, na.rm=TRUE)
close(tmp.file)

# input grid
ingrid.name <- paste(climate.scen.path, "grid.bin", sep="")
#print(paste("Loading reference climate from", tmp.name))
grid.file <- file(ingrid.name, "rb")
seek(grid.file, where=7, origin="start") # skip "LPJGRID"
endian <- "little"
header <- readBin(grid.file, what=integer(), size=4, n=7, endian=endian)
if(header[1] != 1) {
	endian <- "big"
	seek(grid.file, where=7, origin="start")
	header <- readBin(grid.file, what=integer(), size=4, n=7, endian=endian)
	if(header[1] != 1) {
		close(tmp.file)
		stop("Error: cannot determine endian of input grid file")
	}
}
if(header[6] != NCELL.input) {
	close(grid.file)
	stop(paste("Error: input grid", grid.name," and reference temperature file", tmp.name, "do not match."))
}
ingrid.data <- readBin(grid.file, what=integer(), size=2, n=NCELL.input*2, endian=endian)/100
close(grid.file)
dim(ingrid.data) <- c(2,NCELL.input)

# soil code
soil.name <- paste(lpj.input.path, "soil.bin", sep="")
soil.data <- as.integer(readBin(soil.name, raw(), n=file.info(soil.name)$size))
if(length(soil.data) != NCELL.input) {
	stop("Error: Number of cells in climate data and soil data does not match")
}
copy.from.input <- which(soil.data>0)
NCELL <- length(copy.from.input)

reference.atemp <- reference.atemp[copy.from.input]

			
# grid
reference.grid.name <- paste(basepath,reference.path,grid.name, sep="")
print(paste("Loading output grid from", reference.grid.name))
grid.data <- array(readBin(reference.grid.name, what=integer(), size=2, n=NCELL*2), dim=c(2, NCELL))/100
earthradius <- 6371000.785
cellwidth <- earthradius*pi/180

grid.idata <- array(,dim=dim(grid.data))
grid.idata[1,] <- (grid.data[1,]+180.25)/0.5
grid.idata[2,] <- (grid.data[2,]+90.25)/0.5

grid.area <- (cellwidth*0.5)*(cellwidth*0.5)*cos(grid.data[2,]/180*pi)

# FPC
reference.fpcname <- paste(basepath, reference.path, fpc.name, sep="")
print(paste("Loading reference PFT composition from", reference.fpcname))
reference.fpcfile <- file(reference.fpcname, "rb")
reference.offset <- (reference.period[1]-reference.runtime[1])*length(fpc.names)*NCELL*4
seek(reference.fpcfile, where=reference.offset, origin="start")
reference.fpc <- readBin(reference.fpcfile, what=numeric(), size=4, n=length(fpc.names)*NCELL*period.nyear, endian="little")
dim(reference.fpc) <- c(NCELL, length(fpc.names), period.nyear)
close(reference.fpcfile)
reference.fpc.sd <- apply(reference.fpc, c(1,2), sd, na.rm=TRUE) # standard deviation over reference period
reference.fpc <- apply(reference.fpc, c(1,2), mean, na.rm=TRUE) # mean over reference period

reference.vegcname <- paste(basepath, reference.path, vegc.name, sep="")
print(paste("Loading reference vegc from", reference.vegcname))
reference.vegcfile <- file(reference.vegcname, "rb")
reference.offset <- (reference.period[1]-reference.runtime[1])*NCELL*4
seek(reference.vegcfile, where=reference.offset, origin="start")
reference.vegc <- readBin(reference.vegcfile, what=numeric(), size=4, n=NCELL*period.nyear, endian="little")
dim(reference.vegc) <- c(NCELL, period.nyear)
close(reference.vegcfile)
reference.vegc <- apply(reference.vegc, 1, mean, na.rm=TRUE) # mean over reference period

biome.classification <- function(in.fpc, in.atemp, in.vegc, use.vegc=TRUE) {
	if(dim(in.fpc)[1] != length(in.atemp)) {
		stop("Error: in.fpc and in.atemp do not match")
	}
	if(dim(in.fpc)[2] != length(fpc.names)) {
		stop("Error: in.fpc has wrong number of bands")
	}
	if(length(dim(in.fpc)) != 2) {
		stop("Error: in.fpc must be array of dimensions (pft, cell)")
	}
	fpc.tree.total     <- apply(in.fpc[,fpc.trees], 1, sum, na.rm=TRUE)
	fpc.tree.temperate <- apply(in.fpc[,fpc.temperate], 1, sum, na.rm=TRUE)
	fpc.tree.tropical  <- apply(in.fpc[,fpc.tropical], 1, sum, na.rm=TRUE)
	fpc.tree.boreal    <- apply(in.fpc[,fpc.boreal], 1, sum, na.rm=TRUE)
	fpc.tree.needle    <- apply(in.fpc[,fpc.needle], 1, sum, na.rm=TRUE)
	fpc.grass.total    <- apply(in.fpc[,fpc.grass], 1, sum, na.rm=TRUE)
	fpc.total          <- apply(in.fpc[,-1], 1, sum, na.rm=TRUE)
	max.share          <- apply(in.fpc[,-1], 1, max, na.rm=TRUE)
	fpc.tree.broadleaf <- fpc.tree.total-fpc.tree.needle
	
	biome.class <- integer(length(in.atemp))
	biome.class[] <- NA
	
	if(!use.vegc) {
	  in.vegc[] <- 7500 # vegc test always positive
	}
	
	biome.class <- ifelse(fpc.total <= 0.05,
												ifelse(in.atemp < (-2), which(biome.class.names.lpj == "Arctic Tundra"), which(biome.class.names.lpj == "Desert")),
												ifelse(fpc.tree.total >= 0.6,
															 ifelse(in.fpc[,7]==max.share & (fpc.tree.broadleaf < (0.4*fpc.tree.total)),
																			which(biome.class.names.lpj=="Boreal Evergreen Forest"),
																			ifelse(in.fpc[,8]==max.share & (fpc.tree.broadleaf < (0.4*fpc.tree.total)),
																						 which(biome.class.names.lpj=="Boreal Deciduous Forest"),
																						 ifelse((in.fpc[,4]==max.share) & (fpc.tree.broadleaf < (0.4*fpc.tree.total)),# (in.fpc[,4]> (in.fpc[,5]+in.fpc[,6])),
																										which(biome.class.names.lpj=="Temperate Coniferous Forest"),
																										ifelse((fpc.tree.tropical<0.4*fpc.tree.total) & (fpc.tree.needle < (0.4*fpc.tree.total)), #in.fpc[,5]+in.fpc[,6]>=0.6,
																													 ifelse(in.fpc[,5]>in.fpc[,6],
																																	which(biome.class.names.lpj=="Temperate Broadleaved Evergreen Forest"),
																																	which(biome.class.names.lpj=="Temperate Broadleaved Deciduous Forest")),
																													 ifelse((fpc.tree.boreal+fpc.tree.temperate)<(0.4*fpc.tree.total),
																																	ifelse(in.fpc[,2]>in.fpc[,3] & in.vegc >= 7500,
																																				 which(biome.class.names.lpj=="Tropical Rainforest"),
																																				 ifelse(in.vegc >= 7500,
																																								which(biome.class.names.lpj=="Tropical Seasonal & Deciduous Forest"),
																																								which(biome.class.names.lpj=="Warm Woody Savanna, Woodland & Shrubland")
																																								)
																																				), # end tropical forests/savanna
																																	which(biome.class.names.lpj=="Mixed Forest")
																																	) # end tropical or mixed
																													)# end Temperate Broadleaved
																										)# end Temperate
																						)# end Boreal Deciduous
																			), # end Boreal Evergreen and end trees
															 ifelse(fpc.tree.total>=(0.2) & in.atemp >= (-2),# & fpc.tree.boreal==0,
																			ifelse(in.fpc[,9] > in.fpc[,10],
																						 which(biome.class.names.lpj=="Temperate Woody Savanna, Woodland & Shrubland"),
																						 which(biome.class.names.lpj=="Warm Woody Savanna, Woodland & Shrubland")
																						 ),
																			ifelse(fpc.tree.total>=(0.05) & in.atemp >= (-2),# & fpc.tree.boreal==0,
																						 ifelse(in.fpc[,9]>in.fpc[,10],
																										which(biome.class.names.lpj=="Temperate Savanna & Open Shrubland"),
																										which(biome.class.names.lpj=="Warm Savanna & Open Shrubland")
																										),
																						 ifelse(in.atemp >= (-2),
																										ifelse(in.fpc[,9]>in.fpc[,10],
																													 which(biome.class.names.lpj=="Temperate Grassland"),
																													 which(biome.class.names.lpj=="Warm Grassland")
																													 ),
																										which(biome.class.names.lpj=="Arctic Tundra")
																										) # end grasslands
																						) # end dry savanna
																		) # end moist savanna
															) # end forest or savanna
													) # end all ifelse
															 

	return(biome.class)
}
print("Computing biome classes for reference period")

### LPJmL-based biome classification
reference.class <- biome.classification(reference.fpc, reference.atemp, reference.vegc)
writeBin(as.raw(reference.class), paste(basepath, reference.path, "biome_class.bin", sep=""))

### MODIS-based biome classification
modis.fpc <- readBin(paste(basepath, "analysis/modis.fpc_mar2013.bin", sep=""), what=numeric(), size=4, n=NCELL.input*length(fpc.names))
dim(modis.fpc) <- c(NCELL.input, length(fpc.names))
modis.fpc <- modis.fpc[copy.from.input,]
modis.class <- biome.classification(modis.fpc, reference.atemp, reference.vegc, FALSE) # does not use vegc limit for tropical forests
writeBin(as.raw(modis.class), paste(basepath,"analysis/modis.biomeclass.bin", sep=""))

