#####################################################################################
## examples of biome transformations
## various calculations for biome averages
rm(list=ls(all=TRUE))
library(fields)
library(maps)
library(spatial)
library(maptools)
library(raster)
library(RColorBrewer)
library(Hmisc)
library(shape)

### filesystem details (paths) removed ###
basepath <- "***"
climate.scen.path <- "***"
lpj.input.path <- "***"
shape.path <- "***"
rcp.path <- "***"
pattern.trajectory.path <- c("***")
general.path <- "***"
cluster <- FALSE

reference.path <- "***"
scenario.path  <- "***"
metric.path <- "***"
metric.path2 <- "***"
metric.path.sd.in <- "***"
metric.path.sd.out <- "***"


reference.runtime <- c(1901, 2009)
reference.nyear <- reference.runtime[2]-reference.runtime[1]+1
scenario.runtime  <- c(2010, 2115)
scenario.nyear <- scenario.runtime[2]-scenario.runtime[1]+1

period.nyear <- 30 #109 #30 # length of reference and scenario periods

reference.period  <- c(reference.runtime[2]-period.nyear+1, reference.runtime[2])
reference.period2 <- c(1901,1930)
scenario.period   <- c(scenario.runtime[2]-period.nyear+1, scenario.runtime[2])

landuse.ref <- 2000

output.vars <- paste(basepath,
										 reference.path,
										 c("evapinterc.bin","firec.bin", "firef.bin", "fpc.bin", "npp.bin", "rh.bin", "runoff.bin", "soillitc.bin", "swc1.bin", "transp.bin", "vegc.bin"),
										 sep="")

gamma.vars <- list("combined" = "cm_scaled.bin",
									 "deltav"="deltav.bin",
									 "cfluxes"="cfluxes_comb_scaled.bin",
									 "cstocks"="cstocks_comb_scaled.bin",
									 "wfluxes"="wfluxes_comb_scaled.bin",
									 "global"="all_mag_g_scaled.bin",
									 "local"="all_mag_scaled.bin",
									 "balance"="all_1-alpha.bin"
									 )
									 
gamma.sd.vars <- list("all" = c("all_alpha_ts_sd", "all_alpha_ts_g_sd", "all_mag_ts_sd", "all_mag_ts_g_sd"),
                      "cfluxes" = c("cfluxes_alpha_ts_sd", "cfluxes_alpha_ts_g_sd", "cfluxes_mag_ts_sd", "cfluxes_mag_ts_g_sd"),
                      "cstocks" = c("cstocks_alpha_ts_sd", "cstocks_alpha_ts_g_sd", "cstocks_mag_ts_sd", "cstocks_mag_ts_g_sd"),
                      "wfluxes" = c("wfluxes_alpha_ts_sd", "wfluxes_alpha_ts_g_sd", "wfluxes_mag_ts_sd", "wfluxes_mag_ts_g_sd")
                      )

biome.class.filename <- paste(basepath, reference.path, "biome_class.bin", sep="")
NCELL <- file.info(biome.class.filename)$size
biome.lpj <- as.integer(readBin(biome.class.filename, what=raw(), n=NCELL))

print("Load and simplify world borders")
world.borders <- readShapePoly(paste(shape.path, "TM_WORLD_BORDERS-0.3", sep=""))
world.borders.simple <- thinnedSpatialPoly(world.borders, tolerance=0.25, minarea=0.01, topologyPreserve=TRUE)

biome.class.names.lpj <- c(
"Tropical Rainforest",										# 1
"Tropical Seasonal & Deciduous Forest",	# 2
"Temperate Broadleaved Evergreen Forest",# 3
"Temperate Broadleaved Deciduous Forest",# 4
"Mixed Forest",													# 6
"Temperate Coniferous Forest",						# 5
"Boreal Evergreen Forest",								# 7
"Boreal Deciduous Forest",								# 8
"Warm Woody Savanna, Woodland & Shrubland",		# 9
"Warm Savanna & Open Shrubland",							#10
"Warm Grassland",															#11
"Temperate Woody Savanna, Woodland & Shrubland",		# 12 "Temperate" instead of "Cool"
"Temperate Savanna & Open Shrubland",							#13
"Temperate Grassland",															#14
"Arctic Tundra",													#15
"Desert",																#16
"Rocks and Ice",												#17
"Water"																	#18
)
plot.classes <- c(1:16)

biome.class.cols <- c(
	rev(brewer.pal(9, "YlGn"))[-9], # all trees
	rev(brewer.pal(9, "Oranges"))[4:6], # warm below forest
	rev(brewer.pal(9, "GnBu"))[2:5], # cold below forest
	"cornsilk4", # desert
	"white",
	"grey90"
) #

grid.data <- readBin(paste(basepath, reference.path, "grid.out", sep=""), integer(), size=2, n=NCELL*2)/100
dim(grid.data) <- c(2, NCELL)
earthradius <- 6371000.785
cellwidth <- earthradius*pi/180
grid.area <- cos(grid.data[2,]/180*pi)#(cellwidth*0.5)*(cellwidth*0.5)*cos(grid.data[2,]/180*pi)

soil.name <- paste(lpj.input.path, "soil.bin", sep="")
landuse.name <- paste(gsub("cru06", "cru09",lpj.input.path), "cft1700_2005_bioenergy_sc.bin", sep="")

soil.data <- as.integer(readBin(soil.name, raw(), n=file.info(soil.name)$size))
NCELL.input <- length(soil.data)
endian="little"
landuse.file <- file(landuse.name, "rb")
seek(landuse.file, where=10, origin="start")
header <- readBin(landuse.file, what=integer(), size=4, n=7, endian=endian)
if(header[1] != 1) {
	endian <- "big"
	seek(landuse.file, where=10, origin="start")
	header <- readBin(landuse.file, what=integer(), size=4, n=7, endian=endian)
	if(header[1] != 1) stop("Error: cannot determine endian of landuse file")
}
if(header[6] != NCELL.input) {
	close(landuse.file)
	stop("Error: landuse data and input grid do not match")
}
landuse.nbands <- header[7]
landuse.startyear <- header[3]
landuse.nyear <- header[4]
	
landuse.offset <- landuse.ref-landuse.startyear #reference.runtime[2]-landuse.startyear
if(landuse.offset > (landuse.nyear-1)) {
	landuse.offset <- landuse.nyear-1 # use last year of reference period or last year of file if file ends earlier
}
print(paste("Loading landuse data for year", landuse.startyear+landuse.offset, "from file", landuse.name))
seek(landuse.file, where=landuse.offset*landuse.nbands*header[6]*2+38, origin="start")
landuse.data <- readBin(landuse.file, what=integer(), size=2, n=NCELL.input*landuse.nbands, endian=endian)
dim(landuse.data) <- c(landuse.nbands, NCELL.input)
# sum up over all cft's
landuse.data <- apply(landuse.data, 2, sum, na.rm=TRUE)
if(max(landuse.data > 1000)) {
	print("Warning: there are cells with landuse fraction > 100% in landuse input")
}
close(landuse.file)
# use only cells that are in output
landuse.data <- landuse.data[which(soil.data > 0)]
natveg.frac <- (1000-landuse.data)/1000
natveg.frac[which(natveg.frac <= 0)] <- 0
#natveg.frac[] <- 1

create.biome.means <- FALSE
write.mean.maps <- TRUE
if(create.biome.means) {
	for(var in output.vars) {
		print(paste("*************\n", var, "\n***************", sep=""))
		zz <- file(var, "rb")
		nbands <- file.info(var)$size/NCELL/(reference.runtime[2]-reference.runtime[1]+1)/4
		seek(zz, where=(reference.period[1]-reference.runtime[1])*NCELL*nbands*4, origin="start")
		filedata <- readBin(zz, double(), size=4, n=NCELL*nbands*period.nyear)
		dim(filedata) <- c(NCELL, nbands, period.nyear)
		#print(paste("Dimensions input: (", toString(dim(filedata)), ")", sep=""))
		close(zz)
		if(nbands>1) {
			preserve.dims <- c(2,3)
		} else {
			preserve.dims <- c(2)
		}
		if(nbands==1 && 1==2) {
			global.mean <- apply(filedata*grid.area*natveg.frac, c(2,3), sum)/sum(grid.area*natveg.frac)
			global.mean.sd <- sqrt(apply((filedata-rep(global.mean, each=NCELL*nbands))^2*grid.area*natveg.frac, c(2,3), sum)/sum(grid.area*natveg.frac))
			output.filename <- basename(var)
			global.mean <- mean(global.mean)
			global.mean.sd <- mean(global.mean.sd)
			print(toString(c(global.mean, global.mean.sd)))
			output.filename <- gsub(".bin", "", output.filename, fixed=TRUE)
			mean.filename <- paste(basepath, reference.path, output.filename, "_globalmean_", reference.period[1], "-", reference.period[2], ".txt", sep="")
			std.filename <- paste(basepath, reference.path, output.filename, "_globalstd_", reference.period[1], "-", reference.period[2], ".txt", sep="")
			cat(global.mean, file=mean.filename, sep="")
			cat(global.mean.sd, file=std.filename, sep="")
		}
		#next
		outputdata <- array(dim=c(length(biome.class.names.lpj[plot.classes]), nbands, period.nyear))
		if(write.mean.maps) {
		  mean.maps.data <- array(dim=c(NCELL, nbands, period.nyear))
		}
		for(b in 1:length(biome.class.names.lpj[plot.classes])) {
			biome.cells <- which(biome.lpj==plot.classes[b])
			biomedata <- apply(filedata[biome.cells,,]*(grid.area*natveg.frac)[biome.cells], preserve.dims, sum)/sum((grid.area*natveg.frac)[biome.cells])
			outputdata[b,,] <- biomedata
		  if(write.mean.maps) {
		    mean.maps.data[biome.cells,,] <- rep(biomedata, each=length(biome.cells))
		  }
		}
		if(write.mean.maps) {
			output.filename <- paste(dirname(var), "/biome_means/", basename(var), sep="")
			if(!file.exists(dirname(output.filename))) {
				dir.create(dirname(output.filename))
			}
			print(paste("Output filename:", output.filename))
			#print(paste("Dimensions output: (", toString(dim(outputdata)), ")", sep=""))
			writeBin(as.double(mean.maps.data), output.filename, size=4)
		}
		next
		for(b in 1:length(biome.class.names.lpj[plot.classes])) {
			output.filename <- paste(dirname(var), "/biome", b, "/", basename(var), sep="")
			if(!file.exists(dirname(output.filename))) {
				dir.create(dirname(output.filename))
			}
			print(paste("Output filename:", output.filename))
			#print(paste("Dimensions output: (", toString(dim(outputdata)), ")", sep=""))
			writeBin(as.double(outputdata), output.filename, size=4)
			# shuffle biomes
			tmp <- outputdata[1,,]
			outputdata[1:(length(biome.class.names.lpj[plot.classes])-1),,] <- outputdata[-1,,]
			outputdata[length(biome.class.names.lpj[plot.classes]),,] <- tmp
		}
	}
}

create.biome.variability <- FALSE
if(create.biome.variability) {
  gamma.sd <- list()
  gamma.biome.sd <- list()
  for(comp in names(gamma.sd.vars)) {
    #gamma.sd[[comp]] <- array(dim=c(NCELL, length(gamma.sd.vars[[comp]])))
    filedata <- array(dim=c(NCELL, length(gamma.sd.vars[[comp]])))
    biomedata <- array(dim=c(length(plot.classes), length(gamma.sd.vars[[comp]])))
    outfilename <- character();
    for(i in 1:length(gamma.sd.vars[[comp]])) {
      filename <- paste(basepath, scenario.path, "cccma_cgcm3_1/1.5K/", metric.path.sd.in, gamma.sd.vars[[comp]][i], "_", reference.period[1], "-", reference.period[2], ".bin", sep="")
      outfilename <- c(outfilename, basename(filename))
      print(paste("Reading from", filename))
      filedata[,i] <- readBin(filename, double(), size=4, n=NCELL)
    }
    print("Averaging over biomes")
    for(b in plot.classes) {
      biome.cells <- which(biome.lpj==b)
      biomedata[b,] <- apply(filedata[biome.cells,]*grid.area[biome.cells]*natveg.frac[biome.cells], 2, sum)/sum(grid.area[biome.cells]*natveg.frac[biome.cells])
    }
    outpath <- paste(basepath, reference.path, "biome", plot.classes, "/", metric.path.sd.out, sep="")
    print(paste("Writing to", outpath[1], "..."))
    for(i in 1:length(outpath)) {
      if(!file.exists(outpath[i])) dir.create(outpath[i], recursive=TRUE)
      for(j in 1:length(outfilename))
        writeBin(as.double(biomedata[,j]), paste(outpath[i],outfilename[j], sep=""), size=4)
    }
  }
}


components <- list()
plot.gamma.table <- TRUE#FALSE
if(plot.gamma.table) {
	for(metric in names(gamma.vars)[]) {
	  components[[metric]] <- list()
		biome.gamma <- array(dim=c(length(plot.classes), length(plot.classes)))
		biome.ratio <- array(dim=c(length(plot.classes), length(plot.classes)))
		dimnames(biome.gamma)[[2]] <- biome.class.names.lpj[plot.classes]
		dimnames(biome.ratio)[[2]] <- biome.class.names.lpj[plot.classes]
		cell.names <- biome.class.names.lpj[plot.classes]
		for(b in 1:length(plot.classes)) {
			input.filename <- paste(basepath, reference.path, "biome", plot.classes[b], "/", metric.path.sd.out, gamma.vars[[metric]], sep="")
			print(paste("Loading data from", input.filename))
			filedata <- readBin(input.filename, double(), size=4, n=ifelse(file.info(input.filename)$size/4>length(plot.classes), length(plot.classes)*2, length(plot.classes)))
			if(length(filedata)>length(plot.classes)) {
				biome.gamma[b,] <- filedata[1:length(plot.classes)+length(plot.classes)]
			} else {
				biome.gamma[b,] <- filedata
			}
			cell.names <- c(cell.names[-1], cell.names[1])
			names(biome.gamma[b,]) <- paste(biome.class.names.lpj[plot.classes],"->", cell.names, sep="")
			if(metric %in% c("local", "global", "balance")) {
				if(metric=="balance") {
					ratio.filename <- gsub("1-alpha", "alpha_logistic", input.filename, fixed=TRUE)
					biome.gamma[b,] <- biome.gamma[b,]*2
					biome.gamma[which(biome.gamma>1)] <- 1
				} else {
					ratio.filename <- gsub("_scaled", "_logistic", input.filename, fixed=TRUE)
				}
				print(paste("Loading data from", ratio.filename))
				filedata <- readBin(ratio.filename, double(), size=4, n=length(plot.classes))
				#biome.gamma[b,] <- biome.gamma[b,]*filedata
				biome.ratio[b,] <- filedata
			} else {
			  biome.ratio[b,] <- 1
			}
		}

		biome.table <- array(dim=dim(biome.gamma))
		biome.table[,1] <- biome.gamma[,1]
		ratio.table <- array(dim=dim(biome.ratio))
		ratio.table[,1] <- biome.ratio[,1]
		for(b in 2:length(plot.classes)) {
			shift <- sort(seq(length(plot.classes), by=-1, length.out=b-1))
			biome.table[,b] <- c(biome.gamma[shift,b], biome.gamma[-shift,b])
			ratio.table[,b] <- c(biome.ratio[shift,b], biome.ratio[-shift,b])
		}
		
		components[[metric]][["values"]] <- biome.table
		components[[metric]][["ratio"]] <- ratio.table
		#next
		biome.table <- biome.table*ratio.table
		#pdf(paste(basepath, "analysis/biome_gamma_table_", metric, "_jan2013_shaded2.pdf", sep=""), height=5, width=6, pointsize=8)
		setEPS()
		postscript(paste(basepath, "analysis/biome_gamma_table_", metric, "_aug2013_sdnew.eps", sep=""), height=4, width=5, pointsize=6, paper="special")
		par(mar=c(1-0.9,9+1,8,1-0.5)+0.1)
		plot(1,1, type="n", xlim=c(0.5, length(plot.classes)+0.5), ylim=c(0.5, length(plot.classes)+0.5), axes=FALSE, xlab="", ylab="", xaxs="i", yaxs="i")
		axis(2, at=rev(seq(1, length(plot.classes))), labels=FALSE)#unlist(lapply(strwrap(biome.class.names.lpj[plot.classes], 25, simplify = F), paste, sep="", collapse="\n")), las=1, cex.axis=0.75)
		text(x=0.25, y=rev(seq(1, length(plot.classes))), labels=unlist(lapply(strwrap(biome.class.names.lpj[plot.classes], 25, simplify = F), paste, sep="", collapse="\n")), cex=0.75, xpd=NA, pos=2, srt=360)
		axis(3, at=seq(1, length(plot.classes)), labels=FALSE)
		text(x=seq(1, length(plot.classes)), y=rep(length(plot.classes)+1, length(plot.classes)), unlist(lapply(strwrap(biome.class.names.lpj[plot.classes], 25, simplify = F), paste, sep="", collapse="\n")), cex=0.75, srt=45, xpd=NA, adj=0)
		box()
		for(b in 1:length(plot.classes)) {
		  #brewer.pal(9, "YlOrRd")[3], brewer.pal(9, "Blues")[3]
  		cols <- ifelse(biome.table[,b]>0.3, rep("grey60", length(plot.classes)), ifelse(biome.table[,b]>0.1, rep("grey80", length(plot.classes)), rep(NA, length(plot.classes))))
  		cols <- paste("grey", round(100-biome.table[,b]*100), sep="")
  		text.cols <- ifelse(biome.table[,b]>0.6, rep("white", length(plot.classes)), rep("black", length(plot.classes)))
			rect(xleft=b-0.5, ybottom=seq(length(plot.classes), 1, by=-1)-0.5, xright=b+0.5, ytop=seq(length(plot.classes), 1, by=-1)+0.5, col=cols, border=NA)
			text(x=b, y=seq(length(plot.classes), 1, by=-1), labels=round(biome.table[,b],2), cex=0.75, col=text.cols)
		}
		abline(h=seq(1.5, length(plot.classes)), v=seq(1.5, length(plot.classes)), lwd=0.5)
		title(main="Shift from", line=7, cex.main=1)
		title(ylab="Shift to", line=8+1, cex.lab=1)
		metric.title <- switch(metric, 
													 combined=expression(paste("Combined ", Gamma, " metric")),
													 deltav=expression(paste(Delta,"V component")),
													 cfluxes="Carbon fluxes subset",
													 cstocks="Carbon stocks subset",
													 wfluxes="Water fluxes subset",
													 global="Global importance component",
													 local="Local change component",
													 balance="Balance component"
													 )

		mtext(text=metric.title, side=3, line=6, at=-3.5, adj=0, cex=1.5)
		#lines(x=c(rep(length(plot.classes)+1,2),2/3*length(plot.classes)), y=c(1/3*length(plot.classes),0,0),lwd=5, xpd=NA)
		#arrows(length(plot.classes),0,2/3*length(plot.classes),0, xpd=NA, arr.type="triangle", arr.length=0.1, arr.lwd=5, lwd=5, length=0.1)
		dev.off()
	}
}
