#! /usr/bin/csh -f

# change_metric.csh
# computes Gamma between future state and reference state

# filesystem details (paths) removed ###
set reference_dir = PATH_TO_REFERENCE_RUN
set scenario_dir = PATH_TO_SCENARIO_RUNS

set prog_dir = PATH_TO_EXECUTABLES
set grid = $reference_dir/grid.out
set deltav_bin = deltav_oct2012
set metric_bin = change_metric_may2012
set natvegfrac = $reference_dir/natvegfrac.bin

set models = (cccma_cgcm3_1 cnrm_cm3 csiro_mk3_0 gfdl_cm2_0 gfdl_cm2_1 giss_model_e_h giss_model_e_r iap_fgoals inmcm3_0 ipsl_cm4 miroc3_2_hires miroc3_2_medres miub_echo_g mpi_echam5 mri_cgcm2_3_2a ncar_ccsm3_0 ncar_pcm1 ukmo_hadcm3 ukmo_hadgem1)

set scenarios = (1.5K 2.0K 2.5K 3.0K 3.5K 4.0K 4.5K 5.0K)

set cffiles = ( firec rh npp )
set wffiles = (evapinterc runoff transp)
set csfiles = (soillitc vegc)
set allfiles = ($csfiles $wffiles $cffiles swc1 firef)

# Reference Run
set startyear1 = 1980 # reference period
set endyear1 = 2009
set refstartyear = 1901
set refendyear = 2009
set nyear1 = 109
echo Reference period: $startyear1 - $endyear1

# Scenario run
set startyear2 = 2086 # period to calculate change
set endyear2 = 2115 
set scenstartyear = 2010
set scenendyear = 2115
set nyear2 = 106
echo Scenario period: $startyear2 - $endyear2

set npixel = 59199
set snr_logbase = 3
set mag_maxgrowth = 0.5
set mag_growthrate = 6
set a_mag = `echo "-1 / e ("$mag_maxgrowth"*"$mag_growthrate")" | bc -l`
set mag_g_maxgrowth = 0.5
set mag_g_growthrate = 6
set a_mag_g = `echo "-1 / e ("$mag_g_maxgrowth"*"$mag_g_growthrate")" | bc -l`
echo a_mag $a_mag a_mag_g $a_mag_g
echo fpcthresh $fpcthresh

set year1 = `echo $startyear1 - $refstartyear | bc`
set year2 = `echo $endyear1 - $refstartyear | bc `
set year3 = `echo $startyear2 - $scenstartyear | bc`
set year4 = `echo $endyear2 - $scenstartyear | bc `

foreach model ( $models )
	foreach scenario ( $scenarios )
		echo $model $scenario
		set dir = $scenario_dir/$model/$scenario
		set odir = $dir/PATH_FOR_GAMMA_METRIC
		if ( { test -s $dir/grid.out } ) then
			echo $dir
			mkdir $odir
			if ( ! { test -e $dir/soillitc.bin } ) then
				echo sum files
				$prog_dir/filesum_ff $dir/soilc.bin $dir/litc.bin $dir/soillitc.bin $npixel
      endif
          
      if ( ! { test -e $odir/deltav.bin } ) then
				echo deltav
				$prog_dir/$deltav_bin $reference_dir/fpc.bin $dir/fpc.bin $odir/deltav.bin 30 30 $npixel $year1 $year3
			endif
			#time series of deltav
			if ( ! { test -e $odir/deltav_ts.bin } ) then
				echo deltav timeseries
				set tmpyear = `echo $refstartyear - $scenstartyear | bc`
				@ y = $tmpyear
				while ($y <= $year4)
					$prog_dir/$deltav_bin $reference_dir/fpc.bin $dir/fpc.bin $odir/tmp 30 1 $npixel $year1 $y
					if ($y == $tmpyear)  then
						cp $odir/tmp $odir/deltav_ts.bin
					else
						cat $odir/deltav_ts.bin $odir/tmp > $odir/tmp2
						mv $odir/tmp2 $odir/deltav_ts.bin
					endif
					@ y++
				end
				rm $odir/tmp
			endif # deltav timeseries
			
      echo change metrics
      #carbon stocks
      echo cstocks
      $prog_dir/$metric_bin $#csfiles 0 1 $natvegfrac $nyear1 $nyear2 30 $year1 $year3 $grid 0.5 1 $reference_dir/$csfiles[1].bin $dir/$csfiles[1].bin 1 $reference_dir/$csfiles[2].bin $dir/$csfiles[2].bin $npixel $odir/cstocks_alpha.bin $odir/cstocks_mag.bin $odir/cstocks_alpha_ts.bin $odir/cstocks_mag_ts.bin $odir/cstocks_alpha_g.bin $odir/cstocks_mag_g.bin $odir/cstocks_alpha_ts_g.bin $odir/cstocks_mag_ts_g.bin
         
      #carbon fluxes
      echo cfluxes
      $prog_dir/$metric_bin $#cffiles 0 1 $natvegfrac $nyear1 $nyear2 30 $year1 $year3 $grid 0.5 1 $reference_dir/$cffiles[1].bin $dir/$cffiles[1].bin 1 $reference_dir/$cffiles[2].bin $dir/$cffiles[2].bin 1 $reference_dir/$cffiles[3].bin $dir/$cffiles[3].bin $npixel $odir/cfluxes_alpha.bin $odir/cfluxes_mag.bin $odir/cfluxes_alpha_ts.bin $odir/cfluxes_mag_ts.bin $odir/cfluxes_alpha_g.bin $odir/cfluxes_mag_g.bin $odir/cfluxes_alpha_ts_g.bin $odir/cfluxes_mag_ts_g.bin
	
			#water fluxes
			echo echo wfluxes
			$prog_dir/$metric_bin $#wffiles 0 1 $natvegfrac $nyear1 $nyear2 30 $year1 $year3 $grid 0.5 1 $reference_dir/$wffiles[1].bin $dir/$wffiles[1].bin 1 $reference_dir/$wffiles[2].bin $dir/$wffiles[2].bin 1 $reference_dir/$wffiles[3].bin $dir/$wffiles[3].bin $npixel $odir/wfluxes_alpha.bin $odir/wfluxes_mag.bin $odir/wfluxes_alpha_ts.bin $odir/wfluxes_mag_ts.bin $odir/wfluxes_alpha_g.bin $odir/wfluxes_mag_g.bin $odir/wfluxes_alpha_ts_g.bin $odir/wfluxes_mag_ts_g.bin
	
			#whole ecosystem
			set args = (1 $reference_dir/$allfiles[1].bin $dir/$allfiles[1].bin)
			@ y=2
			while ($y <= $#allfiles)
				set tmp = ( $args 1 $reference_dir/$allfiles[$y].bin $dir/$allfiles[$y].bin )
				set args = ($tmp)
				@ y++
			end
			#echo $args
			echo whole ecosystem
			$prog_dir/$metric_bin $#allfiles 0 1 $natvegfrac $nyear1 $nyear2 30 $year1 $year3 $grid 0.5 $args $npixel $odir/all_alpha.bin $odir/all_mag.bin $odir/all_alpha_ts.bin $odir/all_mag_ts.bin $odir/all_alpha_g.bin $odir/all_mag_g.bin $odir/all_alpha_ts_g.bin $odir/all_mag_ts_g.bin
	
	
			# combine angle, global and local change into one metric
			foreach var ( all cfluxes cstocks wfluxes )
				# calculate standard deviation
				foreach comp ( alpha_ts mag_ts alpha_ts_g mag_ts_g )
					echo SNR $var"_"$comp
					$prog_dir/limits_f $odir/$var"_"$comp.bin $odir/tmp2.$$ $npixel -1 1000
					$prog_dir/mean_ff $odir/tmp2.$$ $odir/tmp.$$ $npixel $year1 $year2 1 -sd
					$prog_dir/spectral_subset_f $odir/tmp.$$ $odir/$var"_"$comp"_sd_"$startyear1"-"$endyear1".bin" $npixel 2 1 1
				end # foreach component
				# divide components by their standard deviation
				
				# calculate 1-alpha -> Balance b (range 0 ... 2)
				echo Balance b
				$prog_dir/filescale_ff $odir/$var"_alpha".bin $odir/tmp2.$$ -1 $npixel
				$prog_dir/fileadd_ff $odir/tmp2.$$ $odir/$var"_1-alpha".bin 1 $npixel
				$prog_dir/limits_f $odir/$var"_1-alpha".bin $odir/tmp2.$$ $npixel -1 1000
				# calculate (1-alpha)/std_alpha_ts  -> used for S (formula 16 in ERL paper)
				$prog_dir/filediv_ff $odir/tmp2.$$ $odir/$var"_alpha_ts_sd_"$startyear1"-"$endyear1".bin" $odir/$var"_alpha_sddiv".bin $npixel 0 0 0 0 -9999 >& $odir/log.$$
				
				# calculate diff/std_absa_ts -> used for S (formula 16 in ERL paper)
				echo S_c
				$prog_dir/limits_f $odir/$var"_mag".bin $odir/tmp2.$$ $npixel 0 1000
				$prog_dir/filediv_ff $odir/tmp2.$$ $odir/$var"_mag_ts_sd_"$startyear1"-"$endyear1".bin" $odir/$var"_mag_sddiv".bin $npixel 1 1 0 0 -9999 >& $odir/log.$$
				
				# calculate diff_g/std_absa_ts_g -> used for S (formula 16 in ERL paper)
				echo S_g
				$prog_dir/limits_f $odir/$var"_mag_g".bin $odir/tmp2.$$ $npixel 0 1000
				$prog_dir/filediv_ff $odir/tmp2.$$ $odir/$var"_mag_ts_g_sd_"$startyear1"-"$endyear1".bin"  $odir/$var"_mag_g_sddiv".bin $npixel 1 1 0 0 -9999 >& $odir/log.$$

				echo logistic function, change/variability ratio S, formula 16 in ERL paper
				$prog_dir/logistic_ff $odir/$var"_alpha_sddiv".bin $odir/$var"_alpha_logistic".bin $npixel 2 4
				$prog_dir/logistic_ff $odir/$var"_mag_g_sddiv".bin $odir/$var"_mag_g_logistic".bin $npixel 2 4
				$prog_dir/logistic_ff $odir/$var"_mag_sddiv".bin $odir/$var"_mag_logistic".bin $npixel 2 4
				
				# transformation functions to scale magnitude between 0 and 1 -> T(x) (formula 15 ERL paper)
				echo transformation of c and g
				$prog_dir/logistic_ff $odir/$var"_mag_g".bin $odir/$var"_mag_g_scaled".bin $npixel $mag_g_maxgrowth $mag_g_growthrate $a_mag_g #0.5 4
				$prog_dir/logistic_ff $odir/$var"_mag".bin $odir/$var"_mag_scaled".bin $npixel $mag_maxgrowth $mag_growthrate $a_mag #2.5 1.5

				# add local and global magnitude
				echo combine
				#multiply with logistic function
				# extract length of difference vector (band 2)
				$prog_dir/spectral_subset_f $odir/$var"_mag_g".bin $odir/tmp3.$$ $npixel 4 1 1
				$prog_dir/filemult_ff $odir/tmp3.$$ $odir/$var"_mag_g_logistic".bin $odir/tmpmg.$$ $npixel
				$prog_dir/spectral_subset_f $odir/$var"_mag_g_scaled".bin $odir/tmp3.$$ $npixel 4 1 1
				$prog_dir/filemult_ff $odir/$var"_mag_g_logistic".bin $odir/tmp3.$$ $odir/tmpmgs.$$ $npixel            # g * S(g)
				$prog_dir/spectral_subset_f $odir/$var"_mag".bin $odir/tmp3.$$ $npixel 4 1 1
				$prog_dir/filemult_ff $odir/tmp3.$$ $odir/$var"_mag_logistic".bin $odir/tmpm.$$ $npixel
				$prog_dir/spectral_subset_f $odir/$var"_mag_scaled".bin $odir/tmp3.$$ $npixel 4 1 1                    
				$prog_dir/filemult_ff $odir/$var"_mag_logistic".bin $odir/tmp3.$$ $odir/tmpms.$$ $npixel               # c * S(c)
				
				$prog_dir/filescale_ff $odir/$var"_1-alpha".bin $odir/tmpa2.$$ 2 $npixel
				$prog_dir/limits_f $odir/tmpa2.$$ $odir/tmpa.$$ $npixel 0 1     # limit now 0 ... 60°
				$prog_dir/filemult_ff $odir/tmpa.$$ $odir/$var"_alpha_logistic".bin $odir/tmpa2.$$ $npixel   # b * S(b)
				$prog_dir/filesum_ff $odir/tmpmg.$$ $odir/tmpm.$$ $odir/tmp.$$ $npixel         # unscaled g + c
				$prog_dir/filesum_ff $odir/tmpmgs.$$ $odir/tmpms.$$ $odir/tmps.$$ $npixel      # scaled g + c
				
				# add to combined metric
				$prog_dir/filesum_ff $odir/tmp.$$ $odir/tmpa2.$$ $odir/tmp2.$$ $npixel         # unscaled g + c + 2*b
				$prog_dir/filesum_ff $odir/tmps.$$ $odir/tmpa2.$$ $odir/tmp2s.$$ $npixel       # scaled g + c + 2*b
				# 1/3 weight for each component
				$prog_dir/filescale_ff $odir/tmp2.$$ $odir/tmp.$$ 0.33333333 $npixel           # unscaled
				$prog_dir/filescale_ff $odir/tmp2s.$$ $odir/tmps.$$ 0.33333333 $npixel         # scaled
				# set -99 as new NA value
				$prog_dir/limits_f $odir/tmp.$$ $odir/$var"_comb".bin $npixel -99 1000         # unscaled
				$prog_dir/limits_f $odir/tmps.$$ $odir/$var"_comb_scaled".bin $npixel -99 1000 # scaled
				# remove artefacts from old version
				if ( { test -e $odir/$var"_comb_ts".bin } ) then
					rm $odir/$var"_comb_ts".bin
				endif
			end #foreach var
			
			# sum deltav and all variables metric into one metric
			$prog_dir/filescale_ff $odir/all_comb.bin $odir/tmp.$$ 0.75 $npixel              # unscaled c + g + 2*b
			$prog_dir/filescale_ff $odir/all_comb_scaled.bin $odir/tmps.$$ 0.75 $npixel      # scaled c + g + 2*b
			#weight deltav by 1/4 in metric
			$prog_dir/filescale_ff $odir/deltav.bin $odir/tmp2.$$ 0.25 $npixel
			$prog_dir/filesum_ff $odir/tmp.$$ $odir/tmp2.$$ $odir/tmp3.$$ $npixel            # unscaled
			# set -9 as new NA value
			$prog_dir/limits_f $odir/tmp3.$$ $odir/cm.bin $npixel -9 1000                    # unscaled

			$prog_dir/filesum_ff $odir/tmps.$$ $odir/tmp2.$$ $odir/tmp3.$$ $npixel           # scaled
			# set -9 as new NA value
			$prog_dir/limits_f $odir/tmp3.$$ $odir/cm_scaled.bin $npixel -9 1000             # scaled

			rm $odir/tmp*.*
	
  	endif
  	rm $odir/log.$$
	end #foreach scenario
end #foreach model
