####### script that contains code necessary for figures in paper as submitted #######
## except for Fig. 4 and Fig. S7 (code in metric_components.r)

rm(list=ls(all=TRUE))
library(fields)
library(maps)
library(spatial)
library(maptools)
library(raster)
library(RColorBrewer)
library(Hmisc)
library(shape)

### filesystem details (paths) removed ###
basepath <- c("***")
climate.scen.path <- "***"
lpj.input.path <- "***"
shape.path <- "***"

reference.path <- "***"
scenario.path  <- "***"
metric.path <- "***"
metric.path.all.combined <- "***"

add.rcps2embers <- TRUE #FALSE

reference.runtime <- c(1901, 2009)
reference.nyear <- reference.runtime[2]-reference.runtime[1]+1
scenario.runtime  <- c(2010, 2115)
scenario.nyear <- scenario.runtime[2]-scenario.runtime[1]+1

period.nyear <- 30 # length of reference and scenario periods

reference.period  <- c(reference.runtime[2]-period.nyear+1, reference.runtime[2])
reference.period2 <- c(1901,1930)
scenario.period   <- c(scenario.runtime[2]-period.nyear+1, scenario.runtime[2])

climate.runtime <- c(1901, 2115)

landuse.ref <- 2000

threshold.ember <- 0.3 # threshold to use for ember diagrams
threshold.severe <- 0.3 # threshold between moderate and major change
threshold.moderate <- 0.1
threshold.nonvegetated <- 0.05
ylim.ember <- c(100, 0) # 90,0, limits for ember diagram axis (in million km²) -> actually in % for the standard plot

print("LPJ Runs:")
print(paste("Reference run: ", reference.runtime[1], "-", reference.runtime[2], sep=""))
print(paste("Reference period: ", reference.period[1], "-", reference.period[2], sep=""))
print(paste("Scenario run: ", scenario.runtime[1], "-", scenario.runtime[2], sep=""))
print(paste("Scenario period: ", scenario.period[1], "-", scenario.period[2], sep=""))

models <- c(
	"cccma_cgcm3_1",    # 1
	"cnrm_cm3",         # 2
	"csiro_mk3_0",      # 3
	"gfdl_cm2_0",       # 4
	"gfdl_cm2_1",       # 5
	"giss_model_e_h",   # 6
	"giss_model_e_r",   # 7
	"iap_fgoals",       # 8
	"inmcm3_0",         # 9
	"ipsl_cm4",         #10
	"miroc3_2_hires",   #11
	"miroc3_2_medres",  #12
	"miub_echo_g",      #13
	"mpi_echam5",       #14
	"mri_cgcm2_3_2a",   #15
	"ncar_ccsm3_0",     #16
	"ncar_pcm1",        #17
	"ukmo_hadcm3",      #18
	"ukmo_hadgem1"      #19
)

target.temps <- c(
	"1.5K",
	"2.0K",
	"2.5K",
	"3.0K",
	"3.5K",
	"4.0K",
	"4.5K",
	"5.0K"
)

rcps <- paste("rcp", c(3,45, 6,85), sep="")
rcpnames <- paste("RCP", c("2.6", "4.5", "6", "8.5"))

model.names.official <- c(
"CGCM3.1(T47)",			# 1
"CNRM-CM3",					# 2
"CSIRO-Mk3.0",			# 3
"GFDL-CM2.0",				# 4
"GFDL-CM2.1",				# 5
"GISS-EH",					# 6
"GISS-ER",					# 7
"FGOALS-g1.0",			# 8
"INM-CM3.0",				# 9
"IPSL-CM4",					#10
"MIROC3.2(hires)",	#11
"MIROC3.2(medres)",	#12
"ECHO-G",						#13
"ECHAM5/MPI-OM",		#14
"MRI-CGCM2.3.2",		#15
"CCSM3",						#16
"PCM",							#17
"UKMO-HadCM3",			#18
"UKMO-HadGEM1"			#19
) # end official model names


rcps[1] <- paste(rcps[1], "pd", sep="")

# filenames
grid.name <- "grid.out"
metric.filenames <- list()
metric.filenames[["combined"]] <- list(
	"all" = "cm_scaled.bin",
	"cfluxes" = "cfluxes_comb_scaled.bin",
	"cstocks" = "cstocks_comb_scaled.bin",
	"deltav" = "deltav.bin",
	"wfluxes" = "wfluxes_comb_scaled.bin"
)
metric.filenames[["local"]] <- list(
	"all" = "all_mag_scaled.bin",
	"cfluxes" = "cfluxes_mag_scaled.bin",
	"cstocks" = "cstocks_mag_scaled.bin",
	"wfluxes" = "wfluxes_mag_scaled.bin"
)
metric.filenames[["global"]] <- list(
	"all" = "all_mag_g_scaled.bin",
	"cfluxes" = "cfluxes_mag_g_scaled.bin",
	"cstocks" = "cstocks_mag_g_scaled.bin",
	"wfluxes" = "wfluxes_mag_g_scaled.bin"
)
metric.filenames[["balance"]] <- list(
	"all" = "all_1-alpha.bin",
	"cfluxes" = "cfluxes_1-alpha.bin",
	"cstocks" = "cstocks_1-alpha.bin",
	"wfluxes" = "wfluxes_1-alpha.bin"
)
fpc.mask.filename <- paste("fpcmask_", threshold.nonvegetated, ".bin", sep="")

print("Load and simplify world borders")
world.borders <- readShapePoly(paste(shape.path, "TM_WORLD_BORDERS-0.3", sep=""))
world.borders.simple <- thinnedSpatialPoly(world.borders, tolerance=0.25, minarea=0.01, topologyPreserve=TRUE)


print("Load biome classification derived from LPJ reference run")
biome.filename <- paste(basepath, reference.path, "biome_class.bin", sep="")
biome.lpj <- readBin(biome.filename, what=integer(), size=1, n=file.info(biome.filename)$size)

biome.class.names.lpj <- c(
	"Tropical Rainforest",										# 1
	"Tropical Seasonal & Deciduous Forest",	# 2
	"Temperate Broadleaved Evergreen Forest",# 3
	"Temperate Broadleaved Deciduous Forest",# 4
	"Mixed Forest",													# 6
	"Temperate Coniferous Forest",						# 5
	"Boreal Evergreen Forest",								# 7
	"Boreal Deciduous Forest",								# 8
	"Warm Woody Savanna, Woodland & Shrubland",		# 9
	"Warm Savanna & Open Shrubland",							#10
	"Warm Grassland",															#11
	"Temperate Woody Savanna, Woodland & Shrubland",		# 12 
	"Temperate Savanna & Open Shrubland",							#13
	"Temperate Grassland",															#14
	"Arctic Tundra",													#15
	"Desert",																#16
	"Rocks and Ice",												#17
	"Water"																	#18
)
plot.classes <- c(1:16)
biome.class.names.combined <- c(
	"Tropical Rainforest",										# 1
	"Tropical Seasonal & Deciduous Forest",	# 2
	"Temperate Broadleaved Evergreen Forest",# 3
	"Temperate Broadleaved Deciduous Forest",# 4
	"Mixed Forest",													# 6
	"Temperate Coniferous Forest",						# 5
	"Boreal Forest",								# 7
	"Boreal Forest",								# 8
	"Warm Savanna & Shrubland",		# 9
	"Warm Savanna & Shrubland",							#10
	"Warm Grassland",															#11
	"Temperate Savanna & Shrubland",		# 12 
	"Temperate Savanna & Shrubland",							#13
	"Temperate Grassland",															#14
	"Tundra",													#15
	"Desert",																#16
	"Rocks and Ice",												#17
	"Water"																	#18
)

biome.class.cols <- c(
	rev(brewer.pal(9, "YlGn"))[-9], # all trees
	rev(brewer.pal(9, "Oranges"))[4:6], # warm below forest
	rev(brewer.pal(9, "GnBu"))[2:5], # cold below forest
	"cornsilk4", # desert
	"white",
	"grey90"
) #
biome.class.cols.v2 <- c(rev(brewer.pal(9, "Oranges"))[2:3],
                         rev(brewer.pal(9, "YlGn"))[seq(1,7,by=2)],
                         rev(brewer.pal(9, "GnBu"))[4:3],
                         rev(brewer.pal(9, "Oranges"))[5:7],
                         rev(brewer.pal(9, "GnBu"))[c(6:8,1)],
                         "yellow")
											
soil.name <- paste(lpj.input.path, "soil.bin", sep="")
landuse.name <- paste(gsub("cru06", "cru09",lpj.input.path), "cft1700_2005_bioenergy_sc.bin", sep="")
reference.grid.name <- paste(basepath,reference.path,grid.name, sep="")
NCELL <- file.info(reference.grid.name)$size/4
country.name <- paste(lpj.input.path, "cow_mg_2006.bin", sep="")
name.name <- paste(basepath, "analysis/countrynames.txt", sep="")

grid.data <- array(readBin(reference.grid.name, what=integer(), size=2, n=NCELL*2), dim=c(2, NCELL))/100
earthradius <- 6371000.785
cellwidth <- earthradius*pi/180

grid.idata <- array(,dim=dim(grid.data))
grid.idata[1,] <- (grid.data[1,]+180.25)/0.5
grid.idata[2,] <- (grid.data[2,]+90.25)/0.5

grid.area <- (cellwidth*0.5)*(cellwidth*0.5)*cos(grid.data[2,]/180*pi)

###################################################################
# Helper function to mirror array data in one dimension (either rows or columns)
mirror <- function(in.data, in.dim) {
	if((in.dim%in%c(1:2))==F) stop("in.dim must be 1 or 2")
		if(length(dim(in.data))!=2) stop("in.data must be 2-dimensional array")
			maximum <- dim(in.data)[in.dim]
			for (i in 1:(maximum/2)) {
				if(in.dim==1) {
					templine <- in.data[i,]
					in.data[i,] <- in.data[maximum-i+1,]
					in.data[maximum-i+1,] <- templine
				}
				if(in.dim==2) {
					templine <- in.data[,i]
					in.data[,i] <- in.data[,maximum-i+1]
					in.data[,maximum-i+1] <- templine
				}
			}
			return(in.data)
}
###################################################################


soil.data <- as.integer(readBin(soil.name, raw(), n=file.info(soil.name)$size))
NCELL.input <- length(soil.data)

ingrid.filename <- paste(lpj.input.path, "grid.bin", sep="")
ingrid.file <- file(ingrid.filename, "rb")
seek(ingrid.file, where=7, origin="start")
endian <- "little"
header <- readBin(ingrid.file, what=integer(), size=4, n=7, endian=endian)
if(header[1] != 1) {
	endian <- "big"
	seek(ingrid.file, where=7, origin="start")
	header <- readBin(ingrid.file, what=integer(), size=4, n=7, endian=endian)
	if(header[1] != 1) stop("Error: cannot determine endian of input grid")
}
if(header[6] != NCELL.input) stop("Error: soil input and input grid do not match")
ingrid.data <- array(readBin(ingrid.file, what=integer(), size=2, n=NCELL.input*2, endian=endian), dim=c(2, NCELL.input))/100
close(ingrid.file)
ingrid.area <- (cellwidth*0.5)*(cellwidth*0.5)*cos(ingrid.data[2,]/180*pi)

ingrid.idata <- array(,dim=dim(ingrid.data))
ingrid.idata[1,] <- (ingrid.data[1,]+180.25)/0.5
ingrid.idata[2,] <- (ingrid.data[2,]+90.25)/0.5

# land-use data
endian="little"
landuse.file <- file(landuse.name, "rb")
seek(landuse.file, where=10, origin="start")
header <- readBin(landuse.file, what=integer(), size=4, n=7, endian=endian)
if(header[1] != 1) {
	endian <- "big"
	seek(landuse.file, where=10, origin="start")
	header <- readBin(landuse.file, what=integer(), size=4, n=7, endian=endian)
	if(header[1] != 1) stop("Error: cannot determine endian of landuse file")
}
if(header[6] != NCELL.input) stop("Error: landuse data and input grid do not match")
landuse.nbands <- header[7]
landuse.startyear <- header[3]
landuse.nyear <- header[4]

landuse.offset <- landuse.ref-landuse.startyear
if(landuse.offset > (landuse.nyear-1)) landuse.offset <- landuse.nyear-1 # use 2000 (landuse.ref) or last year of file if file ends earlier
print(paste("Loading landuse data for year", landuse.startyear+landuse.offset, "from file", landuse.name))
seek(landuse.file, where=landuse.offset*landuse.nbands*header[6]*2+38, origin="start")
landuse.data <- readBin(landuse.file, what=integer(), size=2, n=NCELL.input*landuse.nbands, endian=endian)
dim(landuse.data) <- c(landuse.nbands, NCELL.input)
# sum up over all cft's
landuse.data <- apply(landuse.data, 2, sum, na.rm=TRUE)
if(max(landuse.data > 1000)) print("Warning: there are cells with landuse fraction > 100% in landuse input")
close(landuse.file)
# use only cells that are in output
landuse.data <- landuse.data[which(soil.data > 0)]
natveg.frac <- (1000-landuse.data)/1000
natveg.frac[which(natveg.frac <= 0)] <- 0

# vegetation cover 
fpc.ref.filename <- paste(basepath, reference.path, "fpc.bin", sep="")
fpc.ref.offset <- reference.period[1]-reference.runtime[1]
fpc.ref.file <- file(fpc.ref.filename, "rb")
seek(fpc.ref.file, where=NCELL*10*fpc.ref.offset*4, origin="start")
fpc.ref.data <- readBin(fpc.ref.file, what=numeric(), size=4, n=NCELL*10*period.nyear)
dim(fpc.ref.data) <- c(NCELL, 10, period.nyear)
vegcov.ref <- apply(fpc.ref.data[,2:10,], 1, sum, na.rm=TRUE)/period.nyear
close(fpc.ref.file)

total.natveg.area <- grid.area*natveg.frac										# remove croplands/pasture
total.natveg.area <- sum(total.natveg.area)

num.valid <- function(indata) {
	length(which(is.na(indata)==FALSE))
}
#stop("Test")

if(add.rcps2embers) {
	# Load RCP data
	delta.gmt.paper <- array(dim=c(3,length(rcps)))
	delta.gmt.paper[,1] <- c(1.3, 1.5, 1.9)
	delta.gmt.paper[,2] <- c(2.0, 2.4, 3.0)
	delta.gmt.paper[,3] <- c(2.6, 3.0, 3.7)
	delta.gmt.paper[,4] <- c(4.0, 4.9, 6.1)
}
#stop("Test after RCP loading")
## Embers diagram

embers <- colorRampPalette(c("white", brewer.pal(9, "YlOrRd")[-9]), bias=0.9)
embers2 <- colorRampPalette(c("white", brewer.pal(9, "Blues")[-6:-9]), bias=1.1)

col.scale2 <- c(brewer.pal(12, "Paired"), brewer.pal(8, "Set2")) # GCM model colors

skip <- FALSE
if(!skip) {
for(scale in c("combined", "local", "global", "balance")[1]) {
	for(subset in c("all", "cfluxes", "cstocks", "deltav", "wfluxes")[1]) {
		if(subset == "deltav" && scale != "combined")
			next
		
		force.reload <- FALSE
		rdata.filename <- paste(basepath, "analysis/", subset, "_", scale, "_metric.data_dec2012.RData", sep="")
		vegcov.filename <- paste(basepath, "analysis/vegcov_dec2012.RData", sep="")
		if(file.exists(vegcov.filename)) {
			load(vegcov.filename)
			vegcov.isloaded <- TRUE
			print(paste("Reloading vegetation cover maps from", vegcov.filename))
		} else {
			vegcov.isloaded <- FALSE
			vegcov.array <- array(dim=c(NCELL, length(models), length(target.temps)))
			print("Generating vegetation cover maps")
		}
		if(file.exists(rdata.filename) && !force.reload) {
			print(paste("Reloading metric data from previous run from", rdata.filename))
			load(rdata.filename)
		} else {
			nmodel.loaded <- 0
			data.array <- array(dim=c(NCELL, length(models), length(target.temps)))
			fpc.mask <- array(dim=c(NCELL, length(models), length(target.temps)))
			if(scale != "combined") {
				ratio.array <- array(dim=c(NCELL, length(models), length(target.temps)))
			}
			for(model in 1:length(models)) {
				for(target in 1:length(target.temps)) {
					data.filename <- paste(basepath, scenario.path, models[model], "/", target.temps[target], "/", metric.path, metric.filenames[[scale]][[subset]], sep="")
					if(scale=="combined" && (subset=="all" || subset=="deltav")) {
  					data.filename <- paste(basepath, scenario.path, models[model], "/", target.temps[target], "/", metric.path.all.combined, metric.filenames[[scale]][[subset]], sep="")
  				}
					if(!file.exists(data.filename)) {
						print(paste("File", data.filename, "does not exist. Skipping", models[model], target.temps[target]))
						next
					}
					print(paste("Loading data from", data.filename))
					data.file <- file(data.filename, "rb")
					if(scale == "local" || scale == "global") {
						seek(data.file, where=NCELL*4, origin="start") # skip first band in file -> diff is in 2nd band
					}
					data.array[,model,target] <- readBin(data.file, what=numeric(), size=4, n=NCELL)
					nmodel.loaded <- nmodel.loaded+1/length(target.temps)
					if(max(data.array[,model,target], na.rm=TRUE) > 1 || min(data.array[,model,target], na.rm=TRUE) < 0) {
						print(paste("Warning: data in file", data.filename, "is outside range 0...1"))
					}
					close(data.file)
					
					# fpcmask
					fpc.mask.name <- paste(basepath, scenario.path, models[model], "/", target.temps[target], "/", metric.path, fpc.mask.filename, sep="")
					fpc.mask[,model,target] <- as.integer(readBin(fpc.mask.name, what=raw(), n=NCELL))
					
					# S/N ratio
					if(scale != "combined") {
						if(scale=="balance") {
							ratio.filename <- gsub("1-alpha", "alpha_logistic", data.filename, fixed=TRUE)
						} else {
							ratio.filename <- gsub("_scaled", "_logistic", data.filename, fixed=TRUE)
						}
						print(paste("Loading S/N ratio from", ratio.filename))
						ratio.array[,model,target] <- readBin(ratio.filename, double(), size=4, n=NCELL)
					}

					#vegetation cover
					if(!vegcov.isloaded) {
						vegcov.name <- paste(basepath, scenario.path, models[model], "/", target.temps[target], "/vegcov.bin", sep="")
						vegcov.offset <- (scenario.period[1]-scenario.runtime[1])*NCELL*4
						vegcov.file <- file(vegcov.name, "rb")
						seek(vegcov.file, where=vegcov.offset, origin="start")
						vegcov.data <- readBin(vegcov.file, what=numeric(), size=4, n=NCELL*period.nyear)
						dim(vegcov.data) <- c(NCELL, period.nyear)
						vegcov.data <- apply(vegcov.data, 1, mean)
						vegcov.array[, model, target] <- vegcov.data
						rm(vegcov.data)
						close(vegcov.file)
					}
					gc(reset=T)
				}
			}
			if(scale == "balance") {
					## scale balance between 0 ... 1 (60° -> 1)
					data.array <- data.array*2
					data.array[which(data.array>1)] <- 1
					data.array[which(data.array<0)] <- 0
			}
			if(scale != "combined") {
				save(data.array, ratio.array, nmodel.loaded, fpc.mask, file=rdata.filename)
			} else {
				save(data.array, nmodel.loaded, fpc.mask, file=rdata.filename)#, vegcov.array
			}
			if(!vegcov.isloaded) {
			  save(vegcov.array, file=vegcov.filename)
			}
		}
		model.max <- apply(data.array, c(1,3), max, na.rm=TRUE)
		
		## weighting S(x)*x
		if(scale != "combined") {
			print("Weighting with change-to-variability ratio")
			data.array <- data.array*ratio.array
		}
		
		risk.levels <- seq(0,1,length.out=nmodel.loaded+1)
		very.likely <- min(which(risk.levels>0.9)) # very likely -> IPCC treatment of uncertainty
		likely <- min(which(risk.levels>0.66)) # likely -> IPCC treatment of uncertainty
		unlikely <- max(which(risk.levels<0.33))
		low.conf <- min(which(risk.levels>0.20)) # low confidence -> IPCC treatment
		med.conf <- min(which(risk.levels>0.50)) # medium confidence -> IPCC treatment
		high.conf <- min(which(risk.levels>0.80)) # high confidence -> IPCC treatment
		
		# loop over thresholds and load data
		threshold.closed <- FALSE #TRUE
		threshold.loop <- c(0.1, 0.3)#, 0.5)
		model.severe <- array(dim=c(NCELL, length(target.temps), length(threshold.loop)))
		area.severe.risk <- array(dim=c(length(risk.levels), length(target.temps), length(threshold.loop)))
		area.severe.additiverisk <- array(dim=c(length(risk.levels), length(target.temps), length(threshold.loop)))
		
		for(t in 1:length(threshold.loop)) {
			threshold.ember <- threshold.loop[t]
			data.severe <- array(0, dim=dim(data.array))
			if(t < length(threshold.loop) && threshold.closed) {
				data.severe[which(data.array > threshold.loop[t] & data.array <= threshold.loop[t+1])] <- 1
				if(threshold.loop[t]==0) {
					data.severe[which(data.array== threshold.loop[t])] <- 1
				} else {
					print(paste("Closed interval >", threshold.loop[t], " and <=", threshold.loop[t+1]))
				}
			} else {
				data.severe[which(data.array > threshold.loop[t])] <- 1
				if(threshold.loop[t]==0) {
					data.severe[which(data.array== threshold.loop[t])] <- 1
				} else {
					print(paste("Open interval >", threshold.loop[t]))
				}
			}
			model.severe[,,t] <- apply(data.severe, c(1,3), sum, na.rm=TRUE)/apply(data.array, c(1,3), num.valid)
			area.frac.severe <- apply(data.severe*grid.area*natveg.frac, c(2,3), sum, na.rm=TRUE)/apply(fpc.mask*grid.area*natveg.frac, c(2,3), sum, na.rm=TRUE)
			for(risk in 2:length(risk.levels)) {
				for(target in 1:length(target.temps)) {
					area.severe.additiverisk[risk,target,t] <- sum((grid.area*natveg.frac)[which(model.severe[,target,t] > risk.levels[risk-1]*1.00001)], na.rm=TRUE)#/sum(ingrid.area)
					area.severe.risk[risk,target,t] <- sum((grid.area*natveg.frac)[which(model.severe[,target,t] <= risk.levels[risk]*1.00001 & model.severe[,target,t] > risk.levels[risk-1]*1.00001)], na.rm=TRUE)#/sum(ingrid.area)
				}
			}
		} # end threshold loop

		skip <- FALSE#TRUE
		# figure 1 (main text)
		if(!skip) {
		# Plot embers by Gamma threshold
		legend.plotted <- FALSE
		display.dimensions <- integer(2)
		display.dimensions[1] <- ceiling(sqrt(length(threshold.loop)))
		display.dimensions[2] <- ceiling(length(threshold.loop)/display.dimensions[1])
		if(length(threshold.loop) < 3) {
			display.dimensions <- c(1,length(threshold.loop))
		}
		display.dimensions <- array(dim=display.dimensions)
		width <- c(rep(ifelse(ncol(display.dimensions)>1, 3+0.5, 3+0.5),ncol(display.dimensions)), 1)
		if(length(threshold.loop)==1)
			width[1] <- 4
		height <- c(rep(1,nrow(display.dimensions)))
		layout <- matrix(c(1:length(display.dimensions))+1, ncol=ncol(display.dimensions), byrow=TRUE)
		legend.col <- integer(nrow(display.dimensions))
		legend.col[] <- 1
		layout <- cbind(layout,legend.col)
		if(add.rcps2embers) {
			layout <- rbind(layout, c(max(layout)+1:dim(display.dimensions)[2], 1))
			height <- c(height, 0.18)
		}

    add.arrows <- TRUE
		eps.filename <- paste(basepath, "analysis/ember_complete_", gsub(".", "_", gsub(", ", "_", toString(threshold.loop), fixed=T), fixed=TRUE), "_", ifelse(add.rcps2embers, "RCPs_", ""), "sep2013.eps", sep="")
		setEPS()
		postscript(file=eps.filename, width=503.61368/72, height=(0.5+(2.75-0.25)*ifelse(add.rcps2embers,length(height)-1,length(height))+ifelse(add.rcps2embers,0.5-0.15,0)), paper="special")#, family="serif"
		par(oma=c(ifelse(add.rcps2embers,0.5,2),3,2.3-2.3,2.3))
		layout(layout, width, height)
		# plot legend
		par(mar=c(2.5*nrow(layout)+2.5,ifelse(add.rcps2embers,0.85, 0.85)/sqrt(sqrt(ncol(display.dimensions))),2.5*nrow(layout)-2,ifelse(add.rcps2embers,4.5,4.5)))#, cex.lab=0.75, cex.axis=0.75)
		plot(0,0,type="n", xlim=c(0.25,0.75),ylim=c(0,1), xaxt="n", yaxt="n", xlab="", ylab="", yaxs="i")
		rect(xleft=rep(0,length(risk.levels)-1), ybottom=seq(0,1,length.out=length(risk.levels))[-length(risk.levels)], xright=rep(1,length(risk.levels)-1), rep(1,length(risk.levels)-1), col=embers(length(risk.levels)+2)[-1:-3], border=NA)
		box()
		axis(4, at=(floor(seq(1,length(risk.levels)-1,length.out=5))-0.5)/(length(risk.levels)-1), labels=c(paste(floor(seq(1,length(risk.levels)-1,length.out=5))[1:5], " of ", length(risk.levels)-1, sep="")), las=1, cex.axis=1.0)
		mtext("Number of climate models in agreement", side=4, line=0.5, outer=TRUE, cex=0.8-0.1, adj=0.65+0.05)
		metric.desc <- switch(subset,
													all = "biogeochemical and structural shifts",
													cfluxes = "carbon fluxes",
													cstocks = "carbon stocks",
													deltav = "structural shifts",
													wfluxes = "water fluxes")

		if(scale == "local") metric.desc <- paste(metric.desc, "- local change")
		if(scale == "global") metric.desc <- paste(metric.desc, "- global importance")
		legend.plotted <- TRUE
		
		for(t in length(threshold.loop):1) { #c(2,3,1)) {#
			threshold.ember <- threshold.loop[t]
			
			par(mar=c(2.5+1.5-1.5,2.0-1,ifelse(length(threshold.loop)>1,2.5-0.5,0.5),0)+0.1)#, oma=c(2.25,0,1,3))
			for(risk in 2:length(risk.levels)) {
				if(risk == 2) {
					border <- NA #"black"
					add <- FALSE
					axisnames <- FALSE #TRUE
					axes <- FALSE
				} else {
					border <- NA
					add <- TRUE
					axisnames <- FALSE
					axes <- FALSE
				}
				barplot(area.severe.additiverisk[risk,,t]/sum(grid.area*natveg.frac)*100, names.arg=target.temps, col=embers(length(risk.levels)+2)[-1:-3][risk-1], border=border, add=add, ylim=ylim.ember, xlim=c(-0.25,length(target.temps)+0.5+ifelse(add.arrows,2,0)), width=0.95, space=1/19, axisnames=axisnames, axes=axes, las=1, cex.axis=0.25, cex.lab=0.25, xaxs="i")
				if(risk==2) {
				  for(target in 1:length(target.temps)) {
  				  lines(seq(1, length(target.temps))[target]-c(0.95, -0.0), rep(area.severe.additiverisk[risk,target,t]/sum(grid.area*natveg.frac)*100, 2), lwd=0.5)
  				}
        }
			}

      if(add.arrows) {
			  lines(c(2,length(target.temps)+1.5+0.25)-0.475, rep(area.severe.additiverisk[med.conf,2,t]/sum(grid.area*natveg.frac)*100, 2), lty=3, lwd=0.5)
			  lines(c(5,length(target.temps)+1.5+0.25)-0.475, rep(area.severe.additiverisk[med.conf,5,t]/sum(grid.area*natveg.frac)*100, 2), lty=3, lwd=0.5)
			  lines(c(8,length(target.temps)+1.5+0.25)-0.475, rep(area.severe.additiverisk[med.conf,8,t]/sum(grid.area*natveg.frac)*100, 2), lty=3, lwd=0.5)
			  Arrows(length(target.temps)+0.95+0.25, area.severe.additiverisk[med.conf,5,t]/sum(grid.area*natveg.frac)*100, length(target.temps)+0.95+0.25,area.severe.additiverisk[med.conf,2,t]/sum(grid.area*natveg.frac)*100, lty=1, arr.length=0.12, arr.width=0.1, lwd=1.5, code=3, arr.type="T", arr.adj=1)#"triangle"
			  Arrows(length(target.temps)+0.95+0.25, area.severe.additiverisk[med.conf,8,t]/sum(grid.area*natveg.frac)*100, length(target.temps)+0.95+0.25,area.severe.additiverisk[med.conf,5,t]/sum(grid.area*natveg.frac)*100, lty=1, arr.length=0.12, arr.width=0.1, lwd=1.5, code=3, arr.type="T", arr.adj=1)#"triangle"
			  text(x=rep(length(target.temps)+0.6,2), y=area.severe.additiverisk[med.conf,c(2,5,8),t]/sum(grid.area*natveg.frac)*100+strheight("I")*0.6, labels=target.temps[c(2,5,8)], cex=1)
			}

			#Major ecosystem change
			if(length(threshold.loop) > 1) {
				title(main=ifelse(threshold.ember >= threshold.severe, "Major change", ifelse(threshold.ember >= threshold.moderate,ifelse(threshold.closed, "Moderate change", "Moderate or major change"), "Small change")), cex.main=1.25*0.9, line=1.25, adj=ifelse(add.arrows,0.2+0.1,0.5))
				if(t < length(threshold.loop) && threshold.closed) {
					threshold.upper <- threshold.loop[t+1]
					if(threshold.ember > 0) {
						title(main=substitute(paste(plain("("), Gamma > threshold.ember, plain(" & "), Gamma <= threshold.upper, plain(")"), sep=""), list(threshold.ember=threshold.ember, threshold.upper=threshold.upper)), line=0.5, cex.main=1, adj=ifelse(add.arrows,0.3,0.5))
					} else {
						title(main=substitute(paste(plain("("), Gamma <= threshold.upper, plain(")"), sep=""), list(threshold.ember=threshold.ember, threshold.upper=threshold.upper)), line=0.5, cex.main=1, adj=ifelse(add.arrows,0.3,0.5))
					}
				} else {
					title(main=substitute(paste("(", Gamma > threshold, ")", sep=""), list(threshold=threshold.ember)), line=0.5, cex.main=1, adj=ifelse(add.arrows,0.3,0.5))
				}
				title(xlab="GMT increase [K] by around 2100", cex.lab=1.0, line=3-1.5, adj=ifelse(add.arrows,0.1+0.1,0.5))
			} else {
				title(xlab="GMT increase [K] by around 2100", cex.lab=0.9, line=3-1.5, adj=ifelse(add.arrows,0.1+0.1,0.5))
			}
			if(par()[["mfg"]][2]==1) {
				mtext("Fraction of ice-free land area [%]", side=2, cex=0.7, line=2.75, xpd=NA)
			}
			axis(2, las=2, cex.axis=1.0)#, cex.axis=0.75) 
			mtext(gsub("K", "", target.temps), side=1, line=0.5, at = seq(0,length(target.temps)-1)+0.5, cex=0.7)

			lines(seq(0,length(target.temps))[-1]-0.475, area.severe.additiverisk[high.conf,,t]/total.natveg.area*100, pch=17, col="white", lwd=2)
			lines(seq(0,length(target.temps))[-1]-0.475, area.severe.additiverisk[med.conf,,t]/total.natveg.area*100, pch=18, lwd=1.5, lty=1)
			lines(seq(0,length(target.temps))[-1]-0.475, area.severe.additiverisk[low.conf,,t]/total.natveg.area*100, pch=18, lwd=1, lty=3)

			
		} # end plot embers by threshold loop
		spaces <- (layout[nrow(display.dimensions), ncol(display.dimensions)]-1)
		filled <- length(threshold.loop)
		while(filled < spaces) {
			# fill empty spaces
			plot.new()
			filled <- filled+1
		}
		if(add.rcps2embers) {
		  rcp.cols <- c(rgb(0, 0, 255, maxColorValue=255), rgb(121, 188, 255, maxColorValue=255), rgb(255, 130, 45, maxColorValue=255), rgb(255, 0, 0, maxColorValue=255))
			par(mar=c(0,2.0-1,1-0.75,0)+0.1)
			for(blub in 1:ncol(display.dimensions)) {
				plot(1,1, type="n", xlim=c(-0.25, length(target.temps)+0.5+ifelse(add.arrows,2,0)), ylim=c(0.5,length(rcps)+0.5), xaxt="n", yaxt="n", frame.plot=F, ylab="", xlab="", xaxs="i")
				rect((delta.gmt.paper[1,]-1)*2-0.5, 1:length(rcps)-0.3,(delta.gmt.paper[3,]-1)*2-0.5, 1:length(rcps)+0.3, density=NA, col=rcp.cols, border="black", lwd=0.5)
				for(r in 1:length(rcps)) {
					lines(rep((delta.gmt.paper[2,r]-1)*2-0.5,2), r+c(0.29,-0.29), lwd=2)
				}
				if(blub==1) {
				  axis(2, at=1:length(rcps), labels=FALSE, las=2, lwd=-1, lwd.ticks=-1, adj=0, line=-0.5, cex.axis=0.9/sqrt(nrow(display.dimensions)))
				  mtext(rcpnames,side=2, at=1:length(rcps), col=rcp.cols, cex=par("cex")*0.9/sqrt(nrow(display.dimensions)), las=2, line=1)
			  }
			}
		}
		dev.off()
		print(paste("Embers saved to", eps.filename))
		} # end skip
	} # end of subset loop
} # end of scale loop
} # end skip

frac.severe.risk <- array(dim=c(length(models), length(target.temps), length(threshold.loop)))
ensemble.additional.frac <- array(dim=c(length(models), length(target.temps), length(threshold.loop)))
for(m in 1:length(models)) {
	for(t in 1:length(target.temps)) {
		for(l in 1:length(threshold.loop)) {
			frac.severe.risk[m,t,l] <- sum((grid.area*natveg.frac)[which(data.array[,m,t] > threshold.loop[l])]) / total.natveg.area 
			ensemble.additional.frac[m,t,l] <- area.severe.additiverisk[2,t,l]/frac.severe.risk[m,t,l]/total.natveg.area
		}
	}
}
dimnames(ensemble.additional.frac)[[1]] <- model.names.official
dimnames(ensemble.additional.frac)[[2]] <- target.temps
dimnames(ensemble.additional.frac)[[3]] <- threshold.loop
dimnames(frac.severe.risk) <- dimnames(ensemble.additional.frac)

if(FALSE) {
  ## maps of Gamma for each GCM/target.temp
  # figure S8 (supporting information)
  ncols <- 3
  nrows <- 10
  plot.targets <- c(2,5,8) #1:length(target.temps)
  plot.models <- 1:length(models)
  map.layout <- matrix(1:(ncols*nrows), ncol=ncols, byrow=FALSE)
  map.layout <- rbind(map.layout, rep(ncols*nrows+1,ncols))
  widths <- rep(1,ncols)
  heights <- c(rep(1, nrows), 0.5)
  pages.per.target <- ifelse(length(plot.models)%%nrows==0, length(plot.models)%/%nrows, length(plot.models)%/%nrows+1)
  pages.per.model <- ifelse(length(plot.targets)%%ncols==0, length(plot.targets)%/%ncols, length(plot.targets)%/%ncols+1)
  npages <- pages.per.model*pages.per.target
  color.palette <- c("white", embers2(8)[c(4,7)], embers(9)[3:9])
  for(page in 1:npages) {
    pdf(paste(basepath, "analysis/gamma_maps_feb2013_",formatC(page, flag="0", width=max(nchar(npages),2)), "_v2.pdf", sep=""), height=9, width=9/sum(heights)*2.45*ncols, pointsize=8, paper="special", onefile=FALSE)
    par(oma=c(0,1,1,0))
    layout(map.layout, widths, heights)
    page.targets <- 1:ncols+(page-1)%/%pages.per.target*ncols
    page.targets <- page.targets[which(page.targets<=length(plot.targets))]
    page.targets <- plot.targets[page.targets]
    page.models <- (1:nrows)+(ifelse(page%%pages.per.target-1>=0, page%%pages.per.target-1, pages.per.target-1)*nrows)
    page.models <- page.models[which(page.models<=length(plot.models))]
    page.models <- plot.models[page.models]
    print(page.models)
    print(page.targets)
    for(t in 1:ncols) {
      for(m in 1:nrows) {
        par(mar=c(0,0,0,0)+0.1)
        if(t>length(page.targets) || m>length(page.models)) {
          plot(1,1,type="n", axes=FALSE, xlab="", ylab="")
          next
        }
        plotraster <- raster(ncols=720, nrows=360)
        plotraster[cellFromXY(plotraster, t(grid.data))] <- data.array[,page.models[m], page.targets[t]]
        image(plotraster, zlim=c(0,1), axes=FALSE, xlim=c(-180,180), ylim=c(-60,85), xaxs="i", yaxs="i", asp=1, col=color.palette, main="", breaks=seq(0,1,by=0.1))
        plot(world.borders.simple, add=TRUE, lwd=0.25)
        box()
        if(m==1) {
          mtext(paste(target.temps[page.targets[t]]), side=3, outer=TRUE, line=0, at=(t-0.5)/sum(widths), cex=0.75)
        }
        if(t==1) {
          rect(grconvertX(0, from = "npc", to = "user")-strheight("A")*1.9,grconvertY(1, from = "npc", to = "user"),grconvertX(0, from = "npc", to = "user"),grconvertY(0, from = "npc", to = "user"), density=NA, col=col.scale2[page.models[m]], xpd=NA, border="black", lwd=1)
          mtext(paste(model.names.official[page.models[m]]), side=2, outer=TRUE, line=-0.05, at=(sum(heights)-m+0.5)/sum(heights), cex=0.75, font=2)
        }
      }
    }
    plot.new()
    image.plot(zlim=c(0,1), col=color.palette, nlevel=255, horizontal=TRUE, smallplot=c(0.25,0.75,0.7-0.2, 0.9-0.1), legend.only=TRUE, axis.args=list(at=c(0,0.1,0.3,1)), legend.lab=expression(paste(Gamma)), breaks=seq(0,1,by=0.1))
    dev.off()
  }
}

if(FALSE) {
  ## map of managed land
  # figure S1 (supporting information)
  plotraster <- raster(ncols=720, nrows=360)
  plotraster[cellFromXY(plotraster, t(grid.data))] <- 1-natveg.frac
  plotraster[which(plotraster[]==0)] <- NA
  setEPS()
  postscript(paste(basepath, "analysis/managed_lands_mask_may2013.eps", sep=""), width=503.61368/72, height=503.61368/72/2.25, pointsize=6, paper="special")
  landpalette <- colorRampPalette(c("white",brewer.pal(9, "YlOrBr")), bias=1.25)
  par(mar=c(3,0,0,0)+0.1)
  image(plotraster, xlim=c(-180,180), ylim=c(-60,85), zlim=c(0.0005,1),
        axes=FALSE, xaxs="i", yaxs="i", xlab="", ylab="",main="", asp=1,
        col=landpalette(285)[-c(1:30)])
  plot(world.borders.simple, add=TRUE, lwd=0.25)
  mtext("managed land fraction", side=1, line=1.89, at=90, adj=0)
  image.plot(zlim=c(0.0005,1), col=c("white", landpalette(285)[-c(1:30)]), nlevel=255, breaks=c(0,seq(0.01,1,length.out=256)), horizontal=TRUE, legend.only=TRUE, smallplot=c(0.22,0.72,0.07,0.11), axis.args=list(lwd=0.25), legend.args=list(lwd=0.25))
  dev.off()
}

if(FALSE) {
  # figure 5 (main text)
  twocolumn <- FALSE
  GCM.legend <- FALSE
  onecolumn.height <- ifelse(GCM.legend,4, 2.1)
  setEPS()
  postscript(paste(basepath, "analysis/area_affected_individual_GCMs_sep2013_",ifelse(twocolumn,"2col", "1col"),ifelse(GCM.legend,"_GCMs", ""),".eps", sep=""), width=ifelse(twocolumn,503.61368/72,241.84842/72),height=ifelse(twocolumn,3.25,onecolumn.height), paper="special", pointsize=8)#, family="serif"
  if(twocolumn) {
    par(mar=c(4-2,3.5-0.5,0,10.5)+0.1, mgp=c(2.25,1,0))
  } else {
    par(mar=c(4-2,3.5-0.5,0,0)+0.1, fig=c(0,1,ifelse(GCM.legend,1.7,0.4)/onecolumn.height,1), mgp=c(2.25,1,0))
  }
  plot(1,1, type="n", axes=FALSE, xlim=c(0.75, length(target.temps)+0.25), ylim=c(0,1.05), xlab="", ylab="Fraction of ice-free land area at risk", xaxs="i", yaxs="i", cex.lab=0.75)
  #GMT increase [K] by around 2100
  lty <- 1
  col.scale <- colorRampPalette(c("black", "grey50"))
  for(t in dim(frac.severe.risk)[3]:1) {
    for(m in 1:dim(frac.severe.risk)[1]) {
      lines(1:length(target.temps), frac.severe.risk[m,,t], lty=1, lwd=0.25, col=col.scale2[m])
      points(1:length(target.temps), frac.severe.risk[m,,t], lty=1, lwd=0.25, pch=lty, cex=0.5)
    }
    lty <- lty+1
  }
  lty <- 1
  for(t in dim(frac.severe.risk)[3]:1) {
    lines(1:length(target.temps), area.severe.additiverisk[2,,t]/sum(grid.area*natveg.frac), lty=lty, lwd=1)#, col=col.scale(dim(frac.severe.risk)[3])[lty])
    lty <- lty+2
  }  
  axis(2, at=pretty(c(0,1),5), labels=pretty(c(0,1),5), las=1, cex.axis=0.75)
  axis(1, at=1:length(target.temps), labels=gsub("K", "K", target.temps, fixed=TRUE), cex.axis=0.75)
  if(twocolumn) {
    legend.x <- c(8.25, 8.25, 8.25)
    legend.y <- c(1, 0.15, 0)
    legend.ncol <- 1
    rect(8.25,-0.15, 10.25,1, col="white", density=NA, xpd=NA, border="black")
    lines(c(8.25,10.25), rep(0.15,2), xpd=NA)
    lines(c(8.25,10.25), rep(0,2), xpd=NA)
    textwidth <- NULL
    title.adj=0.1
  } else {
    par(fig=c(0,1,0,ifelse(GCM.legend,1.70, 0.4)/onecolumn.height), new=TRUE, mar=c(0,3.5-0.5,0,0)+0.1)
    plot(1,1, type="n", axes=FALSE, xlim=c(0,1), ylim=c(0,1), xaxs="i", yaxs="i", xlab="", ylab="", main="")
    rect(0.025, 1, 0.975,0.005)
    legend.x <- c(0,0,0.45)+0.025
    legend.y <- c(0.75, 1, 1)
    legend.ncol <- 2
    textwidth <- strwidth("moderate or major change", cex=0.75)
    title.adj=0.025
  }
  if(GCM.legend) {
    legend(x=legend.x[1], y=legend.y[1], legend=model.names.official, lwd=0.5*2, col=col.scale2[1:length(models)], xpd=NA, title="GCMs", title.adj=title.adj, bty="n", cex=0.75, y.intersp=1.05, ncol=legend.ncol, text.width=textwidth)
  }
  legend(x=legend.x[2], y=legend.y[2], legend=c("major change", "moderate or major change"), pch=c(1,2), pt.cex=0.5, pt.lwd=0.25, title="area in individual GCMs", title.adj=0.1, seg.len=1, bty="n", xpd=NA, cex=0.75, y.intersp=1, text.width=textwidth)
  legend(x=legend.x[3], y=legend.y[3], legend=c("major change", "moderate or major change"), lty=c(1,3), lwd=1, title="area across all GCMs", title.adj=0.1, seg.len=1.5, bty="n", xpd=NA, cex=0.75, y.intersp=1, text.width=textwidth)
  par(mgp=c(3,1,0), new=TRUE)
  dev.off()
}

## don't do automatically


if(FALSE) {
	## Absolute changes
	# figure 2 (main text)
	temp.levels <- 1:length(target.temps)
	threshold.loop <- c(0.1, 0.3)
	#threshold.loop <- c(0.1)
	data.subarray <- data.array[,,temp.levels]
	model.subsevere <- array(dim=c(NCELL, length(temp.levels), length(threshold.loop)))
	for(t in 1:length(threshold.loop)) {
		data.subsevere <- array(0, dim=dim(data.subarray))
		if(t < length(threshold.loop)) {
			data.subsevere[which(data.subarray > threshold.loop[t] & data.subarray <= threshold.loop[t+1])] <- 1
		} else {
			data.subsevere[which(data.subarray > threshold.loop[t])] <- 1
		}
		model.subsevere[,,t] <- apply(data.subsevere, c(1,3), sum, na.rm=TRUE)/apply(data.subarray, c(1,3), num.valid)
	}
	
	
	# correction for plotting
	model.subsevere.bak <- model.subsevere
	for(t in (length(threshold.loop)-1):1) {
		model.subsevere[,,t] <- ifelse(model.subsevere[,,t] > model.subsevere[,,t+1], model.subsevere[,,t]+model.subsevere[,,t+1], model.subsevere[,,t])
		model.subsevere[,,t+1] <- ifelse(model.subsevere[,,t] > model.subsevere[,,t+1] & model.subsevere[,,t+1] < 0.33, 0, model.subsevere[,,t+1])
	}
	model.subsevere[which(model.subsevere==0)] <- NA
	plot.levels <- c(0.33, 0.66, 0.9, 1)
	plotraster <- raster(ncols=720, nrows=360)
	colored.humanmask <- FALSE

  ## interpolated color scale
  embers_mod <- colorRampPalette(c("white", brewer.pal(9, "YlOrRd")[-8:-9]), bias=0.9)
  embers2_mod <- colorRampPalette(c("white", brewer.pal(9, "Blues")[-8:-9]), bias=1.1)
  cols_moderate <- embers2_mod(length(risk.levels)+5)[-2:-6]
	cols_severe = embers_mod(length(risk.levels)+2)[-2:-3]
  cols_diag <- colorRampPalette(c(cols_severe[length(cols_severe)], cols_moderate[length(cols_moderate)]))(length(risk.levels))
  cols_diag2 <- colorRampPalette(c(cols_severe[length(cols_severe)], brewer.pal(7, "Greens")[4], cols_moderate[length(cols_moderate)]))(length(risk.levels))
  interpol_cols <- array(dim=c(length(risk.levels), length(risk.levels)))
  interpol_cols[1,] <- cols_moderate
  interpol_cols2 <- interpol_cols

  for(agreement in 2:length(risk.levels)) {
    interpol_cols[agreement,1:(length(risk.levels)+1-agreement)] <- colorRampPalette(c(cols_severe[agreement], cols_diag[length(risk.levels)+1-agreement]))(length(risk.levels)+1-agreement)
    interpol_cols2[agreement,1:(length(risk.levels)+1-agreement)] <- colorRampPalette(c(cols_severe[agreement], cols_diag2[length(risk.levels)+1-agreement]))(length(risk.levels)+1-agreement)
  }
  model.subsevere.2d <- model.subsevere.bak * (length(risk.levels)-1)
  if(max(model.subsevere.2d, na.rm=T)> (length(risk.levels)-1)) {
    stop("Error: number of models is not identical for all GMT levels")
  }
  model.subsevere.2d <- model.subsevere.2d[,,1]*length(risk.levels)+model.subsevere.2d[,,2]+1


	
	target.names <- c("", "2 degree target", "","", "current emissions reduction pledges (3.5 K warming)", "", "", "business-as-usual emissions growth (5 K warming)")
	setEPS()
	postscript(paste(basepath, "analysis/change_maps_sep2013_2dscale.eps", sep=""), width=503.61368/72, height=3.5, pointsize=8, paper="special")
	layout(matrix(1:4, ncol=2, byrow=TRUE), widths=c(1,1), heights=c(1,1))
	for(scale in 2) {
	par(mar=c(1.25,0,1.75,0)+0.1, pty="m")
	fignum <- 1
	for(t in c(2,5,8)) {
		heading <- paste("Patterns of ecosystem change risk")
		heading2 <- paste(letters[fignum], ") ", target.names[temp.levels[t]], sep="")
		plot(world.borders.simple, col="white", border=NA, ylim=c(-60,83), xlim=c(-180, 180), asp=1, xaxs="i", yaxs="i", axes=FALSE)
		title(main=heading2, cex.main=1.6*par("cex"), line=0.75, adj=0)
		fignum <- fignum+1
		plotraster[] <- NA
    plotraster[cellFromXY(plotraster, t(grid.data))] <- model.subsevere.2d[,t]
    if(scale==1)
      cols <- c(interpol_cols)	
    if(scale==2)
      cols <- c(interpol_cols2)	
    cols[which(is.na(cols))] <- "#000000"
		image(plotraster, add=TRUE, zlim=c(0.5, length(cols)+0.5), col=cols, axes=FALSE)

		plot(world.borders.simple, add=TRUE, lwd=0.25, axes=FALSE)
	}
  par(mar=c(3,5,2,0)+0.1, pty="s")
  plot(1,1, type="n", xlim=c(-0.5,length(risk.levels)-0.5), ylim=c(-0.5,length(risk.levels)-0.5), xaxt="n", yaxt="n", asp=1, axes=FALSE, adj=0, xlab="", ylab="")
  axis(side=1,at=seq(1,length(models), length.out=4))
  mtext(side=1, line=1, at=length(models)+1, text=paste(" of", length(models)), adj=0, cex=0.9)
  mtext(side=1, line=2, text="agreement on major change", cex=0.9)
  mtext(side=2, line=4.5, text="agreement on moderate change", cex=0.9)
  axis(side=2,at=seq(1,length(models), length.out=4), labels=paste(seq(1,length(models), length.out=4), "of", length(models)), las=1)
  dummy.array <- array(dim=c(length(risk.levels), length(risk.levels)))
  for(c in 1:length(risk.levels)) {
    for(r in 1:length(risk.levels)-1) {
      dummy.array[c,r+1] <- r*length(risk.levels)+c
    }
  }
  cols[which(cols=="#000000")] <- NA
  image(x=seq(0, length(risk.levels)-1), y=seq(0, length(risk.levels)-1), z <- dummy.array, add=TRUE, zlim=c(0.5, length(cols)+0.5), col=cols)
	} # end color scales
  dev.off()
}

stop("before biome classes")

if(FALSE) {
  ## biome class map
  # figure S3 (supporting information)
  human.color <- "grey60"
  add.legend <- TRUE
  add.modis <- TRUE
  legend.height <- 1.2
  figwidth <- 503.61368/72 #7
  figheight <- ifelse(add.modis, 2, 1)*figwidth*150/360+ifelse(add.legend, legend.height, 0)

  panel.height <- c(seq(ifelse(add.legend, legend.height, 0), figheight, length.out=ifelse(add.modis, 3, 2)))/figheight
  if(add.legend) panel.height <- c(0, panel.height)
  panel.bottom <- panel.height[-length(panel.height)]
  panel.top <- panel.height[-1]
  panel.top <- c(panel.top[1], rev(panel.top[-1]))
  panel.bottom <- c(panel.bottom[1], rev(panel.bottom[-1]))

  if(add.modis) {
    # modis-derived biome class map
    biome.modis <- as.integer(readBin(paste(basepath, "analysis/modis.biomeclass.bin", sep=""), raw(), n=NCELL))
  }
  
  setEPS()
  postscript(paste(basepath, "analysis/biome_classification/biome_class_present_", ifelse(add.modis, "modis_",""), "may2013.eps", sep=""), width=figwidth, height=figheight, pointsize=8, paper="special")
  par(fig=c(0,1, panel.bottom[ifelse(add.legend,2,1)], panel.top[ifelse(add.legend,2,1)]), mar=c(0,0,ifelse(add.modis, 1, 0),0)+0.2)
  plotraster <- raster(ncols=720, nrows=360)
  plotraster[cellFromXY(plotraster, t(grid.data))] <- biome.lpj
  plot(1,1, type="n", asp=1, axes=FALSE, ylab="", xlab="", xlim=c(-180,180), ylim=c(-60,85), xaxs="i", yaxs="i")
  image(plotraster, zlim=c(0.5, length(biome.class.cols.v2)+0.5), col=biome.class.cols.v2, add=TRUE)
  if(add.modis) {
    mtext("a) LPJmL-derived biomes", side=3, line=0, font=2, adj=0)
  }
  plotraster[] <- NA
  plotraster[cellFromXY(plotraster, t(grid.data[,which(natveg.frac<0.2)]))] <- 1
  image(plotraster, col=human.color, zlim=c(0.5,1.5), add=TRUE, bg="transparent", useRaster=FALSE)
  plot(world.borders.simple, lwd=0.25, add=TRUE)
  
  if(add.modis) {
    par(fig=c(0,1, panel.bottom[ifelse(add.legend,3,2)], panel.top[ifelse(add.legend,3,2)]), mar=c(0,0,ifelse(add.modis, 1, 0),0)+0.2, new=TRUE)
    plotraster <- raster(ncols=720, nrows=360)
    plotraster[cellFromXY(plotraster, t(grid.data))] <- biome.modis
    plot(1,1, type="n", asp=1, axes=FALSE, ylab="", xlab="", xlim=c(-180,180), ylim=c(-60,85), xaxs="i", yaxs="i")
    image(plotraster, zlim=c(0.5, length(biome.class.cols.v2)+0.5), col=biome.class.cols.v2, add=TRUE)
    if(add.modis) {
      mtext("b) MODIS-derived biomes", side=3, line=0, font=2, adj=0)
    }
    plotraster[] <- NA
    plotraster[cellFromXY(plotraster, t(grid.data[,which(natveg.frac<0.2)]))] <- 1
    image(plotraster, col=human.color, zlim=c(0.5,1.5), add=TRUE, bg="transparent", useRaster=FALSE)
    plot(world.borders.simple, lwd=0.25, add=TRUE)
  }
  if(add.legend) {
    par(fig=c(0,1, panel.bottom[1], panel.top[1]), mar=c(0,0,0,0)+0.2, new=TRUE)
    plot(1,1, type="n", axes=FALSE, xlab="", ylab="", main="")
    ## legend new:
    legend.text <- c(biome.class.names.lpj, "Human-dominated", "")
    legend.cols <- c(biome.class.cols.v2, "white", NA, human.color, NA)
    legend.order <- c(1,2,9,10,11,16, length(biome.class.names.lpj)+2, # first column tropical
                      3,4,5,6,12,13,14, # 2nd column temperate
                      7,8,15, 17, length(biome.class.names.lpj)+2, length(biome.class.names.lpj)+1)
    legend(x="topleft", legend=legend.text[legend.order], xpd=NA, ncol=3, pt.cex=2, pch=ifelse(is.na(legend.cols[legend.order]),NA,22), pt.bg=legend.cols[legend.order], y.intersp=1.2, bty="n")
  }
  dev.off()
  
}


## test for biome classes
#biome.classes <- sort(unique(biome.lpj))
biome.classes <- plot.classes
biome.table <- array(dim=c(length(biome.classes), length(target.temps), length(threshold.loop)))
biome.additive <- array(dim=dim(biome.table))
biome.absolute <- array(dim=dim(biome.table))

dimnames(biome.table)[[1]] <- biome.class.names.lpj[biome.classes]
dimnames(biome.table)[[2]] <- target.temps
dimnames(biome.additive) <- dimnames(biome.table)
biome.area.severe.additiverisk <- array(dim=c(length(biome.classes),length(risk.levels), length(target.temps), length(threshold.loop)))
biome.area.severe.risk <- array(dim=c(length(biome.classes),length(risk.levels), length(target.temps), length(threshold.loop)))
biome.area.total <- numeric(length(biome.classes))

for(k in 1:length(threshold.loop)) {
	for(j in 1:length(target.temps)) {
		for(i in 1:length(biome.classes)) {
			affected.cells <- which(model.severe[,j,k] > 0.66 & biome.lpj == biome.classes[i])
			total.cells <- which(biome.lpj==biome.classes[i])
			biome.table[i,j,k] <- sum((grid.area*natveg.frac)[affected.cells], na.rm=TRUE)/sum((grid.area*natveg.frac)[total.cells], na.rm=TRUE)
			if(i>1) {
				biome.additive[i,j,k] <- biome.additive[i-1,j,k] + sum((grid.area*natveg.frac)[affected.cells]/1e12, na.rm=TRUE)
			} else {
				biome.additive[i,j,k] <- sum((grid.area*natveg.frac)[affected.cells]/1e12, na.rm=TRUE)
			}
			biome.absolute[i,j,k] <- sum((grid.area*natveg.frac)[affected.cells]/1e12, na.rm=TRUE)
		
			biome.area <- (grid.area*natveg.frac)[which(biome.lpj==biome.classes[i])]
			biome.severe <- model.severe[which(biome.lpj==biome.classes[i]),,]

			if(length(threshold.loop)==1) {
				dim(biome.severe) <- c(dim(biome.severe),1)
			}
			for(risk in 2:length(risk.levels)) {
				biome.area.severe.additiverisk[i,risk,j,k] <- sum(biome.area[which(biome.severe[,j,k] > risk.levels[risk-1]*1.00001)], na.rm=TRUE)/sum(biome.area, na.rm=T)
				biome.area.severe.risk[i,risk,j,k] <- sum(biome.area[which(biome.severe[,j,k] <= risk.levels[risk]*1.00001 & biome.severe[,j,k] > risk.levels[risk-1]*1.00001)], na.rm=TRUE)/sum(biome.area)
				biome.area.total[i] <- sum(biome.area)
			}
		}
	}
}
dimnames(biome.area.severe.additiverisk)[[1]] <- biome.class.names.lpj[biome.classes]#.combined
dimnames(biome.area.severe.additiverisk)[[2]] <- risk.levels
dimnames(biome.area.severe.additiverisk)[[3]] <- target.temps
dimnames(biome.area.severe.additiverisk)[[4]] <- threshold.loop



##########################
## Affected area by biome
##########################
# figure S6 (supporting information)
setEPS()
postscript(paste(basepath, "biome_classes_modfire_bars_v15_newmetric_sep2013.eps", sep=""), width=660/72, height=2*length(threshold.loop)+1.5+0.25, pointsize=8, paper="special")
counter <- 1
for(k in length(threshold.loop):1) {
	#display.sequence <- c(1:7,9,11,12,14,15,16)
	display.sequence <- c(1:16)
	par(mar=c(3-1,4,3,0.5)+0.1, fig=c(0,1,1-counter/(length(threshold.loop)+0.875),1-(counter-1)/(length(threshold.loop)+0.875)), new=ifelse(counter>1,TRUE, FALSE))
	print(par("fig"))
	for(b in 1:length(display.sequence)) {
		x.offset <- (b-1) * (length(target.temps)+2)
		space <- c(x.offset/0.95, rep(1/19, times=length(target.temps)-1))
		#		print(x.offset)
		xlim <- c(3, (length(target.temps)+1.5)*length(display.sequence)) 
		barplot(mirror(biome.area.severe.risk[display.sequence[b], -1, , k], 1)*100, width=0.95, space=space, offset=0, xlim=xlim, col=rev(embers(length(risk.levels)+2)[-1:-3]), add=ifelse(b==1, FALSE, TRUE), ylim=c(0,100), axes=ifelse(b==1, TRUE, FALSE), border=NA, las=1)
		if(k!=1) {
      mtext(at=x.offset+c(1,length(target.temps)-0.95), side=1, text=target.temps[c(1,length(target.temps))], cex=0.75, line=0)
    }
		for(t in 1:length(target.temps)) {
		  lines(x=x.offset+t-c(1, 0.05), y=rep(biome.area.severe.additiverisk[display.sequence[b], 2, t, k]*100, 2), lwd=0.5)
		  if(t==1) {
		    lines(x=rep(x.offset+t-1, 2), y=c(0, biome.area.severe.additiverisk[display.sequence[b], 2, t, k]*100), lwd=0.5)
		  } else {
		    lines(x=rep(x.offset+t-1, 2), y=c(biome.area.severe.additiverisk[display.sequence[b], 2, t-1, k]*100, biome.area.severe.additiverisk[display.sequence[b], 2, t, k]*100), lwd=0.5)
		  }
		  if(t==length(target.temps)) {
		    lines(x=rep(x.offset+t-0.05, 2), y=c(biome.area.severe.additiverisk[display.sequence[b], 2, t, k]*100, 0), lwd=0.5)
		  }
		}
		if(k==1) {
			text(x=x.offset+length(target.temps)/2+3, y=-5, biome.class.names.lpj[display.sequence[b]], srt=40, xpd=NA, pos=2, cex=1*0.75)#.combined
		}
		lines(seq(x.offset+0.475, length.out=length(target.temps)), biome.area.severe.additiverisk[display.sequence[b],low.conf,,k]*100, lwd=1, lty=3)
		lines(seq(x.offset+0.475, length.out=length(target.temps)), biome.area.severe.additiverisk[display.sequence[b],med.conf,,k]*100, lwd=1.5)
		lines(seq(x.offset+0.475, length.out=length(target.temps)), biome.area.severe.additiverisk[display.sequence[b],high.conf,,k]*100, lwd=1.5, col="white")
		if(counter==1) {
			text(x.offset+length(target.temps)/2+0.4, 98, round(biome.area.total[display.sequence[b]]*1e-12,2), pos=3, xpd=NA, cex=0.75) 
		}
	}
	title(ylab="Fraction of biome area affected [%]", cex.lab=0.75, cex.axis=0.75)
	text(x=-12.5, y=120, labels=paste(letters[counter],") ",ifelse(threshold.loop[k]>=threshold.severe, "Major", ifelse(threshold.loop[k]>=threshold.moderate, ifelse(threshold.closed, "Moderate", "Moderate or major"), "Small")), "ecosystem change across biomes"), cex.main=1, font=2, xpd=NA, pos=4)
	counter <- counter+1
	if(k==1) {
	  par(fig=c(0,1,0,0.10), new=TRUE)
	  image.plot(zlim=c(0.5, length(models)+0.5), col=embers(length(risk.levels)+2)[-1:-3], legend.only=TRUE, axis.args=list(at=c(1,pretty(c(1,length(models))),length(models)), cex.axis=1), nlevel=255,
	             smallplot=c(0.33, 0.66, 0.8, 1), horizontal=TRUE, cex=0.75, legend.mar=4, legend.args=list(cex=1, text="Number of climate models in agreement", line=-4))#, legend.lab="Number of climate models in agreement"
	}
}
dev.off()


## group some biomes, just biomes in fig.
biome.area.severe.additiverisk.grouped <- biome.area.severe.additiverisk
# combine warm savannas
biome.area.severe.additiverisk.grouped[9:10,,,] <- rep(apply(biome.area.severe.additiverisk[9:10,,,]*biome.area.total[9:10], c(2,3,4), sum)/sum(biome.area.total[9:10]), each=2)
# combine temperate savannas
biome.area.severe.additiverisk.grouped[12:13,,,] <- rep(apply(biome.area.severe.additiverisk[12:13,,,]*biome.area.total[12:13], c(2,3,4), sum)/sum(biome.area.total[12:13]), each=2)
# combine temperate summergreen and mixed forest
biome.area.severe.additiverisk.grouped[4:5,,,] <- rep(apply(biome.area.severe.additiverisk[4:5,,,]*biome.area.total[4:5], c(2,3,4), sum)/sum(biome.area.total[12:13]), each=2)

## figure 3 (main text)
setEPS()
linestyles <- rep(1, length(biome.classes))
figheight <- 1.8
figwidth <- 503.61368/72#7
postscript(paste(basepath, "biome_classes_lines_sep2013.eps", sep=""), width=figwidth, height=figheight, pointsize=8, paper="special")
#panel.top <- c(1.8-0.4, figheight)/figheight
#panel.bottom <- c(0, panel.top[-length(panel.top)])
panel.left <- c(0, seq(0.4/figwidth, 1-(2)/figwidth, length.out=length(threshold.loop)+1),1)
panel.right <- c(panel.left[-1],1)

# Severe and moderate change by biomes
for(k in 1:length(threshold.loop)) {
	par(fig=c(panel.left[length(threshold.loop)+2-k], panel.right[length(threshold.loop)+2-k], 0,1), new=ifelse(k>1, TRUE, FALSE), mar=c(2.1,0,1.35,0))
  plot(1,1, type="n", xlim=c(1-ifelse(k==length(threshold.loop),0.25,0.4),length(target.temps)+ifelse(k==1,0.25,0.4)), ylim=c(0,1.05), yaxs="i", xaxs="i", xaxt="n", yaxt="n", xlab="", ylab="", main="", las=2, cex.axis=0.75)
  axis(side=1, at=1:length(target.temps), labels=target.temps, cex.axis=0.75)
  title <- paste(letters[length(threshold.loop)+1-k],") ",ifelse(threshold.loop[k]>=threshold.severe, "Major", ifelse(threshold.loop[k]>=threshold.moderate, ifelse(threshold.closed, "Moderate", "Moderate or major"), "Small")), "change")#, at least medium confidence")
  mtext(text=title, side=3, line=0.25, cex=1.1, adj=0, font=2)#, at=-1+strwidth(title)/2
  if(k==length(threshold.loop)) {
		mtext("Fraction of biome area affected", side=2, line=2, cex=0.75, xpd=NA)
		axis(side=2, cex.axis=0.75, las=2, xpd=NA)
	}
  for(b in 1:length(biome.classes)) {
    if(b %in% c(4,10,13)) next
    lines(1:length(target.temps), biome.area.severe.additiverisk[b,med.conf, ,k], lty=linestyles[b], col=biome.class.cols.v2[b], lwd=1.5)
  }
  box()
}

# legend biomes
par(fig=c(panel.left[length(threshold.loop)+2],panel.right[length(threshold.loop)+2],0,1), mar=c(0,0.25,1,0), new=TRUE)
plot(1,1,type="n", axes=FALSE, xlab="", ylab="", main="", xlim=c(0,1), ylim=c(0,1),xaxs="i", yaxs="i",)
legend.text <- c(biome.class.names.combined[biome.classes], "")
legend.text[c(7,8)] <- biome.class.names.lpj[c(7,8)]
legend.text[5] <- "Temperate Summergreen & Mixed Forest"
legend.cols <- c(biome.class.cols.v2, NA)
legend.order <- c(1,2,c(9,11),16,17,c(3,5,6),c(12,14),17,7,8,15)
legend("topleft", legend=unlist(lapply(strwrap(legend.text[legend.order], 30+20, simplify = F), paste, sep="", collapse="\n")), lty=linestyles, col=legend.cols[legend.order], ncol=1, cex=0.75, bty="n", lwd=1.5, seg.len=3, y.intersp=1.0, title="Biome classes", title.adj=0.05)

dev.off()

#########################
## copied from biome_diff_gamma.r
#########################
# figure S2 (supporting information)
gamma.vars <- list("combined" = "cm_scaled.bin",
									 "deltav"="deltav.bin",
									 "cfluxes"="cfluxes_comb_scaled.bin",
									 "cstocks"="cstocks_comb_scaled.bin",
									 "wfluxes"="wfluxes_comb_scaled.bin",
									 "global"="all_mag_g_scaled.bin",
									 "local"="all_mag_scaled.bin",
									 "balance"="all_1-alpha.bin"
									 )

plot.gamma.table <- TRUE
if(plot.gamma.table) {
	for(metric in names(gamma.vars)[1]) {
		biome.gamma <- array(dim=c(length(plot.classes), length(plot.classes)))
		dimnames(biome.gamma)[[2]] <- biome.class.names.lpj[plot.classes]
		cell.names <- biome.class.names.lpj[plot.classes]
		for(b in 1:length(plot.classes)) {
			input.filename <- paste(basepath, reference.path, "biome", plot.classes[b], "/", metric.path, gamma.vars[[metric]], sep="")
			print(paste("Loading data from", input.filename))
			filedata <- readBin(input.filename, double(), size=4, n=ifelse(file.info(input.filename)$size/4>length(plot.classes), length(plot.classes)*2, length(plot.classes)))
			if(length(filedata)>length(plot.classes)) {
				biome.gamma[b,] <- filedata[1:length(plot.classes)+length(plot.classes)]
			} else {
				biome.gamma[b,] <- filedata
			}
			cell.names <- c(cell.names[-1], cell.names[1])
			names(biome.gamma[b,]) <- paste(biome.class.names.lpj[plot.classes],"->", cell.names, sep="")
			if(metric %in% c("local", "global", "balance")) {
				if(metric=="balance") {
					ratio.filename <- gsub("1-alpha", "alpha_logistic", input.filename, fixed=TRUE)
					biome.gamma[b,] <- biome.gamma[b,]*2
					biome.gamma[which(biome.gamma>1)] <- 1
				} else {
					ratio.filename <- gsub("_scaled", "_logistic", input.filename, fixed=TRUE)
				}
				print(paste("Loading data from", ratio.filename))
				filedata <- readBin(ratio.filename, double(), size=4, n=length(plot.classes))
				biome.gamma[b,] <- biome.gamma[b,]*filedata
			}
		}

		biome.table <- array(dim=dim(biome.gamma))
		biome.table[,1] <- biome.gamma[,1]
		for(b in 2:length(plot.classes)) {
			shift <- sort(seq(length(plot.classes), by=-1, length.out=b-1))
			biome.table[,b] <- c(biome.gamma[shift,b], biome.gamma[-shift,b])
		}
		setEPS()
		postscript(paste(basepath, "analysis/biome_gamma_table_", metric, "_feb2013_shaded.eps", sep=""), height=4, width=5, pointsize=6, paper="special")
		par(mar=c(1-0.9,9+1,8,1-0.5)+0.1)
		plot(1,1, type="n", xlim=c(0.5, length(plot.classes)+0.5), ylim=c(0.5, length(plot.classes)+0.5), axes=FALSE, xlab="", ylab="", xaxs="i", yaxs="i")
		axis(2, at=rev(seq(1, length(plot.classes))), labels=FALSE)
		text(x=0.25, y=rev(seq(1, length(plot.classes))), labels=unlist(lapply(strwrap(biome.class.names.lpj[plot.classes], 25, simplify = F), paste, sep="", collapse="\n")), cex=0.75, xpd=NA, pos=2, srt=360)
		axis(3, at=seq(1, length(plot.classes)), labels=FALSE)
		text(x=seq(1, length(plot.classes)), y=rep(length(plot.classes)+1, length(plot.classes)), unlist(lapply(strwrap(biome.class.names.lpj[plot.classes], 25, simplify = F), paste, sep="", collapse="\n")), cex=0.75, srt=45, xpd=NA, adj=0)
		box()
		for(b in 1:length(plot.classes)) {
  		cols <- ifelse(biome.table[,b]>0.3, rep("grey60", length(plot.classes)), ifelse(biome.table[,b]>0.1, rep("grey80", length(plot.classes)), rep(NA, length(plot.classes))))
  		cols <- paste("grey", round(100-biome.table[,b]*100), sep="")
  		text.cols <- ifelse(biome.table[,b]>0.6, rep("white", length(plot.classes)), rep("black", length(plot.classes)))
			rect(xleft=b-0.5, ybottom=seq(length(plot.classes), 1, by=-1)-0.5, xright=b+0.5, ytop=seq(length(plot.classes), 1, by=-1)+0.5, col=cols, border=NA)
			text(x=b, y=seq(length(plot.classes), 1, by=-1), labels=round(biome.table[,b],2), cex=0.75, col=text.cols)
		}
		abline(h=seq(1.5, length(plot.classes)), v=seq(1.5, length(plot.classes)), lwd=0.5)
		title(main="Shift from", line=7, cex.main=1)
		title(ylab="Shift to", line=8+1, cex.lab=1)
		metric.title <- switch(metric, 
													 combined=expression(paste("Combined ", Gamma, " metric")),
													 deltav=expression(paste(Delta,"V component")),
													 cfluxes="Carbon fluxes subset",
													 cstocks="Carbon stocks subset",
													 wfluxes="Water fluxes subset",
													 global="Global importance component",
													 local="Local change component",
													 balance="Balance component"
													 )

		mtext(text=metric.title, side=3, line=6, at=-3.5, adj=0, cex=1.5)
		dev.off()
	}
}

