#####################################################################################
## Aggregate MODIS LC to LPJ resolution and transscribe to LPJ PFTs
## Used for Fig. S3

rm(list=ls(all=TRUE))
library(fields)
library(raster)
library(maps)

### filesystem details (paths) removed ###
basepath <- c("***")
modispath <- c("***")
modispath2 <- c("***")
climate.scen.path <- "***"
lpj.input.path <- "***"

reference.path <- "***"

modis.res <- 0.05
lpj.res   <- 0.5

modis.classes <- c("Water",                               # 1
									 "Evergreen Needleleaf Forest",         # 2
									 "Evergreen Broadleaf Forest",          # 3
									 "Deciduous Needleleaf Forest",         # 4
									 "Deciduous Broadleaf Forest",          # 5
									 "Mixed Forests",                       # 6
									 "Closed Shrublands",                   # 7
									 "Open Shrublands",                     # 8
									 "Woody Savannas",                      # 9
									 "Savannas",                            #10
									 "Grasslands",                          #11
									 "Permanent Wetlands",                  #12
									 "Croplands",                           #13
									 "Urban and Built-Up",                  #14
									 "Cropland/Natural Vegetation Mosaic",  #15
									 "Snow and Ice",                        #16
									 "Barren or Sparsely Vegetated",        #17
									 "(IGBP Water Bodies, recoded to 0 for MODIS Land Product consistency)"
									 )

modisname <- paste(modispath, "MODIS_LC_2004.bsq", sep="")
#modis.years <- 2004
modis.years <- c(2001:2004)


modis.treefrac <- c(0,    # 1
										0.95, # 2
										0.95, # 3
										0.95, # 4
										0.95, # 5
										0.95, # 6
										0.8,  # 7
										0.05,  # 8,
										0.5,  # 9
										0.1,  #10
										0,    #11
										0,    #12
										0,    #13
										0,    #14
										0.2,  #15
										0,    #16
										0,    #17
										0     #18
										)

modis.grassfrac <- 1-modis.treefrac
modis.grassfrac[which(modis.classes=="Barren or Sparsely Vegetated")] <- 0.025
modis.grassfrac[which(modis.classes=="Snow and Ice" | modis.classes=="Croplands" | modis.classes=="Urban and Built-Up" | modis.classes=="Water" | modis.classes=="Cropland/Natural Vegetation Mosaic" | modis.classes=="(IGBP Water Bodies, recoded to 0 for MODIS Land Product consistency)")] <- 0

modis.cols <- c("blue",
								"darkgreen",
								"seagreen",
								"green4",
								"green1",
								"green1",
								"lightsalmon4",
								"lightsalmon1",
								"khaki4",
								"khaki1",
								"lightyellow",
								"lightskyblue1",
								"red",
								"red",
								"orange",
								"white",
								"grey",
								"blue"
								)

fpc.names <- c("natvegfrac", 														#1
							 "Tropical Broadleaved Evergreen Tree",		#2
							 "Tropical Broadleaved Raingreen Tree",		#3
							 "Temperate Needleleaved Evergreen Tree",	#4
							 "Temperate Broadleaved Evergreen Tree",	#5
							 "Temperate Broadleaved Summergreen Tree",#6
							 "Boreal Needleleaved Evergreen Tree",		#7
							 "Boreal Broadleaved Summergreen Tree", # actually Boreal Summergreen Tree (broadleaved and needleleaved)
							 "C3 Perennial Grass",										#9
							 "C4 Perennial Grass"											#10
							 ) #

fpc.grass <- c(9,10)
fpc.tree  <- c(2:8)

fpc.temp.limits <- list(c(-1000,1000),  #1
												c(15.5, 1000),  #2
												c(15.5, 1000),  #3
												c(-2, 20),      #4
												c(3, 18.8),     #5
												c(-17, 15.5),   #6
												c(-32.5, -2),   #7
												c(-1000, -2),   #8
												c(-1000, 15.5), #9
												c(15.5, 1000)   #10
												) #

reclassify.modis <- list("2" = c(4,7),
												 "3" = c(2,5),
												 "4" = c(8,3,4),
												 "5" = c(3,6)
												 #"6" = c(2:8),
												 #"7" = c(2:8),
												 #"8" = c(2:8),
												 #"9" = c(2:8),
												 #"10" = c(2:8)
												 ) #
shuffle.modis <- list("6" = c(2:8),
											"7" = c(2:8),
											"8" = c(2:8),
											"9" = c(2:8),
											"10" = c(2:8)
											)


reference.runtime <- c(1901, 2009)
reference.nyear <- reference.runtime[2]-reference.runtime[1]+1
scenario.runtime  <- c(2010, 2115)
scenario.nyear <- scenario.runtime[2]-scenario.runtime[1]+1
landuse.runtime <- c(1700, 2005)
landuse.name <- paste(gsub("cru06", "cru09",lpj.input.path), "cft1700_2005_bioenergy_sc.bin", sep="") 
pasture.bands <- c(14,30)

period.nyear <- 30 # length of reference and scenario periods

reference.period  <- c(reference.runtime[2]-period.nyear+1, reference.runtime[2])
reference.period2 <- c(1901,1930)
scenario.period   <- c(scenario.runtime[2]-period.nyear+1, scenario.runtime[2])

climate.runtime <- c(1901, 2115)

models <- c(
"cccma_cgcm3_1",    # 1
"cnrm_cm3",         # 2
"csiro_mk3_0",      # 3
"gfdl_cm2_0",       # 4
"gfdl_cm2_1",       # 5
"giss_model_e_h",   # 6
"giss_model_e_r",   # 7
"iap_fgoals",       # 8
"inmcm3_0",         # 9
"ipsl_cm4",         #10
"miroc3_2_hires",   #11
"miroc3_2_medres",  #12
"miub_echo_g",      #13
"mpi_echam5",       #14
"mri_cgcm2_3_2a",   #15
"ncar_ccsm3_0",     #16
"ncar_pcm1",        #17
"ukmo_hadcm3",      #18
"ukmo_hadgem1"      #19
)

target.temps <- c("1.5K",
									"2.0K",
									"2.5K",
									"3.0K",
									"3.5K",
									"4.0K",
									"4.5K",
									"5.0K"
									) #


aggregate.modis <- function(in.filename, in.lon=1, in.lat=2, in.class=3, in.res=modis.res, out.res=lpj.res, in.type=integer(), in.size=1, lon.extent=c(-180,180), lat.extent=c(-90,90)) {
	if(round(lpj.res/modis.res)!=lpj.res/modis.res) {
		stop("Error: out.res is not a multiple of in.res")
	}
	if(length(unique(c(in.lon, in.lat, in.class))) != 3 || min(c(in.lon, in.lat, in.class)) != 1 || max(c(in.lon, in.lat, in.class)) != 3) {
		stop("Error: Please specify lon, lat and class dimensions: Which index in lon, lat and class, respectively")
	}
	dimensions <- list("lon"=in.lon, "lat"=in.lat, "class"=in.class)
	print(paste("Loading MODIS data from", in.filename))
	print(paste("Assuming dimensions of input: c(", names(which(dimensions==1)), ", ", names(which(dimensions==2)), ", ", names(which(dimensions==3)), ")", sep=""))
	array.dimensions <- c(3)
	array.dimensions[dimensions[["class"]]] <- length(modis.classes)
	array.dimensions[dimensions[["lon"]]] <- (max(lon.extent)-min(lon.extent))/in.res
	array.dimensions[dimensions[["lat"]]] <- (max(lat.extent)-min(lat.extent))/in.res
	print(paste("Assuming dimensions of input: c(", toString(array.dimensions), ")", sep=""))
	modis.data <- readBin(in.filename, what=in.type, size=in.size, n=array.dimensions[1]*array.dimensions[2]*array.dimensions[3])
	dim(modis.data) <- array.dimensions
	
	earthradius <- 6371000.785
	cellwidth <- earthradius*pi/180
	modis.lat <- seq(min(lat.extent)+in.res/2, by=in.res, length.out=array.dimensions[dimensions[["lat"]]])
	if(in.lon<in.lat) {
		modis.lat <- rep(modis.lat, each=array.dimensions[dimensions[["lon"]]])
	} else {
		modis.lat <- rep(modis.lat, times=array.dimensions[dimensions[["lon"]]])
	} 
	modis.area <- (cellwidth*in.res)*(cellwidth*in.res)*cos(modis.lat/180*pi)
	field.dimensions <- array.dimensions[-dimensions[["class"]]]
	dim(modis.area) <- field.dimensions
	
	out.dimensions <- array.dimensions
	out.dimensions[dimensions[["lon"]]] <- out.dimensions[dimensions[["lon"]]]*in.res/out.res
	out.dimensions[dimensions[["lat"]]] <- out.dimensions[dimensions[["lat"]]]*in.res/out.res
	
	modis.aggregated <- array(dim=out.dimensions)
	
	print(paste("Output will have dimensions of c(", toString(out.dimensions), ")", sep=""))
	for(c in 1:length(modis.classes)) {
		print(paste("Aggregating MODIS Class", modis.classes[c]))
		for(lon in 1:(array.dimensions[dimensions[["lon"]]]*modis.res/lpj.res)) {
			lon.index <- (lon-1)*out.res/in.res+1
			lon.index <- seq(lon.index, length.out=out.res/in.res)
			for(lat in 1:(array.dimensions[dimensions[["lat"]]]*modis.res/lpj.res)) {
				lat.index <- (lat-1)*out.res/in.res+1
				lat.index <- seq(lat.index, length.out=out.res/in.res)
				
				indices.list <- list("class"=c, "lon"=lon.index, "lat"=lat.index)
				out.indices.list <- list("class"=c, "lon"=lon, "lat"=lat)
				tile.data <- modis.data[indices.list[[names(which(dimensions==1))]], indices.list[[names(which(dimensions==2))]], indices.list[[names(which(dimensions==3))]]]
				#print(paste("tile.data <- modis.data[c(",toString(indices.list[[names(which(dimensions==1))]]),"), c(", toString(indices.list[[names(which(dimensions==2))]]), "), c(", toString(indices.list[[names(which(dimensions==3))]]), ")]", sep=""))
				if(in.lon<in.lat) {
					tile.area <- modis.area[lon.index, lat.index]
				} else {
					tile.area <- modis.area[lat.index, lon.index]
				}
				#print(paste("Range tile.data:", toString(range(tile.data)), "; range tile.area:", toString(range(tile.area))))
				#print(paste("dim tile.data:", toString(dim(tile.data)), "; dim tile.area:", toString(dim(tile.area))))
				modis.aggregated[out.indices.list[[names(which(dimensions==1))]], out.indices.list[[names(which(dimensions==2))]], out.indices.list[[names(which(dimensions==3))]]] <- sum(tile.data*tile.area, na.rm=TRUE)/sum(tile.area, na.rm=TRUE)
			}
		}
	}
	rm(modis.data)
	gc(reset=TRUE)
	return(modis.aggregated)
}

#stop("Test")
force.aggregation <- FALSE
save.aggregation <- TRUE
modis.aggregated.filename <- paste(basepath, "analysis/MODIS_LC_", ifelse(length(modis.years)==1, modis.years, paste(min(modis.years), "-", max(modis.years), sep="")), "_aggregated.RData", sep="")
if(!file.exists(modis.aggregated.filename) || force.aggregation) {
	if(length(modis.years)>1) {
		print(paste("Loading MODIS data for", toString(modis.years), "from", modispath2, "and resampling to LPJ resolution.")) # original data, not data already resampled by Ursula
		modis.aggregated <- array(0, dim=c(360/lpj.res, 180/lpj.res, length(modis.classes)))
		for(year in modis.years) {
			print(paste("*******", year, "*******"))
			#modisname2 <- paste(modispath2, year, "/MODIS_fpc.bin", sep="")
			modisname2 <- paste(modispath2, year, "/MOD12C1.A", year, "001.004__GRID_MOD12C1_Land_Cover_Type_1_Percent.dat", sep="")
			ydata <- aggregate.modis(modisname2, in.lon=2, in.lat=3, in.class=1)
			ydata <- ydata/100/length(modis.years)
			#dim(ydata) <- c(length(modis.classes), 360/lpj.res, 180/lpj.res)
			for(i in 1:length(modis.classes)) {
				modis.aggregated[,,i] <- modis.aggregated[,,i]+ydata[i,,]
			}
		}
	} else {		
		print(paste("Loading MODIS data for", modis.years, "from", modisname))
		modis.data <- readBin(modisname, what=integer(), size=1, n=360*180/modis.res/modis.res*length(modis.classes))
		modis.data <- modis.data/100
		dim(modis.data) <- c(360/modis.res, 180/modis.res, length(modis.classes))

		# aggregate to LPJ resolution
		print("Aggregating data to LPJ resolution")
		earthradius <- 6371000.785
		cellwidth <- earthradius*pi/180
		modis.area <- seq(-90+modis.res/2, by=modis.res, length.out=180/modis.res)
		modis.area <- rep(modis.area, each=360/modis.res)
		modis.area <- (cellwidth*modis.res)*(cellwidth*modis.res)*cos(modis.area/180*pi)
		dim(modis.area) <- dim(modis.data)[c(1,2)]


		modis.aggregated <- array(dim=c(360/lpj.res,180/lpj.res, length(modis.classes)))
		for(lon in 1:(360/lpj.res)) {
			print(lon)
			lon.index <- (lon-1)*lpj.res/modis.res+1
			lon.index <- seq(lon.index, length.out=lpj.res/modis.res)
			for(lat in 1:(180/lpj.res)) {
				lat.index <- (lat-1)*lpj.res/modis.res+1
				lat.index <- seq(lat.index, length.out=lpj.res/modis.res)
				tile <- modis.data[lon.index, lat.index,]
				tile.area <- rep(modis.area[lon.index, lat.index], times=length(modis.classes))
				dim(tile.area) <- dim(tile)
				tile.area[which(is.na(tile))] <- NA
				modis.aggregated[lon,lat,] <- apply(tile*tile.area, 3, sum, na.rm=TRUE)/apply(tile.area, 3, sum, na.rm=TRUE)
			}
		}
	}
	if(save.aggregation) {
		print(paste("Saving aggregated MODIS data to", modis.aggregated.filename))
		save(modis.aggregated, file=modis.aggregated.filename)
	}
} else {
	if(!force.aggregation) {
		print(paste("Reloading aggregated MODIS data from previous script run from", modis.aggregated.filename))
		load(modis.aggregated.filename)
	}
}

#stop("Test before normalization")
print("Determining croplands, cropland/natural vegetation mosaics, water bodies and urban areas")
mask.bands <- which(modis.classes=="Croplands" | modis.classes=="Cropland/Natural Vegetation Mosaic" | modis.classes=="Water" | modis.classes=="Urban and Built-Up" | modis.classes=="(IGBP Water Bodies, recoded to 0 for MODIS Land Product consistency)")
#mask.bands <- c(mask.bands, which(modis.classes=="Grasslands"))
#modis.human.frac <- array(0, dim=dim(modis.data)[1:2])
#modis.water.frac <- array(0, dim=dim(modis.data)[1:2])
modis.human.frac.aggregated <- array(0, dim=dim(modis.aggregated)[1:2])
modis.water.frac.aggregated <- array(0, dim=dim(modis.aggregated)[1:2])
for(i in mask.bands) {
	if(i %in% which(modis.classes=="Water" | modis.classes=="(IGBP Water Bodies, recoded to 0 for MODIS Land Product consistency)")) {
		modis.water.frac.aggregated <- modis.water.frac.aggregated + modis.aggregated[,,i]
	} else {
		#	modis.human.frac <- modis.human.frac + modis.data[,,i]
		modis.human.frac.aggregated <- modis.human.frac.aggregated + modis.aggregated[,,i]
	}
}
modis.natural.frac.aggregated <- apply(modis.aggregated, c(1,2), sum)-modis.water.frac.aggregated-modis.human.frac.aggregated


#input grid
print("Determining managed grasslands from LPJ landuse input")
grid.name <- paste(climate.scen.path, "grid.bin", sep="")
print(paste("Loading reference grid from", grid.name))
grid.file <- file(grid.name, "rb")
seek(grid.file, where=7, origin="start") # skip "LPJGRID"
endian <- "little"
header <- readBin(grid.file, what=integer(), size=4, n=7, endian=endian)
if(header[1] != 1) {
	endian <- "big"
	seek(grid.file, where=7, origin="start")
	header <- readBin(grid.file, what=integer(), size=4, n=7, endian=endian)
	if(header[1] != 1) {
		close(grid.file)
		stop("Error: cannot determine endian of grid file")
	}
}
NCELL.input <- header[6]
grid.data <- array(readBin(grid.file, what=integer(), size=2, n=NCELL.input*2, endian=endian), dim=c(2, NCELL.input))/100
close(grid.file)

# Load landuse input data
landuse.file <- file(landuse.name, "rb")
print(paste("Loading landuse data for MODIS year(s) from LPJ landuse file", landuse.name))
seek(landuse.file, where=10, origin="start") # skip "LPJLANDUSE"
endian <- "little"
header <- readBin(landuse.file, what=integer(), size=4, n=7, endian=endian)
if(header[1] != 1) {
	endian <- "big"
	seek(landuse.file, where=10, origin="start")
	header <- readBin(landuse.file, what=integer(), size=4, n=7, endian=endian)
	if(header[1] != 1) {
		close(landuse.file)
		stop("Error: cannot determine endian of landuse file")
	}
}
if(header[6] != NCELL.input) {
	stop("Error: Grid file and landuse file do not match")
}
landuse.nbands <- header[7]
landuse.startyear <- header[3]
landuse.nyear <- header[4]
if(landuse.startyear > min(modis.years)) {
	stop("Error: LPJ landuse input starts later than MODIS years")
}
if((landuse.startyear+landuse.nyear-1) < max(modis.years)) {
	warning("Warning: LPJ landuse input does not cover full MODIS period")
}
landuse.offset <- (min(modis.years)-landuse.startyear)*NCELL.input*landuse.nbands*2+38
seek(landuse.file, where=landuse.offset, origin="start")
landuse.data <- readBin(landuse.file, what=integer(), size=2, n=landuse.nbands*NCELL.input*ifelse(max(modis.years)<=(landuse.startyear+landuse.nyear-1),length(modis.years),landuse.startyear+landuse.nyear-min(modis.years)))
close(landuse.file)
landuse.data.nyear <- length(landuse.data)/landuse.nbands/NCELL.input
if(length(modis.years) > landuse.data.nyear) {
	print("Extending LPJ landuse input to cover full MODIS period")
	landuse.data <- c(landuse.data, integer(landuse.nbands*NCELL.input*(length(modis.years)-landuse.data.nyear))) # extend to full number of modis years
}
dim(landuse.data) <- c(landuse.nbands, NCELL.input, length(modis.years))
y <- landuse.data.nyear
while(y < length(modis.years)) {
	print(paste("Adding year", min(modis.years)+y))
	landuse.data[,,y+1] <- landuse.data[,,landuse.data.nyear]
	y <- y+1
}
landuse.data <- landuse.data[pasture.bands,,]/1000 # use only bands containing pastures
pasture.frac <- apply(landuse.data, 2, sum)/length(modis.years)

lpj.raster <- raster(ncols=360/lpj.res, nrows=180/lpj.res)
pasture.rasterdata <- array(0, dim=c(360/lpj.res, nrows=180/lpj.res))
pasture.rasterdata[cellFromXY(lpj.raster, t(grid.data))] <- pasture.frac

modis.aggregated.nograsschange <- modis.aggregated
# Remove LPJ pasture from MODIS
if(toString(dim(modis.aggregated)[1:2]) == toString(dim(pasture.rasterdata))) {
	strip.grasslands <- ifelse(modis.aggregated[,,which(modis.classes=="Grasslands")] >= pasture.rasterdata, pasture.rasterdata, modis.aggregated[,,which(modis.classes=="Grasslands")])
	incompatible <- length(which(strip.grasslands < pasture.rasterdata))
	unchanged.grasslands <- modis.aggregated[,,which(modis.classes=="Grasslands")]
	if(incompatible>0) {
		print(paste("Managed grasslands in LPJ input are bigger than Grasslands in MODIS data for", incompatible, "cells."))
	}
	for(i in 1:length(modis.classes))  {
		if(i %in% mask.bands) {
			next
		}
		if(i == which(modis.classes=="Grasslands")) {
			modis.aggregated[,,i] <- modis.aggregated[,,i]-strip.grasslands
		} else {
			modis.aggregated[,,i] <- modis.aggregated[,,i] + ifelse(modis.aggregated[,,i]>0, modis.aggregated[,,i]/(modis.natural.frac.aggregated-unchanged.grasslands)*strip.grasslands,0)
		}
	}
#	modis.aggregated[,,which(modis.classes=="Grasslands")] <- modis.aggregated[,,which(modis.classes=="Grasslands")]-strip.grasslands
	modis.human.frac.nograss <- modis.human.frac.aggregated
#	modis.human.frac.aggregated <- modis.human.frac.aggregated + strip.grasslands
} else {
	warning("Warning: Cannot clear managed grasslands from MODIS data because LPJ landuse data and MODIS aggregated data do not match")
}
	
#stop("Test")

print("Clearing human-dominated landcover classes and water from data and re-normalizing")
# clear cropland
#modis.data[,,mask.bands] <- 0
modis.aggregated[,,mask.bands] <- 0
modis.aggregated.nograsschange[,,mask.bands] <- 0

# normalize area fractions without cropland
print("Normalizing data")
#modis.treecover <- array(0, dim=dim(modis.data)[1:2])
modis.treecover.aggregated <- array(0, dim=dim(modis.aggregated)[1:2])
for(i in 1:length(modis.classes)) {
	if(i %in% mask.bands) {
		next
	}
	#modis.data[,,i] <- ifelse(modis.human.frac+modis.water.frac < 1, modis.data[,,i]/(1-modis.human.frac-modis.water.frac), 0)
	#modis.treecover <- modis.treecover+modis.data[,,i]*modis.treefrac[i]
	modis.aggregated[,,i] <- ifelse(modis.human.frac.aggregated+modis.water.frac.aggregated < 1, modis.aggregated[,,i]/(1-modis.human.frac.aggregated-modis.water.frac.aggregated), 0)
	modis.treecover.aggregated <- modis.treecover.aggregated+modis.aggregated[,,i]*modis.treefrac[i]
	modis.aggregated.nograsschange[,,i] <- ifelse(modis.human.frac.nograss+modis.water.frac.aggregated < 1, modis.aggregated.nograsschange[,,i]/(1-modis.human.frac.nograss-modis.water.frac.aggregated), 0)
}

# mask out water and cells totally covered by cropland an mosaic
#modis.treecover[which(modis.water.frac==1)] <- NA
#modis.treecover[which(modis.human.frac==1)] <- NA

modis.treecover.aggregated[which(modis.water.frac.aggregated==1)] <- NA
modis.treecover.aggregated[which(modis.human.frac.aggregated==1)] <- NA
#modis.treecover.raster <- raster(ncols=360/modis.res, nrows=180/modis.res)
modis.treecover.raster.aggregated <- raster(ncols=360/lpj.res, nrows=180/lpj.res)
if(FALSE) {
fill.seq <- seq(0, length(modis.treecover), length.out=4)
for(i in 1:(length(fill.seq)-1)) {
	start <- fill.seq[i]+1
	end   <- fill.seq[i+1]
	modis.treecover.raster[start:end] <- modis.treecover[start:end]
}
}
fill.seq <- seq(0, length(modis.treecover.aggregated), length.out=4)
for(i in 1:(length(fill.seq)-1)) {
	start <- fill.seq[i]+1
	end   <- fill.seq[i+1]
	modis.treecover.raster.aggregated[start:end] <- modis.treecover.aggregated[start:end]
}

treecover.palette <- colorRampPalette(c("yellow", "red", "darkgreen"))

dominant <- function(x) {
	val <- which(x == max(x, na.rm=T))
	if(length(val) == 0) {
		warning("Cannot determine dominant type because of no non-NA values")
	}
	if(length(val) > 1) {
		warning("Cannot determine dominant type because of more than one type with maximum cover")
	}
	if(length(val) == 1) {
		return(val)
	} else {
		return(NA)
	}
}
#stop("Test")


# Load climate data
# reference climate
tmp.name <- paste(climate.scen.path, "cruts3.1_", models[1], "_", target.temps[1], "_", climate.runtime[1], "-", climate.runtime[2], "_tmp.clm", sep="")
print(paste("Loading reference climate from", tmp.name))
tmp.file <- file(tmp.name, "rb")
seek(tmp.file, where=10, origin="start") # skip "LPJCLIMATE"
endian <- "little"
header <- readBin(tmp.file, what=integer(), size=4, n=7, endian=endian)
if(header[1] != 1) {
	endian <- "big"
	seek(tmp.file, where=10, origin="start")
	header <- readBin(tmp.file, what=integer(), size=4, n=7, endian=endian)
	if(header[1] != 1) {
		close(tmp.file)
		stop("Error: cannot determine endian of temperature file")
	}
}
if(header[6] != NCELL.input) {
	stop("Error: Grid file and temperature file do not match")
}
reference.offset <- (reference.period[1]-header[3])*12*NCELL.input*2+38
seek(tmp.file, where=reference.offset, origin="start")
reference.tmp <- readBin(tmp.file, what=integer(), n=12*NCELL.input*period.nyear, size=2, endian=endian)/10
dim(reference.tmp) <- c(12, NCELL.input, period.nyear)
#dayspermonth <- c(31,28,31,30,31,30,31,31,30,31,30,31)
reference.tmp <- apply(reference.tmp, c(1,2), mean, na.rm=TRUE)
reference.tmin <- apply(reference.tmp, c(2), min, na.rm=TRUE)
close(tmp.file)

cells.from.aggregated.raster <- cellFromXY(modis.treecover.raster.aggregated, t(grid.data))

reference.tmin.raster <- modis.treecover.raster.aggregated
reference.tmin.raster[] <- NA
reference.tmin.raster[cells.from.aggregated.raster] <- reference.tmin

# Load reference FPC
reference.fpcname <- paste(basepath, reference.path, "fpc.bin", sep="")
print(paste("Loading reference FPC from", reference.fpcname))
NCELL <- file.info(reference.fpcname)$size/reference.nyear/length(fpc.names)/4
print(paste("NCELL:", NCELL))
reference.offset <- (reference.period[1]-reference.runtime[1])*NCELL*length(fpc.names)*4
reference.fpcfile <- file(reference.fpcname, "rb")
seek(reference.fpcfile, where=reference.offset, origin="start")
reference.fpc <- readBin(reference.fpcfile, what=numeric(), size=4, n=NCELL*length(fpc.names)*period.nyear)
dim(reference.fpc) <- c(NCELL, length(fpc.names), period.nyear)
reference.fpc <- apply(reference.fpc, c(1,2), mean, na.rm=TRUE)
close(reference.fpcfile)

# load outgrid
reference.gridname <- paste(basepath, reference.path, "grid.out", sep="")
reference.griddata <- readBin(reference.gridname, what=integer(), size=2, n=NCELL*2)/100
dim(reference.griddata) <- c(2, NCELL)

reference.treecover <- apply(reference.fpc[,fpc.tree], 1, sum, na.rm=TRUE)
reference.treecover.raster <- modis.treecover.raster.aggregated
reference.treecover.raster[] <- NA
reference.treecover.raster[cellFromXY(reference.treecover.raster, t(reference.griddata))] <- reference.treecover

#stop("Test")

modis.forest.aggregated <- apply(modis.aggregated[,,2:6], c(1,2), sum, na.rm=TRUE)
modis.evergreen.aggregated <- apply(modis.aggregated[,,2:3], c(1,2), sum, na.rm=TRUE)
modis.deciduous.aggregated <- apply(modis.aggregated[,,4:5], c(1,2), sum, na.rm=TRUE)

modis.forest.types <- array(dim=c(dim(modis.evergreen.aggregated), 3))
modis.forest.types[,,1] <- modis.evergreen.aggregated
modis.forest.types[,,2] <- modis.deciduous.aggregated
modis.forest.types[,,3] <- modis.aggregated[,,6]
modis.forest.dominant <- apply(modis.forest.types, c(1,2), dominant)


#stop("Test")


#stop("Test")

# transcribe MODIS classes to LPJ PFTs
reclass.modis <- function(in.modis) {
	output.fpc <- array(0, dim=c(NCELL.input, length(fpc.names)))
	for(modis.class in 2:length(modis.classes)) {
		if(modis.class %in% mask.bands) {
			next
		}
		print(modis.class)
		modis.fraction <- in.modis[,,modis.class]
		for(cell in 1:NCELL.input) {
			if(modis.fraction[cells.from.aggregated.raster[cell]] == 0) {
				#print(paste("skip cell", cell))
				next # if fraction == 0 no need to transcribe class
			}
			# grass
			if(modis.grassfrac[modis.class]>0) {
				grass.possible <- integer(0)
				for(f in fpc.grass) {
					if(reference.tmin[cell] > fpc.temp.limits[[f]][1] && reference.tmin[cell] <= fpc.temp.limits[[f]][2]) {
						grass.possible <- c(grass.possible, f)
					}
				}
				if(length(grass.possible)>=1) {
					output.fpc[cell,grass.possible] <- output.fpc[cell,grass.possible] + modis.fraction[cells.from.aggregated.raster[cell]]*modis.grassfrac[modis.class]/length(grass.possible)
				}
			}
			# trees
			if((modis.class %in% names(reclassify.modis))==TRUE) {
				tree.possible <- integer(0)
				for(f in reclassify.modis[[as.character(modis.class)]]) {
					if(reference.tmin[cell] > fpc.temp.limits[[f]][1] && reference.tmin[cell] <= fpc.temp.limits[[f]][2]) {
						tree.possible <- c(tree.possible, f)
					}
				}
				if(length(tree.possible)>=1) {
					#print(paste("Re-classifying", round(modis.fraction[cells.from.aggregated.raster[cell]]*modis.treefrac[modis.class],3), modis.classes[modis.class], "to", toString(fpc.names[tree.possible])))
					output.fpc[cell,tree.possible] <- output.fpc[cell,tree.possible] + modis.fraction[cells.from.aggregated.raster[cell]]*modis.treefrac[modis.class]/length(tree.possible)
				}
			}
			if(modis.class %in% names(shuffle.modis)) {
				tree.possible <- integer(0)
				for(f in shuffle.modis[[as.character(modis.class)]]) {
					if(reference.tmin[cell] > fpc.temp.limits[[f]][1] && reference.tmin[cell] <= fpc.temp.limits[[f]][2]) {
						tree.possible <- c(tree.possible, f)
					}
				}
#				print(toString(tree.possible))
				target.order <- order(output.fpc[cell, tree.possible], decreasing=TRUE)
#				print(toString(target.order))
				if(length(target.order)>2) {
					target <- target.order[1:2]
				} else {
					target <- target.order
				}
#				print(toString(target))
#				print(paste("Re-shuffling", round(modis.fraction[cells.from.aggregated.raster[cell]]*modis.treefrac[modis.class],3), modis.classes[modis.class], "to", toString(fpc.names[tree.possible[target]])))
				output.fpc[cell, tree.possible[target]] <- output.fpc[cell, tree.possible[target]]+modis.fraction[cells.from.aggregated.raster[cell]]*modis.treefrac[modis.class]/length(target)
			}
		}
	}
	return(output.fpc)
}
lpj.fpc.from.modis <- reclass.modis(modis.aggregated)
lpj.fpc.from.modis.nograsschange <- reclass.modis(modis.aggregated.nograsschange)


stop ("Test")

#writeBin(as.double(lpj.fpc.from.modis), paste(basepath, "analysis/modis.fpc_mar2013.bin", sep=""), size=4)

stop ("Test")

modis.dominant <- apply(modis.aggregated, c(1,2), dominant)
modis.raster <- raster(ncols=360/lpj.res, nrows=180/lpj.res)
modis.raster[] <- as.vector(modis.dominant)
image(modis.raster, col=modis.cols)
image.plot(zlim=c(1, length(modis.classes)), legend.only=TRUE, col=modis.cols, axis.args=list(at=seq(1.5, length(modis.classes)-0.5, length.out=length(modis.classes)), labels=modis.classes))

modis.dominant2 <- apply(modis.aggregated, c(1,2), dominant)
image.plot(modis.dominant2, col=modis.cols, axis.args=list(at=seq(1.5, length(modis.classes)-0.5, length.out=length(modis.classes)), labels=modis.classes))
modis.aggregated.raster <- raster(ncols=360/lpj.res, nrows=180/lpj.res)
modis.aggregated.raster[] <- as.vector(modis.dominant2)
image.plot(modis.aggregated.raster, col=modis.cols)#, axis.args=list(at=seq(1.5, length(modis.classes)-0.5, length.out=length(modis.classes)), labels=modis.classes))
image.plot(zlim=c(1, length(modis.classes)), legend.only=TRUE, col=modis.cols, axis.args=list(at=seq(1.5, length(modis.classes)-0.5, length.out=length(modis.classes)), labels=modis.classes))
modis.dominant <- apply(modis.data, c(1,2), dominant)
