#####################################################################################
## Preprocess LPJmL outputs from simulations runs for Gamma calculation
## Aggregates monthly outputs to yearly and combines some outputs

rm(list=ls(all=TRUE))
library(fields)

### filesystem details (paths) removed ###
basepath <- c("***")
climate.scen.path <- "***"
lpj.input.path <- "***"
cluster <- F

reference.path <- "***"
scenario.path  <- "***"

reference.runtime <- c(1901, 2009)
reference.nyear <- reference.runtime[2]-reference.runtime[1]+1
scenario.runtime  <- c(2010, 2115)
scenario.nyear <- scenario.runtime[2]-scenario.runtime[1]+1

reference.period  <- c(reference.runtime[2]-30+1, reference.runtime[2])
scenario.period   <- c(scenario.runtime[2]-30+1, scenario.runtime[2])

process.scenarios <- TRUE
process.reference <- FALSE

climate.runtime <- c(1901, 2115)

aggregate.lib <- paste(basepath, "analysis/aggregate_monthly_output", ifelse(cluster==TRUE, "_cluster", ""), ".so", sep="")

dyn.load(aggregate.lib)
aggregate.monthly.output <- function(indata, inmethod="sum", inNA=-9999) {
	indata[which(is.na(indata))] <- inNA
	if(length(dim(indata)) != 3) 
		stop("Error: indata must have the following dimensions: dim(indata) = c(ncells, nmonths, nyears)")
	tempdata <- .C("aggregate_monthly_output",
									var_local=as.double(indata), ncells=as.integer(dim(indata)[1]), nmonths=as.integer(dim(indata)[2]), nyears=as.integer(dim(indata)[3]),
									NAvalue=as.double(inNA), var_sum=double(dim(indata)[1]*dim(indata)[3]), return_months=integer(dim(indata)[1]*dim(indata)[3])
									)
	aggregate.sum <- tempdata$var_sum
	aggregate.num <- tempdata$return_months
	aggregate.sum[which(aggregate.sum==inNA)] <- NA
	aggregate.sum[which(aggregate.num==0)] <- NA
	if(inmethod=="mean")
		aggregate.sum <- aggregate.sum/aggregate.num
	return(aggregate.sum)
}
		
print("LPJ Runs:")
print(paste("Reference run: ", reference.runtime[1], "-", reference.runtime[2], sep=""))
print(paste("Reference period: ", reference.period[1], "-", reference.period[2], sep=""))
print(paste("Scenario run: ", scenario.runtime[1], "-", scenario.runtime[2], sep=""))
print(paste("Scenario period: ", scenario.period[1], "-", scenario.period[2], sep=""))

models <- c("cccma_cgcm3_1",    # 1
            "cnrm_cm3",         # 2
            "csiro_mk3_0",      # 3
            "gfdl_cm2_0",       # 4
            "gfdl_cm2_1",       # 5
            "giss_model_e_h",   # 6
            "giss_model_e_r",   # 7
            "iap_fgoals",       # 8
            "inmcm3_0",         # 9
            "ipsl_cm4",         #10
            "miroc3_2_hires",   #11
            "miroc3_2_medres",  #12
            "miub_echo_g",      #13
            "mpi_echam5",       #14
            "mri_cgcm2_3_2a",   #15
            "ncar_ccsm3_0",     #16
            "ncar_pcm1",        #17
            "ukmo_hadcm3",      #18
            "ukmo_hadgem1"      #19
)

target.temps <- c("1.5K",
									"2.0K",
									"2.5K",
									"3.0K",
									"3.5K",
									"4.0K",
									"4.5K",
									"5.0K"
)

# filenames
grid.name <- "grid.out"
output.variables <- c("firec",      # 1
											"firef",      # 2
											"flux_estab", # 3
											"fpc",        # 4
											"litc",       # 5
											"mevap",      # 6
											"minterc",    # 7
											"mnpp",       # 8
											"mrh",        # 9
											"mrunoff",    #10
											"mswc1",      #11
											"mswc2",      #12
											"mtransp",    #13
											"pft_npp.pft",#14
											"soilc",      #15
											"vegc",       #16
											"waterstress" #17
)
soil.name <- paste(lpj.input.path, "soil.bin", sep="")
reference.grid.name <- paste(basepath,reference.path,grid.name, sep="")
NCELL <- file.info(reference.grid.name)$size/4

grid.data <- array(readBin(reference.grid.name, what=integer(), size=2, n=NCELL*2), dim=c(2, NCELL))/100
earthradius <- 6371000.785
cellwidth <- earthradius*pi/180
grid.area <- (cellwidth*0.5)*(cellwidth*0.5)*cos(grid.data[2,]/180*pi)

soil.data <- as.integer(readBin(soil.name, raw(), n=file.info(soil.name)$size))
NCELL.input <- length(soil.data)

if(process.scenarios) {
for(model in models) {
	print(model)
	for(target in target.temps) {
		print(target)
		if(!file.exists(paste(basepath, scenario.path, model, "/", target, "/", grid.name, sep=""))) {
			print(paste("Warning: scenario", model, target, "does not exist. Skip processing."))
			next
		}
		# aggregate monthly data
		print("Aggregate monthly data")
		for(var in output.variables[c(6,7,8,9,10,11,13)]) {
			filename <- paste(basepath, scenario.path, model, "/", target, "/", var, ".bin", sep="")
			outfilename <- paste(basepath, scenario.path, model, "/", target, "/", substr(var, 2, nchar(var)), ".bin", sep="") # same filename without leading "m"
			filesize <- file.info(filename)$size
			filedata <- readBin(filename, what=numeric(), size=4, n=filesize/4)
			dim(filedata) <- c(NCELL, 12, filesize/NCELL/12/4)
			if(var %in% output.variables[c(11,12)])
				yearlydata <- aggregate.monthly.output(filedata, "mean")
			else
				yearlydata <- aggregate.monthly.output(filedata)
			# check for negative values
			rm(filedata)
			gc(reset=T)
			if(length(which(yearlydata<0))>0) {
				print(paste("Correcting", length(which(yearlydata<0)), "negative values in", outfilename))
				yearlydata[which(yearlydata<0)] <- 0
			}
			writeBin(as.numeric(yearlydata), outfilename, size=4)
		}
		
		# combine some outputs
		print("Combine outputs (carbon stocks, water fluxes)")
		soilcname <-  paste(basepath, scenario.path, model, "/", target, "/", "soilc.bin", sep="")
		litcname <-  paste(basepath, scenario.path, model, "/", target, "/", "litc.bin", sep="")
		outfilename <- paste(basepath, scenario.path, model, "/", target, "/", "soillitc.bin", sep="")
		soilcsize <- file.info(soilcname)$size
		litcsize <- file.info(litcname)$size
		if(soilcsize != litcsize) {
			print(paste("Error: soilc and litc have different file size (", soilcname, ": ", soilcsize, ", ", litcname, ": ", litcsize, ")", sep=""))
		} else {
			soilcdata <- readBin(soilcname, what=numeric(), size=4, n=soilcsize/4)
			litcdata <- readBin(litcname, what=numeric(), size=4, n=litcsize/4)
			outputdata <- soilcdata+litcdata
			rm(soilcdata, litcdata)
			gc(reset=T)
			if(length(which(outputdata<0))>0) {
				print(paste("Correcting",length(which(outputdata<0)), "negative values in", outfilename))
				outputdata[which(outputdata<0)] <- 0
			}
			writeBin(as.numeric(outputdata), outfilename, size=4)
		}
		
		evapname <-  paste(basepath, scenario.path, model, "/", target, "/", "evap.bin", sep="")
		intercname <-  paste(basepath, scenario.path, model, "/", target, "/", "interc.bin", sep="")
		outfilename <- paste(basepath, scenario.path, model, "/", target, "/", "evapinterc.bin", sep="")
		evapsize <- file.info(evapname)$size
		intercsize <- file.info(intercname)$size
		if(evapsize != intercsize) {
			print(paste("Error: evap and interc have different file size (", evapname, ": ", evapsize, ", ", intercname, ": ", intercsize, ")", sep=""))
		} else {
			evapdata <- readBin(evapname, what=numeric(), size=4, n=evapsize/4)
			intercdata <- readBin(intercname, what=numeric(), size=4, n=intercsize/4)
			outputdata <- evapdata+intercdata
			rm(evapdata, intercdata)
			gc(reset=T)
			if(length(which(outputdata<0))>0) {
				print(paste("Correcting",length(which(outputdata<0)), "negative values in", outfilename))
				outputdata[which(outputdata<0)] <- 0
			}
			writeBin(as.numeric(outputdata), outfilename, size=4)
			# delete temporary files evap and interc (aggregated from mevap and minterc)
			unlink(c(evapname, intercname))
		}
	}
}
}

if(process.reference) {
	## reference run
	print("Reference Run")
	# aggregate monthly data
	print("Aggregate monthly data")
	for(var in output.variables[c(6,7,8,9,10,11,13)]) {
		filename <- paste(basepath, reference.path, var, ".bin", sep="")
		outfilename <- paste(basepath, reference.path, substr(var, 2, nchar(var)), ".bin", sep="") # same filename without leading "m"
		filesize <- file.info(filename)$size
		filedata <- readBin(filename, what=numeric(), size=4, n=filesize/4)
		dim(filedata) <- c(NCELL, 12, filesize/NCELL/12/4)
		if(var %in% output.variables[c(11,12)])
			yearlydata <- aggregate.monthly.output(filedata, "mean")
			else
				yearlydata <- aggregate.monthly.output(filedata)

		# check for negative values
		rm(filedata)
		gc(reset=T)
		if(length(which(yearlydata<0))>0) {
			print(paste("Correcting", length(which(yearlydata<0)), "negative values in", outfilename))
			yearlydata[which(yearlydata<0)] <- 0
		}
		writeBin(as.numeric(yearlydata), outfilename, size=4)
	}

	# combine some outputs
	print("Combine outputs (carbon stocks, water fluxes)")
	soilcname <-  paste(basepath, reference.path, "soilc.bin", sep="")
	litcname <-  paste(basepath, reference.path, "litc.bin", sep="")
	outfilename <- paste(basepath, reference.path, "soillitc.bin", sep="")
	soilcsize <- file.info(soilcname)$size
	litcsize <- file.info(litcname)$size
	if(soilcsize != litcsize) {
		print(paste("Error: soilc and litc have different file size (", soilcname, ": ", soilcsize, ", ", litcname, ": ", litcsize, ")", sep=""))
	} else {
		soilcdata <- readBin(soilcname, what=numeric(), size=4, n=soilcsize/4)
		litcdata <- readBin(litcname, what=numeric(), size=4, n=litcsize/4)
		outputdata <- soilcdata+litcdata
		rm(soilcdata, litcdata)
		gc(reset=T)
		if(length(which(outputdata<0))>0) {
			print(paste("Correcting",length(which(outputdata<0)), "negative values in", outfilename))
			outputdata[which(outputdata<0)] <- 0
		}
		writeBin(as.numeric(outputdata), outfilename, size=4)
	}

	evapname <-  paste(basepath, reference.path, "evap.bin", sep="")
	intercname <-  paste(basepath, reference.path, "interc.bin", sep="")
	outfilename <- paste(basepath, reference.path, "evapinterc.bin", sep="")
	evapsize <- file.info(evapname)$size
	intercsize <- file.info(intercname)$size
	if(evapsize != intercsize) {
		print(paste("Error: evap and interc have different file size (", evapname, ": ", evapsize, ", ", intercname, ": ", intercsize, ")", sep=""))
	} else {
		evapdata <- readBin(evapname, what=numeric(), size=4, n=evapsize/4)
		intercdata <- readBin(intercname, what=numeric(), size=4, n=intercsize/4)
		outputdata <- evapdata+intercdata
		rm(evapdata, intercdata)
		gc(reset=T)
		if(length(which(outputdata<0))>0) {
			print(paste("Correcting",length(which(outputdata<0)), "negative values in", outfilename))
			outputdata[which(outputdata<0)] <- 0
		}
		writeBin(as.numeric(outputdata), outfilename, size=4)
		# delete temporary files evap and interc (aggregated from mevap and minterc)
		unlink(c(evapname, intercname))
	}
}

dyn.unload(aggregate.lib)
