#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "utilfunc_april2012.h"
#include <math.h>
#include <errno.h>
#define DATA_TYPE float
#define DATA_TYPE_OUT float
#define EPSILON 10e-4
#ifdef NAN
  #define INFCHANGE NAN
#else
  #define INFCHANGE sqrt(-1)
#endif

/* calculate parts of the change vector (angle between vectors):
        nom = (a.b) /nominator)
        denominator: |a|*|b|*/
/* normalization pixel based */
/* and variability taken into account */

void usage(char* progname){
	fprintf(stderr, "Use:\n%s npar is_range_spec (1: range is specified for each parameter, 0: mean/std for each parameter are determined internally) is_natveg_spec (1: natveg fraction is specified as input file, 0: no managed land fraction assumed) [natvegfile(supply only if is_natveg_spec=1)] nyear_ref(years in reference file) nyear_fut(years in future file) nyear_avg(time slice thickness) startyear_ref startyear_fut gridfile res(deg) weight_par1 file_ref_par1 (reference file) file_fut_par1(transient file) [mean1 std1][weight_par2 file1_par2 file2_par2 [mean2 std2]...] npixel ofile1(angle between vectors) ofile2(magnitude of change,4 bands) ofile3(time series of angle) ofile4(time series of magnitude) ofile5(angle, global normalization),ofile6(magnitude, global normalization) ofile7(time series of angle,global normalization) ofile8(time series of magnitude,global normalization)\n", progname);
	exit(-1);
}
int main(int argc, char* argv[]){
	int i,j,p,b,f,y,by,npixel,npar,nbands,nbands2,nbands_old,is_range_spec,is_natveg_spec,nyear_ref,nyear_fut,startyear_ref,startyear_fut,nyear_avg,nr,nc, timelength, nargs_req;
	DATA_TYPE *rbuf1, *rbuf2, *natvegfrac;
	DATA_TYPE_OUT *wbuf;
	char *ifilename1, *ifilename2, *ofilename, *ofilename2,*ofilename3,*ofilename4,*ofilename5,*ofilename6,*ofilename7,*ofilename8,*gridfile_name, *natvegfile_name;
	FILE **ifiles_ref , **ifiles_fut, *ofile,*ofile2,*ofile3,*ofile4,*ofile5,*ofile6,*ofile7,*ofile8,*gridfile,*natvegfile;
	double *weights,weight,wsum,*nom,*diff,*absa,*absb,*nom_g,*diff_g,*absa_g,*absb_g,tmp,*mean_spec,*std_spec,res,*dummy,*nom_ts,*absa_ts,*absb_ts,*nom_ts_g,*absa_ts_g,*absb_ts_g,*mean_ref,*std_ref,*mean_fut,*std_fut,*mean_global,*std_global,mean_global_ref,std_global_ref;
	struct stat filestat;

	/* initialize optional pointers*/
	ofile2=ofile3=ofile4=ofile5=ofile6=NULL; 
	mean_spec=std_spec=dummy=NULL;
	wsum=0;
	/* Parse command line */
	nargs_req = 20; /* minimum number of arguments required to run */
	if (argc < nargs_req) usage(argv[0]);
	i=1;
	npar = atoi(argv[i++]);
	nargs_req += (npar*3); /* 3 mandatory arguments for each parameter */
	is_range_spec=atoi(argv[i++]);
	nargs_req += (npar*2*is_range_spec); /* if is_range_spec 2 additional arguments for each parameter */
	is_natveg_spec=atoi(argv[i++]);
	if(is_natveg_spec) {
		natvegfile_name=argv[i++];
		natvegfile=fopen64(natvegfile_name, "rb");
		check_file(natvegfile);
		nargs_req++; /* if is_natveg_spec 1 additional argument */
	}
	if(argc < nargs_req) {
		fprintf(stderr, "Error: You have %d missing argument(s). Please check again.\nRelevant parameters:\n\tnpar: %d\n\tis_range_spec: %d\n\tis_natveg_spec: %d\n", nargs_req-argc, npar, is_range_spec, is_natveg_spec);
		usage(argv[0]);
	}
	if(argc > nargs_req) {
		fprintf(stderr, "Error: You have %d too many argument(s). Please check again.\nRelevant parameters:\n\tnpar: %d\n\tis_range_spec: %d\n\tis_natveg_spec: %d\n", argc-nargs_req, npar, is_range_spec, is_natveg_spec);
		usage(argv[0]);
	}
	nyear_ref=atoi(argv[i++]);
	nyear_fut=atoi(argv[i++]);
	nyear_avg=atoi(argv[i++]);
	startyear_ref=atoi(argv[i++]);
	startyear_fut=atoi(argv[i++]);
	timelength = nyear_ref;
	
	gridfile_name=argv[i++];
	gridfile=fopen64(gridfile_name,"rb");
	check_file(gridfile);
	res=atof(argv[i++]);
	weights = (double*)malloc(npar*sizeof(double));
	check_ptr(weights);
	if(is_range_spec){
		mean_spec = (double*)malloc(npar*sizeof(double));
		check_ptr(mean_spec);
		std_spec = (double*)malloc(npar*sizeof(double));
		check_ptr(std_spec);
	}
	ifiles_ref = (FILE **)malloc(npar*sizeof(FILE*));
	check_ptr(ifiles_ref);

	ifiles_fut = (FILE **)malloc(npar*sizeof(FILE*));
	check_ptr(ifiles_fut);
	for (f=0; f<npar;++f){
		weights[f]=atof(argv[i++]);
		wsum+=weights[f];
		ifilename1 = argv[i++];
		ifilename2 = argv[i++];
		ifiles_ref[f] = fopen64(ifilename1, "rb");
		check_file(ifiles_ref[f]);
		ifiles_fut[f] = fopen64(ifilename2, "rb");
		check_file(ifiles_fut[f]);
		if(is_range_spec){
			mean_spec[f]=atof(argv[i++]);
			std_spec[f]=atof(argv[i++]);
		}
	}
	printf("Number of parameters: %d\nSum of parameter weights: %lf\n", npar, wsum);
	
	for (f=0; f<npar;++f)
		weights[f]/=wsum;
	
	npixel = atoi(argv[i++]);
	ofilename = argv[i++];
	ofile = fopen64(ofilename, "wb");
	check_file(ofile);
	ofilename2=argv[i++];
	ofile2 = fopen64(ofilename2, "wb");
	check_file(ofile2);
	ofilename3=argv[i++];
	ofile3 = fopen64(ofilename3, "wb");
	check_file(ofile3);
	ofilename4=argv[i++];
	ofile4 = fopen64(ofilename4, "wb");
	check_file(ofile4);
	ofilename5=argv[i++];
	ofile5 = fopen64(ofilename5, "wb");
	check_file(ofile5);
	ofilename6=argv[i++];
	ofile6 = fopen64(ofilename6, "wb");
	check_file(ofile6);
	ofilename7=argv[i++];
	ofile7 = fopen64(ofilename7, "wb");
	check_file(ofile7);
	ofilename8=argv[i++];
	ofile8 = fopen64(ofilename8, "wb");
	check_file(ofile8);
	/* calculate number of bands from filesize of first file +npixel; */
	fstat(fileno(ifiles_ref[0]),&filestat);
	nbands=filestat.st_size/npixel/sizeof(DATA_TYPE)/nyear_ref;
	nr=(int)(180./res+0.0001);
	nc=(int)(360./res+0.0001);

	/*memory allocation*/
	rbuf1 = (DATA_TYPE*)malloc(npixel*nbands*sizeof(DATA_TYPE));
	check_ptr(rbuf1);
	rbuf2 = (DATA_TYPE*)malloc(npixel*nbands*sizeof(DATA_TYPE));
	check_ptr(rbuf2);
	wbuf  = (DATA_TYPE_OUT*)malloc(npixel*sizeof(DATA_TYPE_OUT));
	check_ptr(wbuf);
	nom = (double*)malloc(npixel*sizeof(double)); /*nominator of change metric*/
	check_ptr(nom);
	diff = (double*)malloc(npixel*sizeof(double)); 
	check_ptr(diff);
	absa= (double*)malloc(npixel*sizeof(double)); /* absolute value of vectors */
	check_ptr(absa);
	absb= (double*)malloc(npixel*sizeof(double));
	check_ptr(absb);
	nom_g= (double*)malloc(npixel*sizeof(double));
	check_ptr(nom_g);
	diff_g = (double*)malloc(npixel*sizeof(double)); 
	check_ptr(diff_g);
	absa_g= (double*)malloc(npixel*sizeof(double)); /* absolute value of vectors */
	check_ptr(absa_g);
	absb_g= (double*)malloc(npixel*sizeof(double));
	check_ptr(absb_g);
	nom_ts=(double*)malloc(npixel*timelength*sizeof(double));
	check_ptr(nom_ts);
	absa_ts=(double*)malloc(npixel*timelength*sizeof(double));
	check_ptr(absa_ts);
	nom_ts_g=(double*)malloc(npixel*timelength*sizeof(double));
	check_ptr(nom_ts_g);
	absa_ts_g=(double*)malloc(npixel*timelength*sizeof(double));
	check_ptr(absa_ts_g);
	absb_ts_g=(double*)malloc(npixel*timelength*sizeof(double));
	check_ptr(absb_ts_g);
	absb_ts=(double*)malloc(npixel*timelength*sizeof(double));
	check_ptr(absb_ts);
	mean_ref=(double*)malloc(npixel*sizeof(double));
	check_ptr(mean_ref);
	std_ref=(double*)malloc(npixel*sizeof(double));
	check_ptr(std_ref);
	mean_fut=(double*)malloc(npixel*sizeof(double));
	check_ptr(mean_fut);
	std_fut=(double*)malloc(npixel*sizeof(double));
	check_ptr(std_fut);
	mean_global=(double*)malloc(timelength*sizeof(double));
	check_ptr(mean_global);
	std_global=(double*)malloc(timelength*sizeof(double));
	check_ptr(std_global);

	natvegfrac=(DATA_TYPE*)malloc(npixel*sizeof(DATA_TYPE));
	check_ptr(natvegfrac);
	/*
	
	/*initialize change metric arrays*/
	for (p=0; p<npixel; ++p){
		wbuf[p]=0.0;
		nom[p]=absa[p]=absb[p]=nom_g[p]=absa_g[p]=absb_g[p]=diff[p]=diff_g[p]=0;
		natvegfrac[p]=1.0;
		for(y=0;y<timelength;y++)
			nom_ts[y*npixel+p]=absa_ts[y*npixel+p]=absb_ts[y*npixel+p]=absa_ts_g[y*npixel+p]=absb_ts_g[y*npixel+p]=0; 
	}
	if(is_natveg_spec) {
		i=fseek(natvegfile, 0, SEEK_SET);
		if(fread(natvegfrac, sizeof(DATA_TYPE), npixel, natvegfile) != npixel) {
			fprintf(stderr, "Error: cannot read natvegfrac from %s\n", natvegfile_name);
			exit(1);
		}
	}
	for (p=0; p<npixel; ++p)
		if(natvegfrac[p] < 0) natvegfrac[p] = 0.0;
	

	/* loop over input variables */
	for(f=0;f<npar;f++){
		nbands_old=nbands;
		fstat(fileno(ifiles_ref[0]),&filestat);
		nbands=filestat.st_size/npixel/sizeof(DATA_TYPE)/nyear_ref;
		fstat(fileno(ifiles_fut[0]),&filestat);
		nbands2=filestat.st_size/npixel/sizeof(DATA_TYPE)/nyear_fut;
		//printf("Number of bands: %d\n",nbands);
		if (nbands != nbands2){
			fprintf(stderr, "File1 and File2 have different sizes! Exit.\n");
			exit(-1);
		}
		if(nbands!=nbands_old){
			// reallocate memory
			rbuf1 = (DATA_TYPE*)realloc(rbuf1,npixel*nbands*sizeof(DATA_TYPE));
			check_ptr(rbuf1);
			rbuf2 = (DATA_TYPE*)realloc(rbuf2,npixel*nbands*sizeof(DATA_TYPE));
			check_ptr(rbuf2);
		}

		weight=weights[f];
		/*initializations*/
		for(p=0;p<npixel;p++)
			mean_ref[p]=std_ref[p]=mean_fut[p]=std_fut[p]=0.0;
		for(y=0;y<timelength;y++)
			mean_global[y]=std_global[y]=0.0;
		
		//local mean / std for reference period (for each pixel)
		i=fseek(ifiles_ref[f],startyear_ref*npixel*nbands*sizeof(DATA_TYPE),SEEK_SET);
		rbuf1=realloc(rbuf1,sizeof(DATA_TYPE)*npixel*nbands*nyear_avg);
		if(fread(rbuf1,sizeof(DATA_TYPE),npixel*nbands*nyear_avg,ifiles_ref[f])!=npixel*nbands*nyear_avg)
			fprintf(stderr,"Error reading from file ifiles_ref[%d]\n",f);
		timeseries_mean_bsq_float(rbuf1,npixel,nyear_avg,nbands,mean_ref,std_ref);
		
		// local mean / std for future period
		i=fseek(ifiles_fut[f],startyear_fut*npixel*nbands*sizeof(DATA_TYPE),SEEK_SET);
		if(fread(rbuf1,sizeof(DATA_TYPE),npixel*nbands*nyear_avg,ifiles_fut[f])!=npixel*nbands*nyear_avg)
			fprintf(stderr,"Error reading from file ifiles_ref[%d]\n",f);
		timeseries_mean_bsq_float(rbuf1,npixel,nyear_avg,nbands,mean_fut,std_fut);
		
		/* timeseries of global mean and standard deviation*/
		fseek(ifiles_fut[f],0,SEEK_SET);
		fseek(ifiles_ref[f],0,SEEK_SET);
		rbuf1=realloc(rbuf1,sizeof(DATA_TYPE)*npixel*nbands);
		check_ptr(rbuf1);
		for(y=0;y<timelength;y++){
			if(timelength == nyear_ref) {
				/* assume reference output file contains the whole time period */
				if(fread(rbuf1,sizeof(DATA_TYPE),npixel*nbands,ifiles_ref[f])!=npixel*nbands)
					fprintf(stderr,"Error reading from file ifiles_ref[%d]\n",f);
			} else if(y < nyear_ref) {
				/* assume reference output file contains only the reference period */
				if(fread(rbuf1,sizeof(DATA_TYPE),npixel*nbands,ifiles_ref[f])!=npixel*nbands)
					fprintf(stderr,"Error reading from file ifiles_ref[%d]\n",f);
			} else {
				/* assume future output file contains only the future period, read after reference period output */
				if(fread(rbuf1,sizeof(DATA_TYPE),npixel*nbands,ifiles_fut[f])!=npixel*nbands)
					fprintf(stderr,"Error reading from file ifiles_fut[%d]\n",f);
			}
			global_mean_areaweighted_float(gridfile,rbuf1,npixel,nr,nc,1,nbands, &mean_global[y],&std_global[y], natvegfrac);
		}
		
		mean_global_ref=std_global_ref=0;
		for(y=startyear_ref;y<startyear_ref+nyear_avg;y++){ 
			mean_global_ref+=mean_global[y]/(double)nyear_avg;
			std_global_ref+=std_global[y]/(double)nyear_avg;
		}
		if(!is_range_spec) {
			printf("global_mean_ref %lf global_std_ref %lf\n",mean_global_ref,std_global_ref);
		} else {
			printf("supplied global_mean_ref %lf global_std_ref %lf\n",mean_spec[f],std_spec[f]);
		}
		
		/* calculate change metric components  */
		//calculate change between two points in time (local normalization)
		for(p=0;p<npixel;p++){
			// new: components with average
			absb[p]+=(weight);
			if(fabs(mean_ref[p])>EPSILON){
				diff[p]+= weight*(mean_fut[p]-mean_ref[p])/mean_ref[p]*(mean_fut[p]-mean_ref[p])/mean_ref[p];
				nom[p]+=(weight*mean_fut[p]/mean_ref[p]);	
				absa[p]+=(weight*mean_fut[p]*mean_fut[p]/mean_ref[p]/mean_ref[p]);
			}else if (fabs(mean_fut[p])>EPSILON){ /*huge change from nothing*/
				diff[p]+=(weight*(mean_fut[p]-mean_ref[p])*(mean_fut[p]-mean_ref[p]));
				nom[p]+=(weight*mean_fut[p]);
				absa[p]+=(weight*mean_fut[p]*mean_fut[p]);
			}else{ /* no change */
				diff[p]+= 0;
				nom[p]+=(weight);
				absa[p]+=(weight);
			}
		}/* endfor each pixel*/
		
		//calculate change between two points in time (global normalization)
		/* global normalization */
		if(is_range_spec)
			tmp=mean_spec[f];
		else
			tmp=mean_global_ref;
		//printf("global normalization with %lf\n",tmp);
		for(p=0;p<npixel;p++){
			diff_g[p]+=weight*(mean_fut[p]-mean_ref[p])/tmp*(mean_fut[p]-mean_ref[p])/tmp;
			nom_g[p]+=weight*mean_ref[p]/tmp*mean_fut[p]/tmp;
			absa_g[p]+=weight*mean_fut[p]/tmp*mean_fut[p]/tmp;
			absb_g[p]+=weight*mean_ref[p]/tmp*mean_ref[p]/tmp;
		}
		
		// calculate timeseries of alpha (with respect to normalized vector of reference period) and vector magnitude, local + global   
		i=fseek(ifiles_fut[f],0,SEEK_SET);
		i=fseek(ifiles_ref[f],0,SEEK_SET);
		i=errno;
		rbuf1=realloc(rbuf1,sizeof(DATA_TYPE)*npixel*nbands);
		check_ptr(rbuf1);
		for(y=0;y<timelength;y++){
			if(timelength == nyear_ref) {
				/* assume reference file contains the whole period */
				if(fread(rbuf1,sizeof(DATA_TYPE),npixel*nbands,ifiles_ref[f])!=npixel*nbands) {
					fprintf(stderr,"Error reading from file ifiles_fut[%d],error code %d,year %d\n",f,ferror(ifiles_fut[f]),y);
				}
			} else if(y < nyear_ref) {
				/* take reference file for reference period */
				if(fread(rbuf1,sizeof(DATA_TYPE),npixel*nbands,ifiles_ref[f])!=npixel*nbands) {
					fprintf(stderr,"Error reading from file ifiles_ref[%d],error code %d,year %d\n",f,ferror(ifiles_ref[f]),y);
				}
			} else {
				/* take future file for future period only */
				if(fread(rbuf1,sizeof(DATA_TYPE),npixel*nbands,ifiles_fut[f])!=npixel*nbands) {
					fprintf(stderr,"Error reading from file ifiles_fut[%d],error code %d,year %d\n",f,ferror(ifiles_fut[f]),y-nyear_ref);
				}
			}
			for(b=0;b<nbands;b++)
				for(p=0;p<npixel;p++){
					nom_ts_g[y*npixel+p]+=(weight*rbuf1[b*npixel+p]/tmp*mean_ref[p]/tmp);
					absa_ts_g[y*npixel+p]+=(weight*rbuf1[b*npixel+p]/tmp*rbuf1[b*npixel+p]/tmp); /* future */
					absb_ts_g[y*npixel+p]+=(weight*mean_ref[p]/tmp*mean_ref[p]/tmp);             /* reference */
					/* change metric components */	
					if(fabs(mean_ref[p])>EPSILON){
						nom_ts[y*npixel+p]+=(weight*rbuf1[b*npixel+p]/mean_ref[p]);
						absa_ts[y*npixel+p]+=(weight*rbuf1[b*npixel+p]*rbuf1[b*npixel+p]/mean_ref[p]/mean_ref[p]);
					}
					else{
						nom_ts[y*npixel+p]+=(weight*rbuf1[b*npixel+p]);   
						absa_ts[y*npixel+p]+=(weight*rbuf1[b*npixel+p]*rbuf1[b*npixel+p]);
					}
					absb_ts[y*npixel+p]+=(weight); /* assume a vector {1,1,1,..}*/
				} /* end for each pixel */
		}/*endfor each year*/
	}/*endfor each input parameter*/



	/* write output */
	/* angle, local normalization */
	for (p=0; p<npixel; ++p){
		if (absa[p]>EPSILON && absb[p]>EPSILON){
			wbuf[p]=(DATA_TYPE_OUT)nom[p]/sqrt(absa[p])/sqrt(absb[p]);
			if(wbuf[p]>1)
				printf("p %d nom %lf absa %lf absb %lf\n",p,nom[p],absa[p],absb[p]);
		}
		else if (fabs(absa[p])<=EPSILON && fabs(absb[p])<=EPSILON)
			wbuf[p]=1; /*no change*/
		else
			wbuf[p]=0; /*change from/to nothing is severe*/
	}
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile);

	/*angle, global normalization*/
	for (p=0; p<npixel; ++p){
		if (absa_g[p]>EPSILON && absb_g[p]>EPSILON)
			wbuf[p]=(DATA_TYPE_OUT)nom_g[p]/sqrt(absa_g[p])/sqrt(absb_g[p]);
		else if (absa_g[p]<=EPSILON && absb_g[p]<=EPSILON)
			wbuf[p]=1; /*no change*/
		else
			wbuf[p]=0; /*change from/to nothing is severe*/
	}
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile5);

	/* magnitude of change */
	for (p=0; p<npixel; ++p){
		if(absa[p]>0){
			tmp=absa[p];
			absa[p]=sqrt(tmp); /*changed period*/
		}
		tmp=absb[p];
		absb[p]=sqrt(tmp); /*reference period*/
		if(absb[p]<EPSILON || absa[p]<0)
			wbuf[p]=-1;
		else{
			/*magnitude in % of reference period*/
			wbuf[p]=(DATA_TYPE_OUT)absa[p]/absb[p]*100;
		}
	}
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile2);

	/*magnitude of difference vector*/
	for (p=0; p<npixel; ++p) {
		wbuf[p]=diff[p]>=0?(DATA_TYPE_OUT)sqrt(diff[p]):INFCHANGE;
	}
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile2);
	
	for (p=0; p<npixel; ++p)
		wbuf[p]=(DATA_TYPE_OUT)(absa[p]);
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile2);
	for (p=0; p<npixel; ++p) 
		wbuf[p]=(DATA_TYPE_OUT)(absb[p]);
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile2);

	/* magnitude of change, global normalization */
	for (p=0; p<npixel; ++p){	
		tmp=absa_g[p];
		absa_g[p]=sqrt(tmp); /*changed period*/
		tmp=absb_g[p];
		absb_g[p]=sqrt(tmp); /*reference period*/
		/*magnitude in % of reference period*/
		wbuf[p]=absb_g[p]>EPSILON?(DATA_TYPE_OUT)absa_g[p]/absb_g[p]*100 : -1;
	}
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile6);
	
	/*magnitude as difference */
	for (p=0; p<npixel; ++p) 
		wbuf[p]=sqrt(diff_g[p]);
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile6);
	for (p=0; p<npixel; ++p) 
		wbuf[p]=(DATA_TYPE_OUT)(absa_g[p]);
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile6);
	for (p=0; p<npixel; ++p) 
		wbuf[p]=(DATA_TYPE_OUT)(absb_g[p]);
	fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile6);

	/* time series of angle*/
	for(y=0;y<timelength;y++){
		for (p=0; p<npixel; ++p){
			if (absa_ts[y*npixel+p]>EPSILON && absb_ts[y*npixel+p]>EPSILON)
				wbuf[p]=(DATA_TYPE_OUT)nom_ts[y*npixel+p]/sqrt(absa_ts[y*npixel+p])/sqrt(absb_ts[y*npixel+p]);
			else if (absa_ts[y*npixel+p]<=EPSILON && absb_ts[y*npixel+p]<=EPSILON)
				wbuf[p]=1; /*no change*/
			else
				wbuf[p]=0; /*change from/to nothing is severe*/
		}
		fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile3);
	}

	/*timeseries of magnitude*/
	for(y=0;y<timelength;y++){
		for (p=0; p<npixel; ++p){
			if(absa_ts[y*npixel+p]>=0){
				tmp=absa_ts[y*npixel+p];
				absa_ts[y*npixel+p]=sqrt(tmp); 
				wbuf[p]=(DATA_TYPE_OUT)(absa_ts[y*npixel+p]);
			}else
				wbuf[p]=INFCHANGE;
		}
		fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile4);
	}
	/* time series of angle, global*/
	for(y=0;y<timelength;y++){
		for (p=0; p<npixel; ++p){
			if (absa_ts_g[y*npixel+p]>EPSILON && absb_ts_g[y*npixel+p]>EPSILON)
				wbuf[p]=(DATA_TYPE_OUT)nom_ts_g[y*npixel+p]/sqrt(absa_ts_g[y*npixel+p])/sqrt(absb_ts_g[y*npixel+p]);
			else if (absa_ts_g[y*npixel+p]<=EPSILON && absb_ts_g[y*npixel+p]<=EPSILON)
				wbuf[p]=1; /*no change*/
			else
				wbuf[p]=0; /*change from/to nothing is severe*/
		}
		fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile7);
	}

	/*timeseries of magnitude*/
	for(y=0;y<timelength;y++){
		for (p=0; p<npixel; ++p){	
			tmp=absa_ts_g[y*npixel+p];
			absa_ts_g[y*npixel+p]=sqrt(tmp); 
			wbuf[p]=(DATA_TYPE_OUT)(absa_ts_g[y*npixel+p]);
		}
		fwrite(wbuf,npixel,sizeof(DATA_TYPE_OUT),ofile8);
	}

	free(weights);
	if(mean_spec)
	free(mean_spec);
	if(std_spec)
	free(std_spec);
	free(rbuf1);
	free(rbuf2);
	free(wbuf);
	free(nom);
	free(absa);
	free(absb);
	free(nom_g);
	free(absa_g);
	free(absb_g);
	free(nom_ts);
	free(absa_ts);
	free(absb_ts);
	free(nom_ts_g);
	free(absa_ts_g);
	free(absb_ts_g);
	free(mean_ref);
	free(std_ref);
	free(mean_global);
	free(std_global);
	for (j=0; j<npar; j++){
		fclose(ifiles_ref[j]);
		fclose(ifiles_fut[j]);
	}
	free(ifiles_ref);
	free(ifiles_fut);
	if(is_natveg_spec)
		fclose(natvegfile);
	free(natvegfrac);
	fclose(ofile);
	fclose(ofile2);
	fclose(ofile3);
	fclose(ofile4);
	fclose(ofile5);
	fclose(ofile6);
	return 0;
	}




