#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define DATA_TYPE float
#define NPFT 10
#define NLF 3
#define BSQ

void usage(char* progname);


int main(int argc,char* argv[])
{  
	FILE *ffpc1, *ffpc2;
	FILE *ofp;
	float lifeform[NPFT][NLF];	/* pft, life form */
	float fpc1[NPFT];	/* pft */
	float fpc2[NPFT];	/* pft */
	DATA_TYPE *rbuf1;	
	DATA_TYPE *rbuf2;	
	float v1,v2,totv,deltav;
	int k,pft,startyear1,startyear2;
	float weight[NLF][4]; 	/* life form, attribute type */
	float attribute[NPFT][4];	/* pft, attribute type */
	int ia;
	float a,a1,a2;
	int iamax[3];		/* life form */
	float veg1,veg2;
	float outval;
	long int npft, npixel,nyear_file1, nyear_file2,p,y;
	char *ifilename1, *ifilename2, *ofilename;
	int i,issim;

	/* SETTINGS */
	npft=NPFT;
	issim=0;

	if (argc<9)
		usage(argv[0]);
	ifilename1 = argv[1];
	ifilename2 = argv[2];
	ofilename = argv[3];
	nyear_file1 = atoi(argv[4]);
	nyear_file2 = atoi(argv[5]);
	npixel = atoi(argv[6]);
	startyear1 = atoi(argv[7]);
	startyear2 = atoi(argv[8]);
	if (argc>9)
	issim=1; 
	rbuf1=(DATA_TYPE*)malloc(npft*npixel*nyear_file1*sizeof(DATA_TYPE));
	rbuf2=(DATA_TYPE*)malloc(npft*npixel*nyear_file2*sizeof(DATA_TYPE));
	if(!rbuf1||!rbuf2){
		fprintf(stderr, "Error in memory allocation\n");
		exit(-1);
	}

	/* FILE OPENING */
	if(!(ffpc1=fopen(ifilename1,"rb"))){
		fprintf(stderr,"Warning: File open failed on input file1\n");
	}
	if(!(ffpc2=fopen(ifilename2,"rb"))){
		fprintf(stderr,"Warning: File open failed on input file2\n");
	}
	if(!(ofp=fopen(ofilename,"wb"))){
		fprintf(stderr,"Warning: File open failed on output file\n");
	}




	/* current PFTs
							 Woodiness     Herbaceousness	 Barrenness
	0 total      0             0               1
	1 TR BDL EVG 1             0               0
	2 TR BDL RGR 1             0               0
	3 TE NDL EVG 1             0               0
	4 TE BDL EVG 1             0               0
	5 TE BDL SUG 1             0               0
	6 BO NDL EVG 1             0               0
	7 BO BDL SUG 1             0               0
	8 C3 GRASS   0             1               0
	9 C4 GRASS   0             1               0
	*/

	/* new: 		woodiness of each PFT*/
	lifeform[0][0]=0;
	lifeform[1][0]=1;
	lifeform[2][0]=1;
	lifeform[3][0]=1;
	lifeform[4][0]=1;
	lifeform[5][0]=1;
	lifeform[6][0]=1;
	lifeform[7][0]=1;
	lifeform[8][0]=0;
	lifeform[9][0]=0;

	/* new: 		Herbaceousness of each PFT */ 
	lifeform[0][1]=0;
	lifeform[1][1]=0;
	lifeform[2][1]=0;
	lifeform[3][1]=0;
	lifeform[4][1]=0;
	lifeform[5][1]=0;
	lifeform[6][1]=0;
	lifeform[7][1]=0;
	lifeform[8][1]=1;
	lifeform[9][1]=1;

	/* new: 		Barrenness of each life form */
	lifeform[0][2]=1;
	lifeform[1][2]=0;
	lifeform[2][2]=0;
	lifeform[3][2]=0;
	lifeform[4][2]=0;
	lifeform[5][2]=0;
	lifeform[6][2]=0;
	lifeform[7][2]=0;
	lifeform[8][2]=0;
	lifeform[9][2]=0;

	/* 
	Attributes are:
	FOR WOODY
	0	Needleleavedness	
	1	Evergreenness		
	2	Tropical
	3 Boreal		
	--> iamax=4

	FOR HERBACEOUS
	0 warm
	--> iamax=1

	FOR BARREN
	none
	--> iamax=0

	*/

	/*
	Weights for each life form:
	LIFE FORM: WOODY
	Needleleavedness 	 0.2	
	Evergreenness		 	 0.2
	Tropical		   		 0.3
	Boreal						 0.3
	
	LIFE FORM: HERBACIOUS
	Warm							 0.3

	NOTE: should all add up to 1 within each life form
	NOTE2: differentiation between C3/C4 grass weighted less important
	*/

	iamax[0]=4; /* no of attributes for woodiness */
	iamax[1]=1; /* no of attributes for herbaciousness */
	iamax[2]=0; /* no of attributes for barrenness */

	/* Need only list those with iamax>0 */
	/*     ^ only weights for attributes of woodiness */
	weight[0][0]=0.2; /* needleleavedness */
	weight[0][1]=0.2; /* evergreenness */
	weight[0][2]=0.3; /* tropical */
	weight[0][3]=0.3; /* boreal */
	/*     ^ only weights for attributes of herbaciousness */
	weight[1][0]=0.3; /* warm for grasses, comparable to shift tropical -> temperate or temperate -> boreal */

	/* Need only list those with iamax>0 */
	/* Needleleavedness for trees, warm for grasses */
	attribute[0][0]=0;
	attribute[1][0]=0;
	attribute[2][0]=0;
	attribute[3][0]=1;
	attribute[4][0]=0;
	attribute[5][0]=0;
	attribute[6][0]=1;
	attribute[7][0]=0.25;
	attribute[8][0]=0;
	attribute[9][0]=1;

	/*Evergreenness for trees */
	attribute[0][1]=0;
	attribute[1][1]=1;
	attribute[2][1]=0;
	attribute[3][1]=1;
	attribute[4][1]=1;
	attribute[5][1]=0;
	attribute[6][1]=1;
	attribute[7][1]=0;
	attribute[8][1]=0;
	attribute[9][1]=0;

	/*Tropical for trees */
	attribute[0][2]=0;
	attribute[1][2]=1;
	attribute[2][2]=1;
	attribute[3][2]=0;
	attribute[4][2]=0;
	attribute[5][2]=0;
	attribute[6][2]=0;
	attribute[7][2]=0;
	attribute[8][2]=0;
	attribute[9][2]=0;
	
	/* Boreal for trees */
	attribute[0][3]=0;
	attribute[1][3]=0;
	attribute[2][3]=0;
	attribute[3][3]=0;
	attribute[4][3]=0;
	attribute[5][3]=0;
	attribute[6][3]=1;
	attribute[7][3]=1;
	attribute[8][3]=0;
	attribute[9][3]=0;

	/* Read pixels */
	fseek(ffpc1, sizeof(DATA_TYPE)*startyear1*npft*npixel,SEEK_SET);
	if(fread(rbuf1, sizeof(DATA_TYPE),npft*npixel*nyear_file1,ffpc1) != npft*npixel*nyear_file1) {
		fprintf(stderr, "Error reading data from input file 1\n");
		exit(-1);
	}
	if(startyear2 >= 0) {
		fseek(ffpc2, sizeof(DATA_TYPE)*startyear2*npft*npixel,SEEK_SET);
		if(fread(rbuf2, sizeof(DATA_TYPE),npft*npixel*nyear_file2,ffpc2) != npft*npixel*nyear_file2) {
			fprintf(stderr, "Error reading data from input file 2\n");
			exit(-1);
		}
	} else {
		if(fseek(ffpc1, sizeof(DATA_TYPE)*startyear2*npft*npixel,SEEK_END)) { /* search -startyear2 years from end of input file 1 */
			fprintf(stderr, "Error: cannot go to position %d years before end of input file 1\n", startyear2*(-1));
			exit(-1);
		}
		if(fread(rbuf2, sizeof(DATA_TYPE),npft*npixel*nyear_file2,ffpc1) != npft*npixel*nyear_file2) {
			fprintf(stderr, "Error reading data from end of input file 1\n");
			exit(-1);
		}
	}
	for (p=0;p<npixel;++p){
		/* mean over nyear years */
		for(i=1;i<npft;++i){
			fpc1[i]=0;
			fpc2[i]=0;
			for(y=0;y<nyear_file1;++y){
				fpc1[i]+=rbuf1[y*npixel*npft+i*npixel+p]/(float)nyear_file1;
			}
			for(y=0;y<nyear_file2;++y){
				fpc2[i]+=rbuf2[y*npixel*npft+i*npixel+p]/(float)nyear_file2;
			}
		}

		/* calculate fraction of bare ground*/
		if(rbuf1[(nyear_file1-1)*npixel*npft+p]>0.001){ /* fraction of natural vegetation in the last year*/
			veg1 = veg2 = 0.0;
			for (pft=1; pft<npft; ++pft){
				veg1+=(float)fpc1[pft];
				veg2+=(float)fpc2[pft];
			}
			fpc1[0]=(float)(1-veg1); /* bare ground */
			fpc2[0]=(float)(1-veg2);

			/* calculate importance values of each life form*/
			totv=0.;
			for(k=0;k<NLF;k++){ /* Life forms -> woodiness, herbaciousness, barrenness */
				v1=0.; /* area fraction of lifeform k over all PFTs */
				v2=0.;
				for(pft=0;pft<npft;pft++){
					v1+=fpc1[pft]*lifeform[pft][k];
					v2+=fpc2[pft]*lifeform[pft][k];
				} /* end loop over pfts */

				/* calculate attribute weights for each lifeform*/
				a=0.;
				for(ia=0;ia<iamax[k];ia++){ /* attributes */
					/* currently only lifeform woodiness has attributes */
					a1=0.;
					a2=0.;
					for(pft=0;pft<npft;pft++){ /* pft */
						a1+=attribute[pft][ia]*fpc1[pft]*lifeform[pft][k];
						a2+=attribute[pft][ia]*fpc2[pft]*lifeform[pft][k];
					} /* end loop over pfts */

					a+=weight[k][ia]*fabs(a1-a2);
				} /* end loop over attributes */

				totv+=MIN(v1,v2)*(1.-a);
				/* Note: totv is a similarity index, 1=similar, 0=dissimilar */
			} /* end loop over life forms */
			/*  DISSIMILARITY DEFINITION */ 
			deltav=1.-totv;
		}
		else {
			deltav=0;
			totv=1;
		}
		if (!issim)
			outval=(float)(deltav);
		else
			outval=(float)(totv);
		fwrite(&outval,sizeof(float),1,ofp);

	}/*endfor: loop over pixel*/
	free(rbuf1); free(rbuf2);
	return 0;
}


void usage(char* progname){
fprintf(stderr, "Use: %s fpcfile1 fpcfile2 outfile nyear_file1 nyear_file2 npixel startyear1 startyear2 [sim(to calculate similarity measure)]\n", progname);
fprintf(stderr, "Notice: if startyear2 < 0, data is read from input file 1 instead of input file 2 assuming that both files are continuous.\n");
exit(-1);
}
