#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <math.h>

#define BSQ
#define DATA_TYPE float
#define DATA_TYPE_OUT float
#define STARTBAND 0 
#define NAN 9999
#define CORRECT_LARGE_VAL
#define ROUND(x) ((DATA_TYPE_OUT)((x)+((x)<0?-0.5:0.5))-(DATA_TYPE_OUT)((x)<0?-0.5:0.5))
#define check_ptr(ptr) if((ptr)==NULL) failu("Error allocating memory\n")
#define check_file(file) if((file)==NULL) failu("Error opening file\n")

void failu(const char* str){
  fprintf(stderr, "%s\n", str);
  exit(-1);
}

void usage(char* progname){
  fprintf(stderr, "Use:\n%s ifile1 ifile2 ofile npixel [startband1(default:0) endband1(default:all) startband2(0) endband2(all) nanvalue(%d)]\nCalculates file1/file2\nreturns nanvalue if file2=0\na nanvalue of -1 means to take the value of file1\n", progname,NAN);
  exit(-1);
}

int main(int argc, char* argv[]){
  int i,p,y,b=0, npixel,nbands,nbands1, nbands2,startband1, startband2, endband1, endband2;
  DATA_TYPE *rbuf1,*rbuf2; 
  DATA_TYPE_OUT *wbuf;
  char* ifilename1, *ifilename2, *ofilename;
  FILE* ifile1, *ifile2, *ofile;
  struct stat filestat;
  double tmp,nan = NAN;

  /* Parse command line */
  if (argc < 5) usage(argv[0]);
  ifilename1 = argv[1];
  ifilename2 = argv[2];
  ofilename = argv[3];
  npixel = atoi(argv[4]);
  
  /* Open files */
  ifile1 = fopen64(ifilename1, "rb");
  check_file(ifile1);
  ifile2 = fopen64(ifilename2, "rb");
  check_file(ifile2);
  ofile = fopen64(ofilename, "wb");
  check_file(ofile);
  
  /* file statistics */
  fstat(fileno(ifile1),&filestat);
  nbands1=filestat.st_size/npixel/sizeof(DATA_TYPE);
  fstat(fileno(ifile2),&filestat);
  nbands2=filestat.st_size/npixel/sizeof(DATA_TYPE);
  startband1=startband2=STARTBAND;
  endband1=nbands1-1;
  endband2=nbands2-1;
  
  if (argc > 8){
    startband1 = atoi(argv[5]);
    endband1 = atoi(argv[6]);
    startband2 = atoi(argv[7]);
    endband2=atoi(argv[8]);
  }
  if (argc > 9)
    nan=atof(argv[9]);
  printf("%d %d %d %d %f\n",startband1,endband1,startband2,endband2,nan);
  nbands=endband1-startband1+1;
  if(nbands<1||(endband1-startband1)!=(endband2-startband2)){
    fprintf(stderr, "different number of bands for differencing in file1 and file2! Exit.\n");
    exit(-1);
  }
  
#ifdef BSQ
  /*memory allocation*/
  rbuf1=malloc(npixel*sizeof(DATA_TYPE));
  check_ptr(rbuf1);
  rbuf2=malloc(npixel*sizeof(DATA_TYPE));
  check_ptr(rbuf2);
  wbuf = malloc(nbands*npixel*sizeof(DATA_TYPE_OUT));
  check_ptr(wbuf);
  fseeko64(ifile1,sizeof(DATA_TYPE)*npixel*(startband1+b),SEEK_SET);
  fseeko64(ifile2,sizeof(DATA_TYPE)*npixel*(startband2+b),SEEK_SET);
  for(b=0; b<nbands; ++b){
    if(fread(rbuf1, sizeof(DATA_TYPE),npixel,ifile1)!=npixel){
      fprintf(stderr,"error reading from file1\n");
      exit(-1);
    };
    if(fread(rbuf2, sizeof(DATA_TYPE),npixel,ifile2)!=npixel){
      fprintf(stderr,"Error reading from file2\n");
      exit(-1);
    }
    for(p=0; p<npixel;++p)
      if(fabs(rbuf2[p])<0.00001){
        if(fabs(nan+1)<0.00001){
            tmp=rbuf2[p];
            tmp=rbuf1[p];
            wbuf[b*npixel+p]=ROUND(rbuf1[p]);
            tmp=wbuf[b*npixel+p];
        }
        else{
            wbuf[b*npixel+p]=(DATA_TYPE_OUT)nan;
        }
      }
      else{
        wbuf[b*npixel+p]=ROUND(rbuf1[p]/(double)rbuf2[p]);   
#ifdef CORRECT_LARGE_VAL
        if (wbuf[b*npixel+p] > 1000){
            tmp=rbuf2[p];
            tmp=rbuf1[p];
            fprintf(stderr, "rbuf1/rbuf2 = %.2f; value is set to 1000; band %d pixel %d\n",wbuf[b*npixel+p],b,p);
            wbuf[b*npixel+p]=1000;
        }
#endif
      }
  }
  fwrite(wbuf,sizeof(DATA_TYPE_OUT),nbands*npixel,ofile);
#endif
  free(rbuf1);
  free(rbuf2);
  free(wbuf);
  fclose(ifile1);
  fclose(ifile2);
  fclose(ofile);
  return 0;
}
