#include <stdio.h>
#include <stdlib.h>
#include "utilfunc.h"
#include "math.h"

#define BSQ
#define DATA_TYPE float
#define DATA_TYPE_OUT float
#define ROUND(x) ((DATA_TYPE_OUT)((x)+((x)<0?-0.5:0.5))-(DATA_TYPE_OUT)((x)<0?-0.5:0.5))

void usage(char* progname){
  fprintf(stderr, "Use:\n%s ifile ofile npixel startyear endyear nbandsyear [-gm][-sd] \ncalculates monthly means/standard deviation(sd), annual mean (gm)\n", progname);
  exit(-1);
}

int main(int argc, char* argv[]){
  int i,p,y,b, npixel,nbands_year,startyear,endyear,nyear,gm,sd=0;
  DATA_TYPE *rbuf; 
  DATA_TYPE_OUT *wbuf;
  char* ifilename, *ofilename;
  FILE* ifile, *ofile;
  double tmp;
  gm=0; sd=0;

  /* Parse command line */
  if (argc < 7) usage(argv[0]);
  ifilename = argv[1];
  ofilename = argv[2];
  npixel = atoi(argv[3]);
  startyear = atoi(argv[4]);
  endyear = atoi(argv[5]);
  nbands_year = atoi(argv[6]);
  i=7;
  while(i<argc){
    if(!(strcmp("-gm",argv[i])))
        gm=1;
    else if(!(strcmp("-sd",argv[i])))
        sd=1;
    ++i;
  }
  nyear=endyear-startyear+1;
  printf("%d %d %d\n",startyear,endyear,nyear);
  if(nyear<1){
    fprintf(stderr, "endyear < startyear! Exit.\n");
    exit(-1);
  }
  
  /* Open files */
  ifile = fopen64(ifilename, "rb");
  check_file(ifile);
  ofile = fopen64(ofilename, "wb");
  check_file(ofile);

#ifdef BSQ
  /*memory allocation*/
  rbuf = malloc(nbands_year*npixel*sizeof(DATA_TYPE));
  wbuf = malloc((nbands_year+gm+sd*nbands_year)*npixel*sizeof(DATA_TYPE_OUT));
  for(p=0; p<(nbands_year+gm+sd*nbands_year)*npixel;++p)
    wbuf[p]=0.0;
  
  for(y=startyear; y<=endyear; ++y){
    fseeko64(ifile,y*nbands_year*npixel*sizeof(DATA_TYPE),SEEK_SET);
    /*printf("Year: %d\n",y);*/
    if (fread(rbuf, sizeof(DATA_TYPE),nbands_year*npixel,ifile)!=nbands_year*npixel){
	fprintf(stderr, "Error reading from file\n");
	exit(-1);
    }
    for(b=0; b<nbands_year; ++b)
      for(p=0; p<npixel;++p){
	wbuf[b*npixel+sd*b*npixel+p]+=ROUND(rbuf[b*npixel+p]);
        tmp = wbuf[b*npixel+sd*b*npixel+p];
      }
  }
  for(b=0; b<nbands_year; ++b)
      for(p=0; p<npixel;++p){
        tmp = wbuf[b*npixel+sd*b*npixel+p];
	wbuf[b*npixel+sd*b*npixel+p]=ROUND(((double)wbuf[b*npixel+sd*b*npixel+p]/nyear));//(DATA_TYPE_OUT)((double)wbuf[b*npixel+sd*b*npixel+p]/nyear+0.5)-(DATA_TYPE_OUT)0.5; 
      }
  /*calculate global mean*/
  if(gm){
    for(b=0; b<nbands_year;++b)
      for(p=0;p<npixel;++p)
	wbuf[nbands_year*npixel+sd*nbands_year*npixel+p]+=ROUND(wbuf[b*npixel+sd*b*npixel+p]/nbands_year);  
  }
  /*calculate standard deviation */
  if(sd){
    for(y=startyear; y<=endyear; ++y){
        fseeko64(ifile,y*nbands_year*npixel*sizeof(DATA_TYPE),SEEK_SET);
        /*printf("Year: %d\n",y);*/
        if (fread(rbuf, sizeof(DATA_TYPE),nbands_year*npixel,ifile)!=nbands_year*npixel){
	   fprintf(stderr, "Error reading from file\n");
	   exit(-1);
        }
        for(b=0; b<nbands_year; ++b)
            for(p=0; p<npixel;++p){
               tmp=rbuf[b*npixel+p];
               tmp=wbuf[b*npixel+sd*b*npixel+sd*npixel+p];
               tmp=wbuf[b*npixel+sd*b*npixel+p];
               tmp=(rbuf[b*npixel+p]-wbuf[b*npixel+sd*b*npixel+p]);
	       wbuf[b*npixel+sd*b*npixel+sd*npixel+p]+=ROUND(((rbuf[b*npixel+p]-wbuf[b*npixel+sd*b*npixel+p])*(rbuf[b*npixel+p]-wbuf[b*npixel+sd*b*npixel+p])));
               tmp = wbuf[b*npixel+sd*b*npixel+sd*npixel+p];
            }
    }
    for(b=0; b<nbands_year; ++b)
      for(p=0; p<npixel;++p){
	wbuf[b*npixel+sd*b*npixel+sd*npixel+p]=ROUND((sqrt((double)wbuf[b*npixel+sd*b*npixel+sd*npixel+p]/nyear)));
        tmp=wbuf[b*npixel+sd*b*npixel+sd*npixel+p];
      }
  }
  fwrite(wbuf, sizeof(DATA_TYPE_OUT), (nbands_year+sd*nbands_year+gm)*npixel, ofile);
#endif
  free(rbuf);
  free(wbuf);
  fclose(ifile);
  fclose(ofile);
  return 0;
}
