#include <stdio.h>
#include "utilfunc.h"

unsigned long int getfilesize(FILE* file){
	unsigned long int size, pos;
	pos = ftello64(file);
	fseeko64(file, 0, SEEK_END);
	size = ftello64(file);
	fseeko64(file, pos, SEEK_SET);
	return size;
}

void failu(const char* str){
	fprintf(stderr, "%s\n", str);
	exit(-1);
}

double pixelarea(double lon_e, double lon_w, double lat_n, double lat_s){
	return (double)EARTH_RADIUS_M*EARTH_RADIUS_M*(deg2rad(lon_e)-deg2rad(lon_w))*(sin(deg2rad(lat_n))-sin(deg2rad(lat_s)));
}

/*calculates mean and standard deviation of a timeseries in BSQ ordered data*/
void timeseries_mean_bsq_float(float* data, long npixel,long nyear,long nbands,double* mean,double* std){
	long y,b,p;
	double tmp;
	for(p=0;p<npixel;p++)
		mean[p]=std[p]=0.0;
	/*mean*/
	for(y=0;y<nyear;y++)
		for(b=0;b<nbands;b++)
			for(p=0;p<npixel;p++)
				mean[p]+=data[y*nbands*npixel+b*npixel+p]/(double)(nyear*nbands);
	/*standard deviation*/
	if(std!=NULL){
		for(y=0;y<nyear;y++)
			for(b=0;b<nbands;b++)
				for(p=0;p<npixel;p++)
					std[p]+=(data[y*nbands*npixel+b*npixel+p]-mean[p])*(data[y*nbands*npixel+b*npixel+p]-mean[p])/(double)(nyear*nbands);
		for(p=0;p<npixel;p++){
			tmp=sqrt(std[p]);
			std[p]=tmp;
		}
	}
	return;
}

/* calculates area weighted global mean for each band */
void global_mean_areaweighted_float(FILE* gridfile,float* data, long npix, long nr, long nc, long nyear, long nbands, double* mean,double* std){
	double *lat_weight,lat,lon,wsum,Tsum,Tsd,tmp;
	long *grid,ip,r,b,c,y;
	short rbuf;
	int ilon,ilat;
	float lat_res,start_lat;

	lat_weight = (double*)malloc(sizeof(double)*nr);
	grid = (long*)malloc(sizeof(long)*nr*nc);
	for(ip=0;ip<nr*nc;ip++)
		grid[ip]=-1;
	lat_res = 180./nr;
	start_lat = -90 + lat_res/2;

	/* read grid */
	fseek(gridfile,0,SEEK_SET);
	for(ip=0;ip<npix;ip++){
		fread(&rbuf,sizeof(short),1,gridfile);
		lon=rbuf/100.;
		fread(&rbuf,sizeof(short),1,gridfile);
		lat=rbuf/100.;

		if (lon <0.0) ilon= (int)((lon+360.)/lat_res+0.01);
		else ilon=(int)((lon)/lat_res+0.01);
		/*ilon=(int)((lon+180.)/res+0.01);*/
		/*ilat=(int)((lat+90.)/lat_res+0.01);*/
		ilat=180/lat_res-(int)((lat+90.)/lat_res+0.01);
		grid[ilat*nc+ilon]=ip;
	}
	/* calculate weights for latitudes*/
	wsum = 0;
	for (r=0; r<nr; ++r){
		lat = start_lat + r*lat_res;
		lat_weight[r] = (double)cos(lat/180.*M_PI);
		wsum += lat_weight[r];     
	}
	/* calculated weighted sum for each year*/
	for(y=0;y<nyear;y++){
		Tsum=wsum=Tsd=0.0;
		for(b=0;b<nbands;b++)
			for (r=0; r<nr; ++r)
				for (c=0; c<nc; ++c)
					if (grid[r*nc+c]>-1){
						Tsum += data[y*nbands*npix+b*npix+grid[r*nc+c]]*lat_weight[r];
						wsum += lat_weight[r];
					}
		mean[y]=Tsum/wsum;
		for(b=0;b<nbands;b++)
			for (r=0; r<nr; ++r)
				for (c=0; c<nc; ++c)
					if (grid[r*nc+c]>-1){
						Tsd += (data[y*nbands*npix+b*npix+grid[r*nc+c]]-mean[y])*(data[y*nbands*npix+b*npix+grid[r*nc+c]]-mean[y])*lat_weight[r];
					}
		tmp=sqrt(Tsd/wsum);
		std[y]=tmp;    
	}
	return;
}





